@@ -1,4 +1,4 @@
 #include <sys/wait.h>
 
-INHERENT_fork(){ return 1; }
+int INHERENT_fork(){ return 1; }
 int WAIT_WNOHANG = WNOHANG;
