@@ -19,7 +19,10 @@ History:
 //////////////////////////////////////////////////////////////////////#*/
 
 #include "mime.h"
-char *MimeKit_Version = "1.8";
+static int encodeMIMEpart(const char *boundaries[],FILE*fc,FILE*ts,FILE*cache,int filter,int _);
+static int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filter,int enHTML);
+
+const char *MimeKit_Version = "1.8";
 
 #define FN_CTE		"Content-Transfer-Encoding"
 #define FN_CTE_C	"Content-Transfer-Encoding:"
@@ -28,6 +31,7 @@ char *MimeKit_Version = "1.8";
 #define C_HEAD_EW	0x0001
 #define C_HEAD_CHAR	0x0002
 #define C_HEAD_JREPAIR	0x0004
+#define C_HEAD_SPENC	0x0008 /* space encoding in encoded-word */
 
 #define C_BODY		0xFF00
 #define C_BODY_CTE	0x0100
@@ -51,6 +55,7 @@ int MIME_CONV = MIMECONV;
 static int maybe_MIME;
 static int maybe_NONASCII;
 static int got_EOR;
+static int addQY;
 
 #define DEBUG	syslog_DEBUG
 
@@ -79,25 +84,22 @@ static int got_EOR;
 
 #define C_TEXT		"text/"
 #define C_TEXT_PLAIN	"text/plain"
-static is_text_plain(ctype)
-	char *ctype;
+static int is_text_plain(PCStr(ctype))
 {
 	return strncasecmp(ctype,C_TEXT_PLAIN,strlen(C_TEXT_PLAIN))==0;
 }
-static is_text(ctype)
-	char *ctype;
+static int is_text(PCStr(ctype))
 {
 	return strncasecmp(ctype,C_TEXT,strlen(C_TEXT))==0;
 }
 
-static char *textorso[] = {
+static const char *textorso[] = {
 	"application/x-unknown-content-type",
 	0
 };
-static tobe_charconv(ctype)
-	char *ctype;
+static int tobe_charconv(PCStr(ctype))
 {	int ci;
-	char *ctype1;
+	const char *ctype1;
 
 	if( is_text(ctype) )
 		return 1;
@@ -106,8 +108,7 @@ static tobe_charconv(ctype)
 			return 1;
 	return 0;
 }
-static is8bitCharset(charset)
-	char *charset;
+static int is8bitCharset(PCStr(charset))
 {
 	if( strcaseeq(charset,"x-sjis") )
 		return 1;
@@ -117,77 +118,87 @@ static is8bitCharset(charset)
 		return 1;
 	if( strcaseeq(charset,"EUC-JP") )
 		return 1;
+	if( strcaseeq(charset,"utf-8") )
+		return 1;
 	return 0;
 }
 
-static conv1(spec)
-	char *spec;
+static scanListFunc conv1(PCStr(spec))
 {
 	if( streq(spec,"thru")) MIME_CONV = 0; else
 	if( streq(spec,"all" )) MIME_CONV = C_ALL; else
 	if( streq(spec,"enc" )) MIME_CONV |= C_ENCODE(C_HEAD|C_BODY); else 
 	if( streq(spec,"dec" )) MIME_CONV |= C_DECODE(C_HEAD|C_BODY); else 
+	if( streq(spec,"qy") )	addQY = 1; else
+	if( streq(spec,"nospenc") )
+				MIME_CONV &= ~C_ENCODE(C_HEAD_SPENC); else
 	if( streq(spec,"charcode") )
 				MIME_CONV = C_DECODE(C_CHAR)|C_ENCODE(C_CHAR);
 	else	syslog_ERROR("#### ERROR: unknown MIMECONV=%s\n",spec);
 	return 0;
 }
-scan_MIMECONV(convspec)
-	char *convspec;
+extern int MIME_SPACE_ENCODING;
+void scan_MIMECONV(PCStr(convspec))
 {
 	if( *convspec == 0 )
 		MIME_CONV = C_ENCODE(C_HEAD|C_BODY) | C_DECODE(C_HEAD|C_BODY);
-	else	scan_commaList(convspec,0,conv1);
+	else	scan_commaList(convspec,0,scanListCall conv1);
+	if( (MIME_CONV & C_ENCODE(C_HEAD_SPENC)) == 0 )
+		MIME_SPACE_ENCODING = 0;
 }
 
-static char *fgetsTee(buf,size,in,out,cache)
-	char *buf;
-	FILE *in,*out,*cache;
-{	char *rcode;
+const char *MIME_transAddrSpec;
+static char *fgetsTee(PVStr(buf),int size,FILE *in,FILE *out,FILE *cache)
+{	const char *rcode;
 
-	buf[0] = 0;
+	setVStrEnd(buf,0);
 	if( rcode = fgets(buf,size,in) ){
 		if( !isEOR(buf) ){
-			if( out != NULL )
-				fputs(buf,out);
 			if( cache != NULL )
 				fputs(buf,cache);
+			if( MIME_transAddrSpec )
+				scanAddrInBody(MIME_transAddrSpec,AVStr(buf));
+			if( out != NULL )
+				fputs(buf,out);
 		}
 	}
-	return rcode;
+	return (char*)rcode;
 }
 #define MP_EOF	-1	/* real EOF */
 #define MP_EOR	-2	/* .<CR><LF> */
 #define MP_EOP	-3	/* --boundary-- */
 #define MP_EOC	-4	/* --boundary */
 
-static scan_boundary(hp,boundary,size)
-	char *hp,*boundary;
-{	char *bp;
+static void scan_boundary(PCStr(hp),PVStr(boundary),int size)
+{	const char *bp;
 
 	if( bp = strcasestr(hp,"boundary=") ){
 		bp += strlen("boundary=");
-		valuescanX(bp,boundary,size);
+		valuescanX(bp,AVStr(boundary),size);
 	}
 }
-static push_boundary(boundaries,boundary)
-	char *boundaries[],*boundary;
+#define MAXDEPTH 32
+static int push_boundary(const char *boundaries[],PCStr(boundary))
 {	int bi;
+
 	for( bi = 0; boundaries[bi]; bi++);
-	boundaries[bi] = strdup(boundary);
+	if( MAXDEPTH-1 <= bi ){
+		syslog_ERROR("## MIME FATAL: too deep boundrary (>%d)\n",bi);
+		return bi-1;
+	}
+	boundaries[bi] = stralloc(boundary);
 	boundaries[bi+1] = 0;
 	return bi;
 }
-static top_boundary(boundaries)
-	char *boundaries[];
+static int top_boundary(const char *boundaries[])
 {	int bi;
 	for( bi = 0; boundaries[bi]; bi++);
 	return bi;
 }
-static pop_boundary(boundaries,line)
-	char *boundaries[],*line;
+static int pop_boundary(const char *boundaries[],PCStr(line))
 {	int blen,bi;
-	char *b1,*tp;
+	const char *b1;
+	const char *tp;
 
 	if( line[0] == '-' && line[1] == '-' ){
 		for( bi = 0; b1 = boundaries[bi]; bi++ ){
@@ -195,7 +206,7 @@ static pop_boundary(boundaries,line)
 			if( strncmp(line+2,b1,blen) == 0 ){
 				tp = &line[2+blen];
 				if( tp[0]=='-' && tp[1]=='-' ){
-					free(b1);
+					free((char*)b1);
 					boundaries[bi] = 0;
 					return MP_EOP;
 				}else
@@ -209,9 +220,8 @@ static pop_boundary(boundaries,line)
 	return 0;
 }
 
-static char *guess_charset(text)
-	char *text;
-{	char *sw;
+static const char *guess_charset(PCStr(text))
+{	const char *sw;
 
 	if( sw = strstr(text,"\033$") )
 		if( sw[2] == '@' || sw[2] == 'B' )
@@ -221,13 +231,12 @@ static char *guess_charset(text)
 			return "ISO-2022-JP";
 	return 0;
 }
-extern int str_fromqp();
 #define BASE64CH(ch) (isalpha(ch) || isdigit(ch) || ch=='+' || ch=='/' || ch=='=' )
 
-str_from64_safe(src,leng,dst,size)
-	char *src,*dst;
+void str_from64_safe(PCStr(src),int leng,PVStr(dst),int size)
 {	char ch;
 	int sx,dx,Bx,cx,isB64,ls,ld,lb;
+	const char *sp;
 
 	Bx = -1;
 	dx = 0;
@@ -248,111 +257,124 @@ str_from64_safe(src,leng,dst,size)
 			if( Bx != -1 ){
 				lb = sx - Bx;
 /* str_from64() before mimehead1.2.4 writes NULL at end of source string ... */
-ch = src[Bx+lb];
-				str_from64(&src[Bx],lb,&dst[dx],size);
-src[Bx+lb] = ch;
+sp = src; /* not read-only but "const" */
+ch = ((char*)sp)[Bx+lb];
+				str_from64(&src[Bx],lb,QVStr(&dst[dx],dst),size);
+((char*)sp)[Bx+lb] = ch; /**/
 				Bx = -1;
 				ld = strlen(&dst[dx]);
 				dx += ld;
 				size -= ld;
 			}
 			ls = cx - sx;
-			strncpy(&dst[dx],&src[sx],ls);
+			Xstrncpy(DVStr(dst,dx),&src[sx],ls);
 			dx += ls;
-			dst[dx] = 0;
+			setVStrEnd(dst,dx);
 			size -= ls;
 		}
 	}
 	if( Bx != -1 )
-		str_from64(&src[Bx],leng-Bx,&dst[dx],size);
+		str_from64(&src[Bx],leng-Bx,QVStr(&dst[dx],dst),size);
 }
 
-static decodeLine(line,dline,encoding)
-	char *line,*dline,*encoding;
-{	int (*conv)();
+typedef void (*convFunc)(const char*,int,PVStr(s),int);
+static int decodeLine(PCStr(line),PVStr(dline),PCStr(encoding))
+{	convFunc conv;
 	int len;
 
 	if( strcasecmp(encoding,"quoted-printable") == 0 )
-		conv = str_fromqp;
+		conv = (convFunc)str_fromqp;
 	else
 	if( strcasecmp(encoding,"base64") == 0 )
 		conv = str_from64_safe;
 	else	return 0;
 
 	len = strlen(line);
-	(*conv)(line,len,dline,len);
+	(*conv)(line,len,AVStr(dline),len+1);
 
 	{ /* bug of str_fromqp ? */
-	unsigned char *sp;
+	unsigned char *sp; /**/
 	for( sp = (unsigned char*)dline; *sp; sp++ )
 		if( *sp == 0xFF )
-			strcpy((char*)sp,(char*)sp+1);
+			ovstrcpy((char*)sp,(char*)sp+1);
 	}
 
 	return 1;
 }
 
-static readHeader(fs,buf,size,cache,filter,ctype,ichset,boundaries,encoding,enHTML)
-	FILE *fs,*cache;
-	char *buf;
-	char *ctype,*ichset,*boundaries[],*encoding;
-{	char *hp,*hs,hc;
+static void scan_charset(PCStr(header),PVStr(chset),int len);
+static void scan_ctype(PCStr(fvalue),PVStr(ctype),PVStr(ichset),PVStr(boundary),int bsize)
+{	const char *dp;
+
+	if( ctype[0] == 0 ){
+		wordscanX(fvalue,AVStr(ctype),128);
+		if( dp = strchr(ctype,';') )
+			truncVStr(dp);
+	}
+	scan_charset(fvalue,AVStr(ichset),64);
+	if( strncasecmp(ctype,"multipart/",10) == 0 )
+		scan_boundary(fvalue,AVStr(boundary),bsize);
+}
+
+static int readHeader(FILE *fs,PVStr(buf),int size,FILE *cache,int filter,PVStr(ctype),PVStr(ichset),const char *boundaries[],PVStr(encoding),int enHTML)
+{	refQStr(hp,buf); /**/
+	const char *hs;
+	char hc;
 	int rem,len;
-	char cur_field[256];
+	CStr(cur_field,256);
 	int conv_field;
-	char boundary[256];
-	char tmp[LINESIZE];
+	CStr(boundary,256);
+	CStr(tmp,LINESIZE);
 
 	maybe_MIME = 0;
 	maybe_NONASCII = 0;
 	got_EOR = 0;
-	strcpy(ctype,C_TEXT_PLAIN);
-	ichset[0] = 0;
-	boundary[0] = 0;
-	encoding[0] = 0;
+	setVStrEnd(ctype,0);
+	setVStrEnd(ichset,0);
+	setVStrEnd(boundary,0);
+	setVStrEnd(encoding,0);
 
-	hp = buf;
-	buf[0] = 0;
+	setVStrEnd(buf,0);
 	conv_field = 0;
 	cur_field[0] = 0;
 
 	for( rem = size; LINESIZE < rem; ){
-		if( fgetsTee(hp,LINESIZE,fs,NULL,cache) == NULL )
+		if( fgetsTee(AVStr(hp),LINESIZE,fs,NULL,cache) == NULL )
 			break;
 		if( got_EOR = isEOR(hp) ){
 			if( !EOR_ALSO(filter) )
-				*hp = 0;
+				setVStrEnd(hp,0);
 			break;
 		}
 		if( isEOH(hp) ){
 			if( !DELIM_ALSO(filter) )
-				*hp = 0;
+				setVStrEnd(hp,0);
 			break;
 		}
 
 		if( hp[0] == ' ' || hp[0] == '\t' ){
 			if( strcasecmp(cur_field,"Content-Type") == 0 ){
-				scan_boundary(hp,boundary,sizeof(boundary));
+				scan_ctype(hp+1,AVStr(ctype),AVStr(ichset),
+					AVStr(boundary),sizeof(boundary));
 			}
 		}else{
-			wordscanY(hp,cur_field,sizeof(cur_field),"^: \t\r\n");
+			wordscanY(hp,AVStr(cur_field),sizeof(cur_field),"^: \t\r\n");
 			conv_field = strncasecmp(hp,"Subject:",8) == 0
 				  || strncasecmp(hp,"From:",5) == 0;
 
 			if( strncasecmp(hp,"Content-Type:",13) == 0 ){
-				wordscanX(hp+13,ctype,128);
-				scan_charset(hp,ichset,64);
-				scan_boundary(hp,boundary,sizeof(boundary));
+				scan_ctype(hp+13,AVStr(ctype),AVStr(ichset),
+					AVStr(boundary),sizeof(boundary));
 			}else
 			if( strncasecmp(hp,FN_CTE_C,26)==0 )
-				wordscanX(hp+26,encoding,128);
+				wordscanX(hp+26,AVStr(encoding),128);
 			else
 			if( strncasecmp(hp,"Date:",5) == 0 )
-				canon_date(hp+5);
+				canon_date(QVStr(hp+5,buf));
 		}
 
 		if( enHTML && conv_field ){
-			encode_entities(hp,tmp);
+			encode_entitiesX(hp,AVStr(tmp),sizeof(tmp));
 			strcpy(hp,tmp);
 		}
 
@@ -372,22 +394,21 @@ static readHeader(fs,buf,size,cache,filter,ctype,ichset,boundaries,encoding,enHT
 	}
 	if( boundary[0] )
 		push_boundary(boundaries,boundary);
+	if( ctype[0] == 0 )
+		strcpy(ctype,C_TEXT_PLAIN);
 	return rem;
 }
 
+typedef int (*encdecFunc)(const char*[],FILE*,FILE*,FILE*,int,int);
+
 static
-scan_multipart(boundaries,endline,size,func,src,dst,cache,filter,arg)
-	char *boundaries[];
-	char *endline;
-	int (*func)();
-	FILE *src,*dst,*cache;
-	char *arg;
-{	char line[LINESIZE];
+int scan_multipart(const char *boundaries[],PVStr(endline),int size,encdecFunc func,FILE*src,FILE*dst,FILE*cache,int filter,int arg)
+{	CStr(line,LINESIZE);
 	int rcode;
 	int top;
 
 	/* preamble */
-	while( fgetsTee(line,size,src,NULL,cache) != NULL ){
+	while( fgetsTee(AVStr(line),size,src,NULL,cache) != NULL ){
 		if( !MIME2PGP(filter) )
 		if( !TEXTONLY(filter) )
 		fputs(line,dst);
@@ -413,12 +434,12 @@ scan_multipart(boundaries,endline,size,func,src,dst,cache,filter,arg)
 	 * output already at (*func)(),  or it has really empty
 	 * in case of MP_EOF, or ... ??
 	 */
-	endline[0] = 0;
+	setVStrEnd(endline,0);
 
 	/* epilogue */
 	if( rcode == MP_EOP ){
 		for(;;){
-			if( fgetsTee(endline,size,src,NULL,cache) == NULL ){
+			if( fgetsTee(AVStr(endline),size,src,NULL,cache) == NULL ){
 				rcode = MP_EOF;
 				break;
 			}
@@ -434,16 +455,11 @@ scan_multipart(boundaries,endline,size,func,src,dst,cache,filter,arg)
 	return rcode;
 }
 
-static char *readPart(src,dst,cache,boundaries,line,codep,lengp)
-	FILE *src,*dst,*cache;
-	char *boundaries[];
-	char *line;
-	int *codep,*lengp;
+static char *readPart(FILE*src,FILE*dst,FILE*cache,const char *boundaries[],PVStr(line),int *codep,int *lengp)
 {	int rcode;
 	int leng,len1,reqsize;
-	char buff[0x10000];
+	CStr(buff,0x10000);
 	FILE *bfile;
-	char *tmp;
 
 	rcode = 0;
 	leng = 0;
@@ -451,8 +467,8 @@ static char *readPart(src,dst,cache,boundaries,line,codep,lengp)
 	bfile = NULL;
 
 	for(;;){
-		line[0] = 0;
-		if( fgetsTee(line,LINESIZE,src,dst,cache) == NULL ){
+		setVStrEnd(line,0);
+		if( fgetsTee(AVStr(line),LINESIZE,src,dst,cache) == NULL ){
 			/*DEBUG("GotEOF: length=%d\n",leng);*/
 			rcode = MP_EOF;
 			break;
@@ -469,55 +485,55 @@ static char *readPart(src,dst,cache,boundaries,line,codep,lengp)
 		}
 		if( bfile )
 			fputs(line,bfile);
-		else	strcpy(buff+leng,line);
+		else	Xstrcpy(DVStr(buff,leng),line);
 		leng += len1;
 	}
 	*lengp = leng;
 	*codep = rcode;
 	if( bfile ){
+		const char *tmp;
 		DEBUG("### readPart: got large message (%d)\n",leng);
 		fflush(bfile);
 		fseek(bfile,0,0);
-		tmp = malloc(leng+1);
-		fread(tmp,1,leng,bfile);
-		tmp[leng] = 0;
+		tmp = (char*)malloc(leng+1);
+		fread((char*)tmp,1,leng,bfile);
+		((char*)tmp)[leng] = 0;
 		fclose(bfile);
-		return tmp;
+		return (char*)tmp;
 	}else{
-		return strdup(buff);
+		return stralloc(buff);
 	}
 }
 
 #define JBSIZE(leng)	(leng*2+128)
 
-static scan_charset(header,chset,len)
-	char *header,*chset;
-{	char *cp;
+static void scan_charset(PCStr(header),PVStr(chset),int len)
+{	const char *cp;
 
 	if( cp = strcasestr(header,"charset=") )
-		valuescanX(cp+8,chset,len);
+		valuescanX(cp+8,AVStr(chset),len);
 }
-static external_charcode(src,dst,dlen,ichset)
-	char *src,*dst,*ichset;
+static void external_charcode(PCStr(src),PVStr(dst),int dlen,PCStr(ichset))
 {	int ccx[32];
 
-	CCXcreate(ichset,"ISO-2022-JP",ccx);
-	CCXexec(ccx,src,strlen(src),dst,dlen);
+	if( strcaseeq(ichset,"ISO-2022-JP") )
+		ichset = "*";
+	CCXcreate((char*)ichset,"ISO-2022-JP",(CCXP)ccx);
+	CCXexec((CCXP)ccx,src,strlen(src),AVStr(dst),dlen);
 }
 
-static encodeBODYpart(fc,ts,ctype,ichset,boundaries,encoding,charsetp)
-	FILE *fc,*ts;
-	char *ctype,*ichset;
-	char *boundaries[],*encoding,**charsetp;
-{	unsigned char line[LINESIZE];
-	char endline[LINESIZE];
-	char *charset = 0;
+static int encodeBODYpart(FILE *fc,FILE *ts,PCStr(ctype),PCStr(ichset),const char *boundaries[],PCStr(encoding),const char **charsetp)
+{
+	CStr(endline,LINESIZE);
+	const char *charset = 0;
 	int rcode,leng;
-	char *tmpa,*tmpb,*tmpc;
+	const char *tmpa;
+	defQStr(tmpb); /*alloc*/
+	defQStr(tmpc); /*alloc*/
 
-	tmpa = readPart(fc,NULL,NULL,boundaries,endline,&rcode,&leng);
-	tmpb = malloc(JBSIZE(leng));
-	tmpc = malloc(JBSIZE(leng));
+	tmpa = readPart(fc,NULL,NULL,boundaries,AVStr(endline),&rcode,&leng);
+	setQStr(tmpb,(char*)malloc(JBSIZE(leng)),JBSIZE(leng));
+	setQStr(tmpc,(char*)malloc(JBSIZE(leng)),JBSIZE(leng));
 
 DEBUG("BODY-LENG1:%d+%d\n",strlen(tmpa),strlen(endline));
 
@@ -532,11 +548,11 @@ DEBUG("BODY-LENG1:%d+%d\n",strlen(tmpa),strlen(endline));
 	 * implementation) C-T-E will be replaced with "7bit".
 	 * Otherwise the body is passed through as is.
 	 */
-	if( !decodeLine(tmpa,tmpb,encoding) )
+	if( !decodeLine(tmpa,AVStr(tmpb),encoding) )
 		strcpy(tmpb,tmpa);
 
 	if( ENCODE(C_BODY_CHAR) )
-		external_charcode(tmpb,tmpc,JBSIZE(leng),ichset);
+		external_charcode(tmpb,AVStr(tmpc),JBSIZE(leng),ichset);
 	else	strcpy(tmpc,tmpb);
 	if( charset == 0 ){
 		if( charset = guess_charset(tmpc) )
@@ -557,53 +573,51 @@ OUT:
 	}
 
 	fputs(endline,ts);
-	free(tmpa);
-	free(tmpb);
-	free(tmpc);
+	free((char*)tmpa);
+	free((char*)tmpb);
+	free((char*)tmpc);
 
-	*charsetp = charset;
+	*charsetp = (char*)charset;
 	return rcode;
 }
-relayBODYpart(src,dst,boundaries,extract,endline)
-	FILE *src,*dst;
-	char *boundaries;
-	char *endline;
-{	char *tmpa,*ep;
+int relayBODYpart(FILE *src,FILE *dst,const char *boundaries[],int extract,PVStr(endline))
+{	const char *tmpa;
+	const char *ep;
 	int rcode,leng;
 
-	tmpa = readPart(src,NULL,NULL,boundaries,endline,&rcode,&leng);
+	tmpa = readPart(src,NULL,NULL,boundaries,AVStr(endline),&rcode,&leng);
 	if( tmpa[0] && extract ){
 		ep = &tmpa[strlen(tmpa)-1];
 		if( *ep == '\n' ){
-			*ep = 0;
+			truncVStr(ep);
 			if( tmpa <= --ep && *ep == '\r' )
-				*ep = 0;
+				truncVStr(ep);
 		}
 	}
 	fputs(tmpa,dst);
-	free(tmpa);
+	free((char*)tmpa);
 	return rcode;
 }
-static decodeBODYpart(fs,tc,cache,ctype,boundaries,encoding,decodeto,do_enHTML,endline)
-	FILE *fs,*tc,*cache;
-	char *ctype,*boundaries[];
-	char *encoding,*decodeto;
-	char *endline;
+
+static int decodeBODYpart(FILE *fs,FILE *tc,FILE *cache,PCStr(ctype),const char *boundaries[],PCStr(encoding),PCStr(decodeto),int do_enHTML,PVStr(endline))
 {	int rcode,leng;
-	char *tmpa,*tmpb;
+	defQStr(tmpa); /*alloc*/
+	defQStr(tmpb); /*alloc*/
+	int tsiz;
 	int do_conv,plain2html;
-	char *xcharset;
+	const char *xcharset;
 	int len,elen;
-	char *eol = NULL, *beol = NULL;
+	const char *eol = NULL;
+	const char *beol = NULL;
 	FILE *out_thru;
 
 	do_conv = codeconv_get(ctype,&xcharset,&plain2html);
 
-	*endline = 0;
+	setVStrEnd(endline,0);
 	if( DECODE(C_BODY) == 0 )
 		out_thru = tc;
 	else	out_thru = NULL;
-	tmpa = readPart(fs,out_thru,cache,boundaries,endline,&rcode,&leng);
+	tmpa = readPart(fs,out_thru,cache,boundaries,AVStr(endline),&rcode,&leng);
 	if( tmpa == NULL )
 		return -1;
 	if( leng == 0 )
@@ -619,32 +633,32 @@ static decodeBODYpart(fs,tc,cache,ctype,boundaries,encoding,decodeto,do_enHTML,e
 		/* message body was already relayed thru ... */
 		/* end of body line (except EOR) was already put too */
 		if( rcode != MP_EOR )
-			endline[0] = 0;
+			setVStrEnd(endline,0);
 		goto PUTEOL;
 	}
 
 	if( plain2html ){
-		tmpa = realloc(tmpa,leng*10);
-		tmpb = malloc(leng*10);
+		tsiz = leng * 10;
 	}else{
-		tmpa = realloc(tmpa,JBSIZE(leng));
-		tmpb = malloc(JBSIZE(leng));
+		tsiz = JBSIZE(leng);
 	}
+	setQStr(tmpa,(char*)realloc((char*)tmpa,tsiz),tsiz);
+	setQStr(tmpb,(char*)malloc(tsiz),tsiz);
 
 	if( DECODE(C_BODY_CTE) && encoding != NULL && decodeto != NULL ){
 		DEBUG("decode: %s -> %s\n",encoding,decodeto);
-		if( decodeLine(tmpa,tmpb,encoding) )
+		if( decodeLine(tmpa,AVStr(tmpb),encoding) )
 			strcpy(tmpa,tmpb);
 	}
 	if( DECODE(C_BODY_CHAR) && do_conv ){
-		codeconv_line(tmpa,tmpb,ctype,0);
+		codeconv_line(tmpa,AVStr(tmpb),ctype,0);
 		strcpy(tmpa,tmpb);
 	}
 	if( do_enHTML ){
-		encode_entities(tmpa,tmpb);
+		encode_entitiesX(tmpa,AVStr(tmpb),tsiz);
 		strcpy(tmpa,tmpb);
 	}
-	free(tmpb);
+	free((char*)tmpb);
 EXIT:
 	fputs(tmpa,tc);
 
@@ -653,38 +667,45 @@ PUTEOL:
 		eol = strpbrk(endline,"\r\n"); /* maybe .CRLF via NNTP */
 	if( eol == NULL )
 		eol = beol;
+
+	/* can be converted CRLF to/from LF */
+	if( eol && (elen = strlen(eol)) )
+	if( DECODE(C_BODY_CHAR) && do_conv ){
+		CStr(xeol,LINESIZE);
+		codeconv_line(eol,AVStr(xeol),ctype,0);
+		if( strcmp((char*)eol,(char*)xeol) != 0 ){
+			eol = xeol;
+		}
+	}
+
 	if( eol && (elen = strlen(eol)) )
 	if( len = strlen(tmpa) )
 	if( len < elen || strcmp(&tmpa[len-elen],eol) != 0 ){
 		fputs(eol,tc);
 		DEBUG("supply missing <CR><LF> at the end of BODY part\n");
 	}
-	free(tmpa);
+	free((char*)tmpa);
 	return rcode;
 }
 
-decodeBODY(fs,tc,filter,encoding,decodeto,do_enHTML)
-	FILE *fs,*tc;
-	char *encoding,*decodeto;
-{	char *boundaries[32];
-	char *ctype = "text/plain";
-	char endline[LINESIZE];
+int decodeBODY(FILE *fs,FILE *tc,int filter,PCStr(encoding),PCStr(decodeto),int do_enHTML)
+{	const char *boundaries[MAXDEPTH]; /**/
+	const char *ctype = "text/plain";
+	CStr(endline,LINESIZE);
 	int rcode;
 
 	boundaries[0] = 0;
 	rcode = decodeBODYpart(fs,tc,NULL,ctype,
-			boundaries,encoding,decodeto,do_enHTML,endline);
+			boundaries,encoding,decodeto,do_enHTML,AVStr(endline));
 	if( EOR_ALSO(filter) || endline[0] != '.' )
 		fputs(endline,tc);
 	return rcode;
 }
 
-typedef FILE *(*FFUNC)();
-static FFUNC HEAD_filter;
+static headFilter HEAD_filter;
 
-FFUNC set_HEAD_filter(filter)
-	FFUNC filter;
-{	FFUNC ofilter;
+headFilter set_HEAD_filter(headFilter filter)
+{	headFilter ofilter;
 
 	ofilter = HEAD_filter;
 	HEAD_filter = filter;
@@ -694,30 +715,36 @@ FFUNC set_HEAD_filter(filter)
 extern FILE *NULLFP();
 
 static
-decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
-	char *boundaries[];
-	FILE *fs,*tc,*cache;
-{	char head[2][0x10000];
-	char ctype[128],ichset[64],encoding[128],*decodeto;
+int decodeMIMEpart(const char *boundaries[],FILE*fs,FILE*tc,FILE*cache,int filter,int enHTML)
+{	ACStr(head,2,0x10000);
+	CStr(ctype,128);
+	CStr(ichset,64);
+	CStr(encoding,128);
+	const char *decodeto;
 	int hi,nhi;
 	int do_enHTML;
 	int putPRE;
 	int rcode;
-	char endline[LINESIZE];
+	CStr(endline,LINESIZE);
 	int do_conv,plain2html;
-	char *xcharset;
+	const char *xcharset;
 	FILE *ntc;
+	int ismainpart;
 
+	ismainpart = boundaries[0] == 0;
 	do_conv = codeconv_get(NULL,&xcharset,&plain2html);
-	readHeader(fs,head[0],sizeof(head[0]),
-		cache,filter,ctype,ichset,boundaries,encoding,enHTML);
+	readHeader(fs,FVStr(head[0]),sizeof(head[0]),
+		cache,filter,AVStr(ctype),AVStr(ichset),boundaries,AVStr(encoding),enHTML);
 
 	if( got_EOR )
 		rcode = MP_EOR;
 	else	rcode = 0;
 
+	/*
 	if( HEAD_filter && boundaries[0] == NULL ){
-		ntc = (*HEAD_filter)(head[0],tc,cache);
+	*/
+	if( HEAD_filter && ismainpart ){
+		ntc = (*HEAD_filter)(EVStr(head[0]),tc,cache);
 		if( ntc != NULL )
 			tc = ntc;
 		else{
@@ -731,16 +758,19 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 	if( DECODE(C_HEAD_EW) )
 	if( maybe_MIME ){
 		nhi = (hi + 1 ) % 2;
-		MIME_strHeaderDecode(head[hi],head[nhi],sizeof(head[1]));
+		MIME_strHeaderDecode(head[hi],FVStr(head[nhi]),sizeof(head[1]));
 		hi = nhi;
 		maybe_NONASCII = 1;
 	}
 
 	if( !plain2html )
 	if( DECODE(C_HEAD_CHAR) )
+	/*
 	if( maybe_NONASCII && do_conv ){
+	*/
+	if( maybe_NONASCII && do_conv || (do_conv & 0xFFFE) ){
 		nhi = (hi + 1 ) % 2;
-		codeconv_line(head[hi],head[nhi],C_TEXT_PLAIN,1);
+		codeconv_line(head[hi],FVStr(head[nhi]),C_TEXT_PLAIN,1);
 		hi = nhi;
 	}
 
@@ -751,7 +781,7 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 
 		ofilter = filter;
 		if( TEXTONLY(filter) ){
-			replaceContentType(head[hi],"text/plain");
+			replaceContentType(FVStr(head[hi]),"text/plain");
 			filter |= O_MULTIPART;
 			mask = O_HEAD | O_DELIM;
 		}
@@ -780,7 +810,7 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 		}
 
 		filter &= ~mask;
-		rcode = scan_multipart(boundaries,endline,sizeof(endline),
+		rcode = scan_multipart(boundaries,AVStr(endline),sizeof(endline),
 				decodeMIMEpart,fs,mtc,cache,filter,enHTML);
 		filter = ofilter;
 		goto EXIT;
@@ -794,7 +824,7 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 			if( plain2html ){
 				putPRE = 1;
 				/*do_enHTML = 1;*/
-				replaceContentType(head[hi],"text/html");
+				replaceContentType(FVStr(head[hi]),"text/html");
 			}
 		}
 
@@ -802,13 +832,20 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 		 || DECODE(C_BODY_CTE ) && strcaseeq(encoding,"quoted-printable")
 	 	 || DECODE(C_BODY_CTE ) && strcaseeq(encoding,"base64") ){
 			decodeto = "8bit";
-			replaceFieldValue(head[hi],FN_CTE,decodeto);
+			replaceFieldValue(FVStr(head[hi]),FN_CTE,decodeto);
 			DEBUG("decodeMIME[encoding=%s]\n",decodeto);
 		}else	decodeto = NULL;
 
+		/*
 		if( DECODE(C_BODY_CHAR) && xcharset ){
-			replace_charset(head[hi],xcharset);
+		*/
+		if( DECODE(C_BODY_CHAR) && xcharset && *xcharset ){
+			if( streq(xcharset,"guess") ){
+				/* cannot guess ... */
+			}else{
+			replace_charset(FVStr(head[hi]),xcharset);
 			DEBUG("decodeMIME[charset=%s]\n",xcharset);
+			}
 		}
 	}else{
 		do_enHTML = 0;
@@ -835,23 +872,23 @@ decodeMIMEpart(boundaries,fs,tc,cache,filter,enHTML)
 	 && strncasecmp(ctype,"text/plain",10) != 0 ){
 		DEBUG("## TEXTONLY skip non text/plain: %s\n",ctype);
 		rcode = decodeBODYpart(fs,NULLFP(),cache,ctype,boundaries,
-				encoding,decodeto,do_enHTML,endline);
+				encoding,decodeto,do_enHTML,AVStr(endline));
 	}else
 	if( BODY_ALSO(filter) ){
 		if( putPRE ) fprintf(tc,"<PRE>\n");
 
 		if( MIME2PGP(filter) )
 			if( substr(ctype,C_PGPSIGN) )
-				rcode = relay_pgpSIGN(fs,tc,boundaries,endline);
-			else	rcode = relay_pgpSIGNED(fs,tc,boundaries,endline);
+				rcode = relay_pgpSIGN(fs,tc,boundaries,AVStr(endline));
+			else	rcode = relay_pgpSIGNED(fs,tc,boundaries,AVStr(endline));
 		else
 		rcode = decodeBODYpart(fs,tc,cache,ctype,boundaries,
-				encoding,decodeto,do_enHTML,endline);
+				encoding,decodeto,do_enHTML,AVStr(endline));
 		if( putPRE ) fprintf(tc,"</PRE>\n");
 	}else{
 		if( fseek(fs,0,0) == -1 ){
 			DEBUG("skip body from stream input.\n");
-			RFC821_skipbody(fs,NULL,endline,sizeof(endline));
+			RFC821_skipbody(fs,NULL,AVStr(endline),sizeof(endline));
 		}
 	}
 EXIT:
@@ -860,17 +897,63 @@ EXIT:
 	if( EOR_ALSO(filter) || endline[0] != '.' && !MIME2PGP(filter) )
 		fputs(endline,tc);
 
+/*
+	if( addQY ){
+		CStr(date,64);
+		CStr(qy,128);
+		int clock;
+		if( getFieldValue2(head,"Date",date,sizeof(date)) ){
+			clock = scanNNTPtime(date);
+			makeQY(clock,qy,sizeof(qy));
+		}
+		fprintf(tc,"%s%s",qy,"\n");
+	}
+*/
+
 	return rcode;
 	/*
 	 * Result (real) Encoding and Charcode should be set in the header
 	 * after this decoding...
 	 */
 }
+/*
+makeQY(int clock,PVStr(qy),int siz)
+{	CStr(year,64);
+	CStr(mon,64);
+	refQStr(yp,qy);
+	int yi,mi,i;
 
-fputsCRLF(str,out)
-	char *str;
-	FILE *out;
-{	char *bp;
+	StrftimeLocal(year,sizeof(year),"%Y",clock);
+	yi = atoi(year);
+	StrftimeLocal(mon,sizeof(mon),"%m",clock);
+	mi = atoi(mon);
+
+	if( yi < 1990 ) yi = 0; else
+	if( 2009 < yi ) yi = 20; else
+		yi = yi - 1990; 
+
+	yp = qy;
+	for( i = 0; i < yi; i++ ){
+		strcpy(yp,"qy ");
+		yp += strlen(yp);
+	}
+	for( ; i < 20; i++ ){
+		strcpy(yp,"yq ");
+		yp += strlen(yp);
+	}
+	for( i = 1; i <= mi; i++ ){
+		strcpy(yp,"qk ");
+		yp += strlen(yp);
+	}
+	for( ; i <= 12; i++ ){
+		strcpy(yp,"kq ");
+		yp += strlen(yp);
+	}
+}
+*/
+
+void fputsCRLF(PCStr(str),FILE *out)
+{	const char *bp;
 	int ch,pch;
 
 	pch = 0;
@@ -885,18 +968,19 @@ fputsCRLF(str,out)
 }
 
 static
-encodeMIMEpart(boundaries,fc,ts,cache,filter)
-	char *boundaries[];
-	FILE *fc,*ts,*cache;
-{	char head[2][0x10000];
+int encodeMIMEpart(const char *boundaries[],FILE*fc,FILE*ts,FILE*cache,int filter,int _)
+{	ACStr(head,2,0x10000);
 	int hi,nhi;
 	FILE *tmp;
-	char *charset,ctype[128],ichset[64],encoding[128];
+	const char *charset;
+	CStr(ctype,128);
+	CStr(ichset,64);
+	CStr(encoding,128);
 	int rcode;
-	char endline[LINESIZE];
+	CStr(endline,LINESIZE);
 
-	readHeader(fc,head[0],sizeof(head[0]),
-		cache,filter,ctype,ichset,boundaries,encoding,0);
+	readHeader(fc,FVStr(head[0]),sizeof(head[0]),
+		cache,filter,AVStr(ctype),AVStr(ichset),boundaries,AVStr(encoding),0);
 	if( got_EOR )
 		rcode = MP_EOR;
 	else	rcode = 0;
@@ -904,19 +988,19 @@ encodeMIMEpart(boundaries,fc,ts,cache,filter)
 	hi = 0;
 	if( maybe_NONASCII ){
 		if( ENCODE(C_HEAD_CHAR) )
-			external_charcode(head[0],head[1],0x10000,ichset);
-		else	strcpy(head[1],head[0]);
+			external_charcode(head[0],FVStr(head[1]),0x10000,ichset);
+		else	Xstrcpy(EVStr(head[1]),head[0]);
 		if( strcmp(head[0],head[1]) != 0 ){
 			DEBUG("POST: code converted.\n");
 		}
 		if( ENCODE(C_HEAD_EW) )
-			MIME_strHeaderEncode(head[1],head[0],sizeof(head[0]));
+			MIME_strHeaderEncode(head[1],FVStr(head[0]),sizeof(head[0]));
 	}
 
 	if( strncasecmp(ctype,"multipart/",10) == 0 ){
 		fputs(head[hi],ts);
-		rcode = scan_multipart(boundaries,endline,sizeof(endline),
-				encodeMIMEpart,fc,ts,cache,filter,NULL);
+		rcode = scan_multipart(boundaries,AVStr(endline),sizeof(endline),
+				encodeMIMEpart,fc,ts,cache,filter,0);
 		fputs(endline,ts);
 		return rcode;
 	}
@@ -929,9 +1013,9 @@ encodeMIMEpart(boundaries,fc,ts,cache,filter)
 		charset = 0;
 		rcode = encodeBODYpart(fc,tmp,ctype,ichset,boundaries,encoding,&charset);
 		if( charset ){
-			replace_charset(head[hi],charset);
+			replace_charset(FVStr(head[hi]),charset);
 			if( findField(head[hi],FN_CTE,NULL) )
-			replaceFieldValue(head[hi],FN_CTE,"7bit");
+			replaceFieldValue(FVStr(head[hi]),FN_CTE,"7bit");
 		}
 		fflush(tmp);
 		fseek(tmp,0,0);
@@ -940,36 +1024,38 @@ encodeMIMEpart(boundaries,fc,ts,cache,filter)
 		fclose(tmp);
 	}else{
 		fputs(head[hi],ts);
-		rcode = relayBODYpart(fc,ts,boundaries,0,endline);
+		rcode = relayBODYpart(fc,ts,boundaries,0,AVStr(endline));
 		fputs(endline,ts);
 	}
 	return rcode;
 }
 
-thruRESP(fs,tc)
-	FILE *fs,*tc;
-{	char line[LINESIZE];
+void thruRESP(FILE *fs,FILE *tc)
+{	CStr(line,LINESIZE);
 
-	relayRESPBODY(fs,tc,line,sizeof(line));
+	relayRESPBODY(fs,tc,AVStr(line),sizeof(line));
 	fputs(line,tc);
 }
 
-static delWhites(str)
-	char *str;
-{	char ch,*sp,*dp;
+static void delWhites(PVStr(str))
+{	char ch;
+	const char *sp;
+	refQStr(dp,str); /**/
 
-	dp = str;
+	cpyQStr(dp,str);
 	for( sp = str; ch = *sp; sp++ ){
 		if( ch != '\t' && ch != '\r' && ch != '\n' )
-			*dp++ = ch;
+			setVStrPtrInc(dp,ch);
 	}
-	*dp = 0;
+	setVStrEnd(dp,0);
 }
-decodeTERM1(line,xline)
-	char *line,*xline;
-{	char *xp,*tp,dc,*np;
+void decodeTERM1(PCStr(line),PVStr(xline))
+{	refQStr(xp,xline); /**/
+	const char *tp;
+	char dc;
+	const char *np;
 	int do_conv,plain2html;
-	char *xcharset;
+	const char *xcharset;
 
 	if( !DECODE(C_HEAD) ){
 		strcpy(xline,line);
@@ -978,31 +1064,32 @@ decodeTERM1(line,xline)
 
 	do_conv = codeconv_get(NULL,&xcharset,&plain2html);
 
-	xp = xline;
+	cpyQStr(xp,xline);
+
 	for( tp = line; *tp; tp = np ){
 		if( np = strpbrk(tp,"\t\r\n") ){
 			dc = *np;
-			*np = 0;
+			*(char*)np = 0; /**/
 		}
 		if( *tp ){
-			decodeHEAD1(tp,xp,
+			decodeHEAD1(tp,AVStr(xp),
 				DECODE(C_HEAD_EW),
 				DECODE(C_HEAD_CHAR) && !plain2html);
 			xp += strlen(xp);
 		}
 		if( np ){
-			*np++ = dc;
-			*xp++ = dc;
+			*(char*)np++ = dc; /**/
+			setVStrPtrInc(xp,dc);
 		}else	break;
 	}
-	*xp = 0;
+	setVStrEnd(xp,0);
 }
 
-decodeHEAD1(tp,xp,decode,cconv)
-	char *tp,*xp;
+void decodeHEAD1(PCStr(tp),PVStr(xp),int decode,int cconv)
 {	int convert;
-	char term[0x4000],xterm[0x4000];
-	char *ts;
+	CStr(term,0x4000);
+	CStr(xterm,0x4000);
+	const char *ts;
 
 	convert = 0;
 	for( ts = tp; *ts; ts++ ){
@@ -1017,23 +1104,23 @@ decodeHEAD1(tp,xp,decode,cconv)
 	}
 	if( convert ){
 		if( decode )
-			MIME_strHeaderDecode(tp,term,sizeof(term));
+			MIME_strHeaderDecode(tp,AVStr(term),sizeof(term));
 		else	strcpy(term,tp);
 
 		/* remove possible white spaces
 		 * insterted by MIME decoder (-_-; */
-		delWhites(term);
+		delWhites(AVStr(term));
 
 		if( cconv ){
-			codeconv_line(term,xterm,C_TEXT_PLAIN,1);
+			codeconv_line(term,AVStr(xterm),C_TEXT_PLAIN,1);
 			strcpy(xp,xterm);
 		}else	strcpy(xp,term);
 	}else	strcpy(xp,tp);
 }
 
-decodeTERM(fs,tc,enHTML)
-	FILE *fs,*tc;
-{	char line[0x8000],xline[0x8000];
+void decodeTERM(FILE *fs,FILE *tc,int enHTML)
+{	CStr(line,0x8000);
+	CStr(xline,0x8000);
 
 	while( fgets(line,sizeof(line),fs) != NULL){
 		if( isEOR(line) ){
@@ -1041,23 +1128,21 @@ decodeTERM(fs,tc,enHTML)
 			break;
 		}
 		if( DECODE(C_HEAD) ){
-			decodeTERM1(line,xline);
+			decodeTERM1(line,AVStr(xline));
 			fputs(xline,tc);
 		}else	fputs(line,tc);
 	}
 }
 
-encodeMIME(fc,ts)
-	FILE *fc,*ts;
-{	char *boundaries[32];
+void encodeMIME(FILE *fc,FILE *ts)
+{	const char *boundaries[MAXDEPTH]; /**/
 
 	boundaries[0] = 0;
-	encodeMIMEpart(boundaries,fc,ts,NULL,O_ALL);
+	encodeMIMEpart(boundaries,fc,ts,NULL,O_ALL,0);
 }
 
-decodeMIME(fs,tc,cache,filter,codeconv,enHTML)
-	FILE *fs,*tc,*cache;
-{	char *boundaries[32];
+void decodeMIME(FILE*fs,FILE*tc,FILE*cache, int filter,int codeconv,int enHTML)
+{	const char *boundaries[MAXDEPTH]; /**/
 	int ssc;
 
 	ssc = codeconv_set(codeconv,NULL,-1);
@@ -1066,13 +1151,11 @@ decodeMIME(fs,tc,cache,filter,codeconv,enHTML)
 	codeconv_set(ssc,NULL,-1);
 }
 
-deMime(ac,av)
-	char *av[];
+void deMime(int ac,const char *av[])
 {
 	decodeMIME(stdin,stdout,NULL,O_ALL,1,0);
 }
-enMime(ac,av)
-	char *av[];
+void enMime(int ac,const char *av[])
 {
 	encodeMIME(stdin,stdout);
 }
