@@ -31,12 +31,9 @@ Bugs:
 	Should support any charsets & encodings in ISO-2022 or ISO-8859
 //////////////////////////////////////////////////////////////////////#*/
 
-#include <stdio.h>
+#include "mime.h"
 #include "str_stdio.h"
-#include "ystring.h"
-FILE *tmpfile();
-FILE *str_fopen();
-char *getenv();
+
 #define MAX_LNSIZE	1024
 typedef unsigned char MsgLine[MAX_LNSIZE];
 
@@ -45,7 +42,6 @@ typedef unsigned char MsgLine[MAX_LNSIZE];
 #define DEBUG(a)	a
 */
 
-
 #define ES_NONE	0
 #define ES_IN	1	/* type-A encoding (not supported yet...) */
 #define ES_OUT	2	/* type-B encoding */
@@ -122,7 +118,7 @@ int MIME_SPACE_ENCODING = ES_OUT;
  */
 typedef struct {
 	char	 iso2022[8];
-	char	*name;
+  const	char	*name;
 	int	 delspace;	/* as indication code for space deletion */
 } Charset;
 
@@ -146,7 +142,7 @@ static Charset BasicCharsets[] = {
 #define GOTO_ASCII_SEQ		BasicCharsets[B_US_ASCII].iso2022
 #define GOTO_ASCII_SEQ_LEN	strlen(GOTO_ASCII_SEQ)
 
-static char *DELSP_SEQUENCE[8] = {
+static const char *DELSP_SEQUENCE[8] = {
 	0,
 	BasicCharsets[B_JP_KANJI1].iso2022,
 	BasicCharsets[B_JP_KANJI2].iso2022,
@@ -168,9 +164,9 @@ char M_UTF_8[]		= "UTF-8";
 typedef struct {
 	int	local;
 	char	codesw;
-	char   *mcharset;
+  const	char   *mcharset;
 	int	basic_charset;
-	char   *encoding;
+  const	char   *encoding;
 } MimeCharset;
 
 static MimeCharset Codes1[16] = {
@@ -185,10 +181,9 @@ static MimeCharset Codes2[16] = {
 	0
 };
 
-MIME_localCharset(mcharset)
-	char *mcharset;
+int MIME_localCharset(PCStr(mcharset))
 {	int csi;
-	char *cs;
+	const char *cs;
 
 	for(csi = 0; cs = Codes1[csi].mcharset; csi++)
 		if( strcasecmp(cs,mcharset) == 0 )
@@ -205,7 +200,7 @@ MIME_localCharset(mcharset)
  */
 typedef struct {
 	int	c_ch;		/* character code value */
-	char*	c_mcharset;	/* MIME charset */
+  const char*	c_mcharset;	/* MIME charset */
 	int	c_bcharset;	/* basic charset */
 } CHAR;
 static CHAR NULL_CHAR = { 0,  M_US_ASCII, B_US_ASCII };
@@ -224,17 +219,16 @@ static RRBUFF NULL_RRBUFF = {0};
 typedef struct {
 	FILE	*in_file;
 	int	 in_column;
-	char	*in_mcharset;		/* current MIME charset */
+  const char	*in_mcharset;		/* current MIME charset */
 	int	 in_bcharset;		/* current basic charset */
 	int	 in_bcharset_got;	/* bcharset was got explicitly */
-	char	*in_encoding;		/* B or Q */
+  const	char	*in_encoding;		/* B or Q */
 	int	 in_prevch;		/* EN_FGETC() / DE_FGETC() local */
 	RRBUFF	 in_BUFF;		/* EN_FGETC() local */
 	RRBUFF	 in_PUSHED;		/* EN_UNGETC() -> EN_FGETC() */
 
 	FILE	*out_file;
 	int	 out_column;
-	int	 out_nonLWSP;		/* non LWSP output on the line */
 	int	 out_lastputch;		/* EN_FPUTC() -> encode_word() */
 	int	 out_whichASCII;	/* disp_word() -> DE_FPUTC(),C1() */
 	int	 out_enLWSP;		/* EN_FPUTC() local */
@@ -255,9 +249,7 @@ typedef struct {
 } INOUT;
 #define ENCODE_EXT	io_MODES.mode.ext_SPACE
 
-static INOUT_init(io,in,out)
-	INOUT *io;
-	FILE *in,*out;
+static void INOUT_init(INOUT *io,FILE *in,FILE *out)
 {
 	io->in_file           = in;
 	io->in_column         = 0;
@@ -271,7 +263,6 @@ static INOUT_init(io,in,out)
 
 	io->out_file          = out;
 	io->out_column        = 0;
-	io->out_nonLWSP       = 0;
 	io->out_lastputch     = 0;
 	io->out_whichASCII    = B_US_ASCII;
 	io->out_enLWSP        = 0;
@@ -312,8 +303,7 @@ static INOUT_init(io,in,out)
 
 
 static int end_CRLF;
-static NLfgetc(in)
-	FILE *in;
+static int NLfgetc(FILE *in)
 {	int ch;
 
 	ch = fgetc(in);
@@ -334,27 +324,23 @@ static NLfgetc(in)
 	}
 	return ch;
 }
-static NLfputc(ch,out)
-	FILE *out;
+static void NLfputc(int ch,FILE *out)
 {
 	if( ch == NL && end_CRLF )
 		fputc(CR,out);
 	fputc(ch,out);
 }
 
-static EN_UNGETC(CH,io)
-	INOUT *io;
-	CHAR *CH;
+static void EN_UNGETC(CHAR *CH,INOUT *io)
 {
 	PUT_RRBUFF(&io->in_PUSHED,CH);
 }
 
-static CHAR *EN_FGETC(io)
-	INOUT *io;
+static CHAR *EN_FGETC(INOUT *io)
 {	int ch;
 	MimeCharset *csw;
 	int ci;
-	char *mcharset;
+	const char *mcharset;
 	FILE *infile = io->in_file;
 	CHAR *CH;
 	RRBUFF *BP;
@@ -428,16 +414,14 @@ EXIT:
 	return CH;
 }
 
-static ew_overhead(charset,encoding)
-	char *charset,*encoding;
-{	MsgLine overhead;
+static int ew_overhead(PCStr(charset),PCStr(encoding))
+{	CStr(overhead,MAX_LNSIZE);
 
-	sprintf((char*)overhead,"=?%s?%s?x?= ",charset,encoding);
+	sprintf(overhead,"=?%s?%s?x?= ",charset,encoding);
 	return strlen(overhead) - 1;
 }
 
-static EN_FPUTC0(ch,io)
-	INOUT *io;
+static void EN_FPUTC0(int ch,INOUT *io)
 {
 	if( ch == NL ){
 		io->SPACE_WAS_GEN = 0;
@@ -454,11 +438,10 @@ static EN_FPUTC0(ch,io)
  *	put ?= at the end of non-ASCII
  *	put SPACE after it if the next char is not DELIMITER
  */
-static EN_FPUTC1(ch,io,charset,encoding)
-	INOUT *io;
-	char *charset,*encoding;
-{	unsigned char *cp;
-	MsgLine line;
+static void EN_FPUTC1(int ch,INOUT *io,PCStr(charset),PCStr(encoding))
+{	const unsigned char *cp;
+	/*MsgLine line;*/
+	unsigned CStr(line,MAX_LNSIZE);
 
 	if( charset != io->out_prevCHAR.c_mcharset ){
 
@@ -487,7 +470,7 @@ static EN_FPUTC1(ch,io,charset,encoding)
 				EN_FPUTC0(NL,io);
 				EN_FPUTC0(FOLD_CHAR,io);
 			}
-			sprintf((char*)line,"=?%s?%s?",charset,encoding);
+			Xsprintf(AVStr(line),"=?%s?%s?",charset,encoding);
 			for( cp = line; *cp; cp++ )
 				EN_FPUTC0(*cp,io);
 			io->MIME_ENCODED = 1;
@@ -505,15 +488,13 @@ static EN_FPUTC1(ch,io,charset,encoding)
 		}
 		EN_FPUTC0(ch,io);
 	}
-	io->out_prevCHAR.c_mcharset = charset;
+	io->out_prevCHAR.c_mcharset = (char*)charset;
 	io->out_prevCHAR.c_ch = ch;
 }
 
 #define PENDING_LWSP out_enLWSP
 
-static EN_FPUTC(ch,io,charset,encoding)
-	INOUT *io;
-	char *charset,*encoding;
+static void EN_FPUTC(int ch,INOUT *io,PCStr(charset),PCStr(encoding))
 {	int lwsp;
 
 	if( (ch & 0xFF) == ch )
@@ -542,8 +523,7 @@ static EN_FPUTC(ch,io,charset,encoding)
 /*
  *	PASS THROUGH AN ASCII WORD
  */
-static noencode_word(io)
-	INOUT *io;
+static int noencode_word(INOUT *io)
 {	CHAR *CH,*NCH;
 	int ch,inx;
 	int canbe_folded;
@@ -596,32 +576,34 @@ if( DELIMITER(ch) )
 	}
 	return ch;
 }
-static encode_one(encoding,ins,ilen,outs,osize)
-	char *encoding,*ins,*outs;
+static int encode_one(PCStr(encoding),PCStr(ins),int ilen,PVStr(outs),int osize)
 {	int len;
 
 	if( strcasecmp(encoding,ENCODE_QP) == 0 )
-		len = str_toqp(ins,ilen,outs,osize);
+		len = str_toqp(ins,ilen,AVStr(outs),osize);
 	else
 	if( strcasecmp(encoding,ENCODE_BASE64) == 0 )
-		len = str_to64(ins,ilen,outs,osize,0);
+		len = str_to64(ins,ilen,AVStr(outs),osize,0);
 	else{
 		strncpy(outs,ins,ilen);
 		len = ilen;
 	}
-	outs[len] = 0;
+	setVStrEnd(outs,len);
 	return len;
 }
 
-static encode_word(io)
-	INOUT *io;
-{	char *charset;	/* charset of this encoded-word */
-	char *encoding;	/* encoding of this encoded-word */
-	MsgLine ins,outs;
+static int encode_word(INOUT *io)
+{	const char *charset;	/* charset of this encoded-word */
+	const char *encoding;	/* encoding of this encoded-word */
+	CStr(Ins,MAX_LNSIZE);
+	CStr(Outs,MAX_LNSIZE);
+	refQStr(ins,Ins);
+	refQStr(outs,Outs);
 	int inx,outx,prefold;
 	int char_bytes,nchar,reqlen,remlen,outlen;
 	int nwspchar;
-	char ch,encoded_ch;
+	char encoded_ch;
+	int ch;
 	int prech = 0;
 	int postch = 0;
 	int delop = 0;
@@ -640,8 +622,8 @@ static encode_word(io)
 
 	if( io->ENCODE_EXT ){
 		strcat(ins,in_CODESW_SEQ(io));
-		outlen = encode_one(encoding,ins,strlen(ins),outs,sizeof(outs));
-	}else	outlen = encode_one(encoding,ins,inx,outs,sizeof(outs));
+		outlen = encode_one(encoding,ins,strlen(ins),QVStr(outs,Outs),sizeof(Outs));
+	}else	outlen = encode_one(encoding,ins,inx,QVStr(outs,Outs),sizeof(Outs));
 
 	/*
 	 *	if remaining length is not enough, fold the line
@@ -658,15 +640,15 @@ static encode_word(io)
 	 */
 	nwspchar = 0;
 	for(nchar = 0; ;nchar++){
-		strcpy(&ins[inx],GOTO_ASCII_SEQ);
+		Xstrcpy(QVStr(&ins[inx],Ins),GOTO_ASCII_SEQ);
 		outlen = encode_one(encoding,ins,inx+GOTO_ASCII_SEQ_LEN,
-			 outs,sizeof(outs));
+			 QVStr(outs,Outs),sizeof(Outs));
 
 		CH = EN_FGETC(io);
 		ch = CH->c_ch;
 		if( ch == EOF || ch == NL || CH->c_mcharset != charset ){
 			if( io->in_mcharset == M_US_ASCII )
-				strcpy(&ins[inx],in_CODESW_SEQ(io));
+				Xstrcpy(QVStr(&ins[inx],Ins),in_CODESW_SEQ(io));
 				/* ASCII family like JIS-X0201-Roman */
 
 
@@ -674,7 +656,7 @@ static encode_word(io)
 			/*if( 4 <= (remlen-outlen) )*/
 			if( LWSP_CHAR(ch) ){
 				CHAR *NCH;
-				char lwspb[4];
+				CStr(lwspb,4);
 				int lwspx,cx;
 
 				lwspb[0] = ch;
@@ -683,18 +665,18 @@ static encode_word(io)
 					if( !LWSP_CHAR(NCH->c_ch) )
 						break;
 					if( ENCODE_LWSPS )
-						lwspb[lwspx++] = NCH->c_ch;
+						setVStrElemInc(lwspb,lwspx,NCH->c_ch);  /**/
 				}
 
 				if( NCH->c_mcharset == charset ){
 					EN_UNGETC(NCH,io);
 					inx += strlen(&ins[inx]);
 					for( cx = 0; cx < lwspx; cx++ )
-						ins[inx++] = lwspb[cx];
-					ins[inx] = 0;
+						setVStrElemInc(ins,inx,lwspb[cx]);
+					setVStrEnd(ins,inx);
 					if( 12 <= (remlen-outlen) ){
 /* TO BE CATENATED */
-strcpy(&ins[inx],CODESW_SEQ(NCH->c_bcharset));
+Xstrcpy(QVStr(&ins[inx],Ins),CODESW_SEQ(NCH->c_bcharset));
 inx = strlen(ins);
 nwspchar += lwspx;
 continue;
@@ -720,8 +702,8 @@ break;
 		if( ch == EOF )
 			break;
 
-		ins[inx++] = ch;
-		ins[inx] = 0;
+		setVStrElemInc(ins,inx,ch);
+		setVStrEnd(ins,inx);
 	}
 	inx += strlen(&ins[inx]);
 
@@ -748,7 +730,7 @@ break;
 			delop |= DELSP_POST;
 
 		if( delop ){
-			MsgLine tmp;
+			CStr(tmp,MAX_LNSIZE);
 
 			strcpy(tmp,ins);
 			strcpy(ins,DELSP_SEQ(delop));
@@ -757,7 +739,7 @@ break;
 		}
 	}
 
-	outlen = encode_one(encoding,ins,inx,outs,sizeof(outs));
+	outlen = encode_one(encoding,ins,inx,QVStr(outs,Outs),sizeof(Outs));
 	for(outx = 0; outx < outlen; outx++){
 		encoded_ch = outs[outx];
 		if( encoded_ch == NL )
@@ -770,10 +752,8 @@ break;
  *  shorter than MAXCOL, but will be splitted in current line.  */
 
 
-MIME_headerEncode0(in,out)
-	FILE *in,*out;
-{	char *ip,*op;
-	INOUT iob,*io = &iob;
+int MIME_headerEncode0(FILE *in,FILE *out)
+{	INOUT iob,*io = &iob;
 	CHAR *CH;
 	int ch;
 
@@ -805,10 +785,9 @@ MIME_headerEncode0(in,out)
 		EN_FPUTC(ch,io,M_US_ASCII,ENCODE_NONE);
 	return ch;
 }
-MIME_headerEncode(in,out)
-	FILE *in,*out;
+void MIME_headerEncode(FILE *in,FILE *out)
 {	int ch;
-	MsgLine line;
+	CStr(line,MAX_LNSIZE);
 
 	ch = MIME_headerEncode0(in,out);
 	if( ch != EOF ){
@@ -822,9 +801,7 @@ MIME_headerEncode(in,out)
 /*
  *	FINAL OUTPUT WITH ISO-2022 CHARACTER SET SWITCH SEQUENCE
  */
-static DE_FPUTC1X(CH,Out)
-	CHAR *CH;
-	INOUT *Out;
+static void DE_FPUTC1X(CHAR *CH,INOUT *Out)
 {	FILE *out;
 	int cset;
 	int ch;
@@ -844,11 +821,8 @@ static DE_FPUTC1X(CH,Out)
 	switch( ch ){
 		case EOF:			return;
 		case XC_DE_OFLUSH:		return;
-		case NL: Out->out_nonLWSP = 0;
-			 Out->out_column = 0;	break;
-		default: if( !LWSP_CHAR(ch) )
-				Out->out_nonLWSP++;
-			 Out->out_column++;	break;
+		case NL: Out->out_column = 0;	break;
+		default: Out->out_column++;	break;
 	}
 	NLfputc(ch,out);
 }
@@ -856,8 +830,7 @@ static DE_FPUTC1X(CH,Out)
 /*
  *	PUT ASCII (or CONTROL) CHARACTER IN A CURRENT ASCII FAMILY
  */
-static DE_FPUTC1(ch,io)
-	INOUT *io;
+static void DE_FPUTC1(int ch,INOUT *io)
 {	CHAR CH;
 
 	CH.c_bcharset = io->out_whichASCII;
@@ -865,11 +838,9 @@ static DE_FPUTC1(ch,io)
 	DE_FPUTC1X(&CH,io);
 }
 
-static dumpDECODER(CH,io)
-	CHAR *CH;
-	INOUT *io;
+static void dumpDECODER(CHAR *CH,INOUT *io)
 {	int i,ch;
-	char sym[16];
+	CStr(sym,16);
 
 	switch( ch = CH->c_ch ){
 		case NLNL:		strcpy(sym,"NLNL"); break;
@@ -906,9 +877,7 @@ static dumpDECODER(CH,io)
  */
 #define CLEAR_LWSP(io)	(io->out_deLWSP[0].c_ch = 0)
 
-static DE_FPUTCX(CH,io)
-	CHAR *CH;
-	INOUT *io;
+static void DE_FPUTCX(CHAR *CH,INOUT *io)
 {	int ch;
 	CHAR PCH;
 
@@ -987,7 +956,6 @@ static DE_FPUTCX(CH,io)
 	}
 	if( PCH.c_ch ){
 		if( ch == NL && PCH.c_ch == NL )
-		if( io->out_nonLWSP == 0 ) /* L-W-SP */
 			return;
 		if( ch == NL || LWSP_CHAR(ch) ){
 			if( ch == NL && PCH.c_ch != NL ){
@@ -1022,8 +990,7 @@ static DE_FPUTCX(CH,io)
 /*
  *	PUT ASCII (or CONTROL) CHARACTER IN A CURRENT ASCII FAMILY
  */
-static DE_FPUTC(ch,io)
-	INOUT *io;
+static void DE_FPUTC(int ch,INOUT *io)
 {	CHAR CH;
 
 	CH.c_bcharset = io->out_whichASCII;
@@ -1032,26 +999,33 @@ static DE_FPUTC(ch,io)
 }
 
 
-static scan_eword(in,reads,charset,encoding,text)
-	FILE *in;
-	char *reads,*charset,*encoding,*text;
+static
+int scan_eword(FILE *in,xPVStr(reads),PVStr(charset),PVStr(encoding),PVStr(text))
 {	int i,cs;
 
 	for(i = 0; ;i++){
+		if( 32 <= i ){
+			cs = XC_DE_TOOLONG;
+			goto error;
+		}
 		cs = NLfgetc(in);
 		if(cs==NL || cs==EOF) goto error;
-		*reads++ = cs;
+		setVStrPtrInc(reads,cs); /**/
 		if(cs==CHARSET_DONE) break;
-		charset[i] = cs;
-		charset[i+1] = 0;
+		setVStrElem(charset,i,cs); /**/
+		setVStrEnd(charset,i+1);
 	}
 	for(i = 0; ;i++){
+		if( 32 <= i ){
+			cs = XC_DE_TOOLONG;
+			goto error;
+		}
 		cs = NLfgetc(in);
 		if(cs==NL || cs==EOF) goto error;
-		*reads++ = cs;
+		setVStrPtrInc(reads,cs); /**/
 		if(cs==ENCODING_DONE) break;
-		encoding[i] = cs;
-		encoding[i+1] = 0;
+		setVStrElem(encoding,i,cs); /**/
+		setVStrEnd(encoding,i+1);
 	}
 	for(i = 0;;i++){
 		if( 80 <= i ){
@@ -1060,30 +1034,28 @@ static scan_eword(in,reads,charset,encoding,text)
 		}
 		cs = NLfgetc(in);
 		if(cs==NL || cs==EOF) goto error;
-		*reads++ = cs;
+		setVStrPtrInc(reads,cs); /**/
 		if(cs == ENCODE_DONE[0]){
 			cs = NLfgetc(in);
 			if(cs==NL || cs==EOF) goto error;
-			*reads++ = cs;
+			setVStrPtrInc(reads,cs); /**/
 			if( cs == ENCODE_DONE[1] ){
-				text[i] = 0;
+				setVStrEnd(text,i);
 				break;
 			}
 			ungetc(cs,in);
 			cs = ENCODE_DONE[0];
 		}
-		text[i] = cs;
-		text[i+1] = 0;
+		setVStrElem(text,i,cs); /**/
+		setVStrEnd(text,i+1);
 	}
 	return 0;
 error:
-	*reads = 0;
+	setVStrEnd(reads,0);
 	return cs;
 }
 
-static disp_word(Out,dtext,len)
-	INOUT *Out;
-	char *dtext;
+static int disp_word(INOUT *Out,PCStr(dtext),int len)
 {	FILE *DecodedText;
 	INOUT tmpInb,*tmpIn = &tmpInb;
 	int sdlen,dlen;
@@ -1096,7 +1068,7 @@ static disp_word(Out,dtext,len)
 	if( Out )
 		sdlen = disp_word(0,dtext,len);
 
-	DecodedText = str_fopen(dtext,len,"r");
+	DecodedText = str_fopen((char*)dtext,len,"r");
 	INOUT_init(tmpIn,DecodedText,NULL);
 
 	dlen = 0;
@@ -1142,14 +1114,16 @@ static disp_word(Out,dtext,len)
 	return dlen;
 }
 
-static decode_word(io)
-	INOUT *io;
-{	MsgLine reads,charset,encoding,itext,dtext;
+int bad_etext(PCStr(charset),PCStr(dtext),int len);
+static int decode_word(INOUT *io)
+{	MsgLine reads,charset,encoding,itext;
+	CStr(dtext,MAX_LNSIZE);
 	int ilen,dsize,len,pad,dlen;
 	int eow;
 
 	*charset = *encoding = *itext = 0;
-	eow = scan_eword(io->in_file,reads,charset,encoding,itext);
+
+	eow = scan_eword(io->in_file,FVStr(reads),FVStr(charset),FVStr(encoding),FVStr(itext));
 
 	if( eow == NL || eow == EOF || eow == XC_DE_TOOLONG ){
 		DE_FPUTC(XC_DE_OFLUSH,io);
@@ -1159,7 +1133,7 @@ static decode_word(io)
 		return eow;
 	}
 
-	if( !MIME_localCharset(charset) ){
+	if( !MIME_localCharset((char*)charset) ){
 BAD_EWORD:
 		DE_FPUTC(XC_DE_OFLUSH,io);
 		fprintf(io->out_file,"=?%s?%s?%s?=",charset,encoding,itext);
@@ -1168,27 +1142,26 @@ BAD_EWORD:
 		return 0;
 	}
 
-	ilen = strlen(itext);
+	ilen = strlen((char*)itext);
 	dsize = sizeof(dtext);
-	if( strcasecmp(encoding,ENCODE_QP) == 0 ){
-		len = str_fromqp(itext,ilen,dtext,dsize);
-		if( bad_etext(charset,dtext,len) )
+	if( strcasecmp((char*)encoding,ENCODE_QP) == 0 ){
+		len = str_fromqp((char*)itext,ilen,AVStr(dtext),dsize);
+		if( bad_etext((char*)charset,dtext,len) )
 			goto BAD_EWORD;
 	}else
-	if( strcasecmp(encoding,ENCODE_BASE64) == 0 ){
-		len = str_from64(itext,ilen,dtext,dsize);
-		if( bad_etext(charset,dtext,len) )
+	if( strcasecmp((char*)encoding,ENCODE_BASE64) == 0 ){
+		len = str_from64((char*)itext,ilen,AVStr(dtext),dsize);
+		if( bad_etext((char*)charset,dtext,len) )
 			goto BAD_EWORD;
 	}else{
-		strcpy(dtext,itext);
+		strcpy(dtext,(char*)itext);
 		len = ilen;
 	}
 	disp_word(io,dtext,len);
 	return 0;
 }
 
-bad_etext(charset,dtext,len)
-	char *charset,*dtext;
+int bad_etext(PCStr(charset),PCStr(dtext),int len)
 {	int ci,ch;
 
 	if( strcasecmp(charset,M_ISO_2022_JP) == 0 ){
@@ -1199,14 +1172,13 @@ bad_etext(charset,dtext,len)
 			if( ch != 0xE && ch != 0xF ) /* SO, SI */
 				return 1;
 		}
-		if( FIX_2022(dtext,NULL,"text/plain") )
+		if( FIX_2022(dtext,VStrNULL,"text/plain") )
 			return 1;
 	}
 	return 0;
 }
 
-static nodecode_word(io,ch)
-	INOUT *io;
+static void nodecode_word(INOUT *io,int ch)
 {
 	if( io->MIME_ENCODED ){
 		/* if the next noencoded-word ends before DISPCOLS ...*/
@@ -1231,8 +1203,7 @@ static nodecode_word(io,ch)
 	DE_FPUTC(ch,io);
 }
 
-static DE_FGETC(io)
-	INOUT *io;
+static int DE_FGETC(INOUT *io)
 {	FILE *in;
 	int ch;
 
@@ -1265,8 +1236,7 @@ static DE_FGETC(io)
 	io->in_prevch = ch;
 	return ch;
 }
-MIME_headerDecode(in,out,bodytoo)
-	FILE *in,*out;
+void MIME_headerDecode(FILE *in,FILE *out,int bodytoo)
 {	int ch,prev_ch,next_ch;
 	INOUT iob,*io = &iob;
 
@@ -1292,6 +1262,8 @@ MIME_headerDecode(in,out,bodytoo)
 			}
 		}else{
 			if( ch == NLNL ){
+				/* real fix for 1.8 problem */
+				DE_FPUTC(XC_DE_OFLUSH,io); /* flush LWSP buf. */
 				io->UNFOLD_LINE = 0;
 				DE_FPUTC(NL,io);
 				DE_FPUTC(NL,io);
@@ -1312,36 +1284,33 @@ MIME_headerDecode(in,out,bodytoo)
 	DE_FPUTC(EOF,io);
 }
 
-MIME_strHeaderDecode(ins,outs,osize)
-	char *ins,*outs;
+void MIME_strHeaderDecode(PCStr(ins),PVStr(outs),int osize)
 {	FILE *In,*Out;
 	int oi;
 
-	In = str_fopen(ins,strlen(ins),"r");
-	Out = str_fopen(outs,osize,"w");
+	In = str_fopen((char*)ins,strlen(ins),"r");
+	Out = str_fopen((char*)outs,osize,"w");
 	MIME_headerDecode(In,Out,1);
 	fflush(Out);
 	for(oi = 0; outs[oi]; oi++)
 		if((outs[oi] & 0xFF) == 0xFF)
-			strcpy(&outs[oi],&outs[oi+1]);
+			ovstrcpy((char*)&outs[oi],&outs[oi+1]);
 	str_fclose(In);
 	str_fclose(Out);
 }
-MIME_strHeaderEncode(ins,outs,osize)
-	char *ins,*outs;
+void MIME_strHeaderEncode(PCStr(ins),PVStr(outs),int osize)
 {	FILE *In,*Out;
 
-	In = str_fopen(ins,strlen(ins),"r");
-	Out = str_fopen(outs,osize,"w");
+	In = str_fopen((char*)ins,strlen(ins),"r");
+	Out = str_fopen((char*)outs,osize,"w");
 	MIME_headerEncode(In,Out);
 	fflush(Out);
 	str_fclose(In);
 	str_fclose(Out);
 }
 
-is_MIME_header(fp)
-	FILE *fp;
-{	MsgLine line;
+int is_MIME_header(FILE *fp)
+{	CStr(line,MAX_LNSIZE);
 	int off;
 
 	off = ftell(fp);
@@ -1360,9 +1329,7 @@ is_MIME_header(fp)
 	return 0;
 }
 
-FILE *
-MIME_tmpHeaderDecode(fp,bodytoo)
-	FILE *fp;
+FILE *MIME_tmpHeaderDecode(FILE *fp,int bodytoo)
 {	FILE *tfp;
 
 	if( fp == NULL )
@@ -1379,15 +1346,13 @@ MIME_tmpHeaderDecode(fp,bodytoo)
 	fseek(tfp,0,0);
 	return tfp;
 }
-FILE *
-MIME_tmpHeaderEncode(fp,savFILE)
-	FILE *fp,savFILE;
+FILE *MIME_tmpHeaderEncode(FILE *fp)
 {	FILE *tin,*tfp;
-	MsgLine line;
+	CStr(line,MAX_LNSIZE);
 	int ch;
 
 	if( fp == NULL )
-		return;
+		return NULL;
 	tin = tmpfile();
 	while( fgets(line,sizeof(line),fp) != NULL ){
 		fputs(line,tin);
@@ -1413,14 +1378,13 @@ MIME_tmpHeaderEncode(fp,savFILE)
 }
 
 /*/////////////////////////////////////////////////////////////////////#*/
-MIME_localStrColumns(str)
-	char *str;
+int MIME_localStrColumns(PCStr(str))
 {	INOUT iob,*io = &iob;
 	FILE *sfp;
 	int len;
 	CHAR *CH;
 
-	sfp = str_fopen(str,strlen(str),"r");
+	sfp = str_fopen((char*)str,strlen(str),"r");
 	INOUT_init(io,sfp,NULL);
 
 	len = 0;
