@@ -19,21 +19,28 @@ History:
 	951029	extracted from DelaGate
 //////////////////////////////////////////////////////////////////////#*/
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-extern FILE *tmpfile();
+#include <fcntl.h>
 #include "ystring.h"
 #include <ctype.h>
 #include <errno.h>
-extern char *getenv();
+#include <time.h>
+#include "file.h"
+
+int getNonblockingIO(int fd);
+int setCloseOnExec(int fd);
+int clearCloseOnExec(int fd);
+
 
 #ifdef S_ISUID /* set UID on execution */
 #define IS_UNIX_FS
-FS_maybeUnix(){ return 1; }
-FS_withSetuid(){ return 1; }
+int FS_maybeUnix(){ return 1; }
+int FS_withSetuid(){ return 1; }
 #else
-FS_maybeUnix(){ return 0; }
-FS_withSetuid(){ return 0; }
+int FS_maybeUnix(){ return 0; }
+int FS_withSetuid(){ return 0; }
 #endif
 
 #if defined(_MSC_VER) || defined(__CYGWIN__) || defined(__EMX__)
@@ -43,9 +50,9 @@ int FS_dosPath = 0;
 #endif
 
 #ifdef S_IFLNK
-FS_withSymlink(){ return 1; }
+int FS_withSymlink(){ return 1; }
 #else
-FS_withSymlink(){ return 0; }
+int FS_withSymlink(){ return 0; }
 #endif
 
 #ifndef S_ISREG
@@ -56,9 +63,7 @@ FS_withSymlink(){ return 0; }
 #define S_ISDIR(m)  (((m)&S_IFMT) == S_IFDIR)
 #endif
 
-int File_stats(path,link,mode,uid,gid,size,mtime)
-	char *path;
-	int *mode,*uid,*gid,*size,*mtime;
+int File_stats(PCStr(path),int link,int *mode,int *uid,int *gid,int *size,int *mtime)
 {	struct stat st;
 	int code;
 
@@ -74,61 +79,81 @@ int File_stats(path,link,mode,uid,gid,size,mtime)
 	}
 	return code;
 }
-int File_is(path)
-	char *path;
+int File_ident(PCStr(path),PVStr(ident))
+{	struct stat st;
+
+	if( stat(path,&st) == 0 ){
+		sprintf(ident,"%x:%x:%x",st.st_dev,st.st_ino,st.st_ctime);
+		return 0;
+	}
+	setVStrEnd(ident,0);
+	return -1;
+}
+int File_is(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return 1;
 	else	return 0;
 }
-int File_isreg(path)
-	char *path;
+int File_isreg(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return S_ISREG(st.st_mode);
 	else	return 0;
 }
-int File_mtime(path)
-	char *path;
+int File_ctime(PCStr(path))
+{	struct stat st;
+
+	if( stat(path,&st) == 0 )
+		return st.st_ctime;
+	else	return -1;
+}
+int File_mtime(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_mtime;
 	else	return -1;
 }
-int File_size(path)
-	char *path;
+int File_size(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_size;
 	else	return -1;
 }
-int File_uid(path)
-	char *path;
+int File_uid(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_uid;
 	else	return 0;
 }
-int File_ino(path)
-	char *path;
+int File_ino(PCStr(path))
 {	struct stat st;
 
 	if( stat(path,&st) == 0 )
 		return st.st_ino;
 	else	return 0;
 }
-File_cmp(path1,path2)
-	char *path1,*path2;
+int file_cmp(int fd1,int fd2)
 {	struct stat st1,st2;
-	static struct stat st0;
 
-	st1 = st0;
-	st2 = st0;
+	bzero(&st1,sizeof(st1));
+	bzero(&st2,sizeof(st2));
+	if( fstat(fd1,&st1) != 0 ) return -1;
+	if( fstat(fd2,&st2) != 0 ) return -1;
+	if( bcmp(&st1,&st2,sizeof(st1)) == 0 )
+		return 0;
+	else	return 1;
+}
+int File_cmp(PCStr(path1),PCStr(path2))
+{	struct stat st1,st2;
+
+	bzero(&st1,sizeof(st1));
+	bzero(&st2,sizeof(st2));
 	if( stat(path1,&st1) != 0 ) return -1;
 	if( stat(path2,&st2) != 0 ) return -1;
 
@@ -137,6 +162,8 @@ File_cmp(path1,path2)
  * (/net/host/path != /net/alias/path)
  */
 	if( st1.st_dev == st2.st_dev &&
+	    st1.st_mtime == st2.st_mtime &&
+	    st1.st_size == st2.st_size &&
 	    st1.st_ino == st2.st_ino )
 		return 0;
 	else	return 1;
@@ -145,9 +172,9 @@ return bcmp(&st1,&st2,sizeof(struct stat));
 */
 }
 
-fcompare(fp1,fp2)
-	FILE *fp1,*fp2;
-{	char buf1[1024],buf2[1024];
+int fcompare(FILE *fp1,FILE *fp2)
+{	CStr(buf1,1024);
+	CStr(buf2,1024);
 	int off1,off2;
 	int rc1,rc2,ri;
 	int diff;
@@ -156,8 +183,8 @@ fcompare(fp1,fp2)
 	off2 = ftell(fp2);
 	diff = 0;
 	for(;;){
-		rc1 = fread(buf1,1,1024,fp1);
-		rc2 = fread(buf2,1,1024,fp2);
+		rc1 = fread(buf1,1,sizeof(buf1),fp1);
+		rc2 = fread(buf2,1,sizeof(buf2),fp2);
 		if( rc1 != rc2 ){
 			diff = 1;
 			break;
@@ -175,8 +202,7 @@ fcompare(fp1,fp2)
 	return diff;
 }
 
-Ftruncate(fp,offset,whence)
-	FILE *fp;
+int Ftruncate(FILE *fp,int offset,int whence)
 {	long savoff;
 	int rcode;
 
@@ -186,11 +212,14 @@ Ftruncate(fp,offset,whence)
 	fseek(fp,savoff,0);
 	return rcode;
 }
+int Lseek(int fd,int off,int wh)
+{
+	return lseek(fd,(off_t)off,wh);
+}
 
-extern char *getcwd();
-static char *_pathsep;
-static char *_PATHSEP(){
-	char cwd[1024];
+static const char *_pathsep;
+static const char *_PATHSEP(){
+	CStr(cwd,1024);
 	if( _pathsep == NULL ){
 		getcwd(cwd,sizeof(cwd));
 		if( strchr(cwd,'\\') )
@@ -200,9 +229,8 @@ static char *_PATHSEP(){
 	return _pathsep;
 }
 
-FullpathOfExe(path)
-	char *path;
-{	char xpath[1024];
+int FullpathOfExe(PVStr(path))
+{	CStr(xpath,1024);
 
 	if( isFullpath(path) )
 		return 1;
@@ -228,9 +256,9 @@ FullpathOfExe(path)
 }
 
 static int colonForDrive;
-ColonForDrive()
+int ColonForDrive()
 {	FILE *fp;
-	char path[1024];
+	CStr(path,1024);
 
 	if( colonForDrive == 0 ){
 		/* cannot use ':' in a file name if on Windows */
@@ -239,10 +267,19 @@ ColonForDrive()
 		if( isalpha(path[0]) && path[1] == ':' )
 			colonForDrive =  1;
 		else{
-			sprintf(path,"/tmp/%d:%d",time(0),getpid());
+			/*
+			const char *dir = "/share/dgroot/tmp";
+			sprintf(path,"%s/%d:%d",dir,time(NULL),getpid());
+			*/
+			sprintf(path,"/tmp/%d:%d",time(NULL),getpid());
 			if( fp = fopen(path,"w") ){
 				fclose(fp);
 				unlink(path);
+				/*
+				if( file_matches(dir,":") == 0 )
+					colonForDrive = 1;
+				else
+				*/
 				colonForDrive = -1;
 			}else	colonForDrive =  1;
 		}
@@ -250,8 +287,7 @@ ColonForDrive()
 	return 0 < colonForDrive;
 }
 
-char *fullPath(path,buff)
-	char *path,*buff;
+char *fullPath(PCStr(path),PVStr(buff))
 {
 #ifndef IS_UNIX_FS
 	/*
@@ -261,16 +297,15 @@ char *fullPath(path,buff)
 	 && 'A' <= path[2] && path[2] <= 'Z'
 	 && path[3] == '/' )
 	{
-		buff[0] = path[2];
-		buff[1] = ':';
-		strcpy(&buff[2],&path[3]);
+		setVStrElem(buff,0,path[2]);
+		setVStrElem(buff,1,':');
+		Xstrcpy(DVStr(buff,2),&path[3]);
 		path = buff;
 	}
 #endif
-	return path;
+	return (char*)path;
 }
-isFullpath(path)
-	char *path;
+int isFullpath(PCStr(path))
 {
 	if( FS_dosPath || *_PATHSEP() == '\\' ){
 		if( isalpha(path[0]) && path[1] == ':' ){
@@ -285,8 +320,7 @@ isFullpath(path)
 		return 1;
 	return 0;
 }
-isBoundpath(path)
-	char *path;
+int isBoundpath(PCStr(path))
 {
 	if( isFullpath(path) )
 		return 1;
@@ -300,20 +334,22 @@ isBoundpath(path)
 	}
 	return 0;
 }
-chdir_cwd(cwd, go, userdir)
-	char *cwd,*go;
-{	char *tp,*cp;
+void chdir_cwd(PVStr(cwd),PCStr(go),int userdir)
+{	refQStr(tp,cwd); /**/
+	const char *cp;
 	int uplen,len;
 
-	for( tp = cwd; *tp; tp++ ){}
+	for( cpyQStr(tp,cwd); *tp; tp++ ){}
 	if( len = isFullpath(go) ){
-		tp = cwd;
-		while( 0 < len-- )
-			*tp++ = *go++;
+		tp = (char*)cwd;
+		while( 0 < len-- ){
+			assertVStr(cwd,tp+1);
+			setVStrPtrInc(tp,*go++);
+		}
 	}else
 	if( userdir && go[0] == '~' ){
-		if( go[1] == 0   ){ tp = cwd; go += 1; }else
-		if( go[1] == '/' ){ tp = cwd; go += 2; }
+		if( go[1] == 0   ){ tp = (char*)cwd; go += 1; }else
+		if( go[1] == '/' ){ tp = (char*)cwd; go += 2; }
 	}
 
 	while( *go ){
@@ -333,12 +369,12 @@ chdir_cwd(cwd, go, userdir)
 						tp--;
 					else	break;
 				}
-				*tp = 0;
+				setVStrEnd(tp,0);
 				if( cp = strrchr(cwd,'/') ){
 					if( cp == cwd  )
-						tp = cp + 1;
-					else	tp = cp;
-				}else	tp = cwd;
+						tp = (char*)cp + 1;
+					else	tp = (char*)cp;
+				}else	tp = (char*)cwd;
 			}
 			go += uplen;
 		}else{
@@ -346,8 +382,9 @@ chdir_cwd(cwd, go, userdir)
 				len = (cp - go) + 1;
 			else	len = strlen(go);
 
-			if( cwd < tp && tp[-1] != '/' )
-				*tp++ = '/';
+			if( cwd < tp && tp[-1] != '/' ){
+				setVStrPtrInc(tp,'/');
+			}
 			strncpy(tp,go,len);
 			tp += len;
 			go += len;
@@ -358,72 +395,71 @@ chdir_cwd(cwd, go, userdir)
 			tp--;
 		else	break;
 	}
-	*tp = 0;
+	setVStrEnd(tp,0);
 }
 
 static int _TMPSEQ;
 static int _TMPDIR_set;
 #ifdef IS_UNIX_FS
-static char *_TMPDIR;
+static const char *_TMPDIR;
 #else
-static char *_TMPDIR = "\\tmp";
+static const char *_TMPDIR = "\\tmp";
 #endif
 
-char *getTMPDIR(){ return _TMPDIR; }
+const char *getTMPDIR(){ return _TMPDIR; }
 
 static void *shared_frex;
-extern void *frex_append();
 
-static normpath(src,dst)
-	char *src,*dst;
-{	char *dp,*sp;
+static void normpath(PCStr(src),PVStr(dst))
+{	refQStr(dp,dst); /**/
+	const char *sp;
+	int ch;
 
-	dp = dst;
-	for( sp = src; *dp++ = *sp; sp++ ){
+	for( sp = src; ; sp++ ){
+		ch = *sp;
+		setVStrPtrInc(dp,ch);
+		if( ch == 0 )
+			break;
+		assertVStr(dst,dp+1);
 		if( *sp == '/' && sp[1] == '/' )
 			sp++;
 	}
-	*dp++ = 0;
+	setVStrPtrInc(dp,0);
 }
 
-setSHARE(pathpat)
-	char *pathpat;
-{	char xpathpat[1024];
+int setSHARE(PCStr(pathpat))
+{	CStr(xpathpat,1024);
 
-	normpath(pathpat,xpathpat);
-	shared_frex = frex_append(shared_frex,xpathpat);
+	normpath(pathpat,AVStr(xpathpat));
+	shared_frex = frex_append((struct fa_stat*)shared_frex,xpathpat);
 	return 0;
 }
-static beshared(path,isdir)
-	char *path;
+static int beshared(PCStr(path),int isdir)
 {	int match;
-	char xpath[1024];
+	CStr(xpath,1024);
 
 	if( shared_frex == 0 )
 		return 0;
 
-	normpath(path,xpath);
+	normpath(path,AVStr(xpath));
 	if( isdir && strtailchr(xpath) != '/' )
 		strcat(xpath,"/");
 
-	match = frex_match(shared_frex,xpath);
+	match = frex_match((struct fa_stat*)shared_frex,xpath) != 0;
 	return match;
 }
 
-static char *OWNER_DIR;
+static const char *OWNER_DIR;
 static int OWNER_UID = -1;
 static int OWNER_GID = -1;
 
-setOWNER(dir,uid,gid)
-	char *dir;
+void setOWNER(PCStr(dir),int uid,int gid)
 {
-	OWNER_DIR = strdup(dir); 
+	OWNER_DIR = stralloc(dir); 
 	OWNER_UID = uid;
 	OWNER_GID = gid;
 }
-static bemine(path,uidp,gidp)
-	char *path;
-	int *uidp,*gidp;
+static int bemine(PCStr(path),int *uidp,int *gidp)
 {
 	if( OWNER_DIR == 0 || OWNER_UID < 0 )
 		return 0;
@@ -435,8 +471,7 @@ static bemine(path,uidp,gidp)
 	}
 	return 0;
 }
-chmodShared(path)
-	char *path;
+void chmodShared(PCStr(path))
 {	int uid,gid,isdir;
 
 	if( bemine(path,&uid,&gid) )
@@ -448,8 +483,7 @@ chmodShared(path)
 		else	chmod(path,0666);
 	}
 }
-FILE *fopenShared(path,mode)
-	char *path,*mode;
+FILE *fopenShared(PCStr(path),PCStr(mode))
 {	FILE *fp;
 	int uid,gid;
 	int mine;
@@ -467,9 +501,8 @@ FILE *fopenShared(path,mode)
 		chmodShared(path);
 	return fp;
 }
-extern FILE *fopentmpfile();
-FILE *fopentmpfileShared(path,remove)
-	char *path;
+extern FILE *fopentmpfile(const char*,int);
+FILE *fopentmpfileShared(PCStr(path),int remove)
 {	FILE *fp;
 	int mine,uid,gid;
 
@@ -484,8 +517,7 @@ FILE *fopentmpfileShared(path,remove)
 	}
 	return fp;
 }
-mkdirShared(dir,mode)
-	char *dir;
+int mkdirShared(PCStr(dir),int mode)
 {	int uid,gid,rcode;
 	int share,mine;
 
@@ -507,8 +539,7 @@ mkdirShared(dir,mode)
 		chmod(dir,0777);
 	return rcode;
 }
-chmodIfShared(file,mode)
-	char *file;
+int chmodIfShared(PCStr(file),int mode)
 {	int uid,gid;
 
 	if( bemine(file,&uid,&gid) )
@@ -518,21 +549,21 @@ chmodIfShared(file,mode)
 	return chmod(file,mode);
 }
 
-setTMPDIR(dir)
-	char *dir;
-{	char buff[1024];
+void setTMPDIR(PCStr(dir))
+{	CStr(buff,1024);
 
 	if( dir ){
 		if( _TMPDIR && _TMPDIR_set )
-			free(_TMPDIR);
-		_TMPDIR = strdup(fullPath(dir,buff));
+			free((char*)_TMPDIR);
+		_TMPDIR = stralloc(fullPath(dir,AVStr(buff)));
 		_TMPDIR_set = 1;
 	}
 }
 
-FILE *TMPFILEX(path)
-	char *path;
-{	char *tmpdir,pathb[1024],buff[1024];
+FILE *TMPFILEXX(PCStr(what),xPVStr(path))
+{	const char *tmpdir;
+	CStr(pathb,1024);
+	CStr(buff,1024);
 	FILE *fp;
 
 	tmpdir = _TMPDIR;
@@ -546,29 +577,32 @@ FILE *TMPFILEX(path)
 		mkdirShared(tmpdir,0);
 
 	if( path == NULL )
-		path = pathb;
-	sprintf(path,"%s/dg%d.%d.%d",tmpdir,getpid(),++_TMPSEQ,time(0));
+		setPStr(path,pathb,sizeof(pathb));
+	sprintf(path,"%s/dg%d.%d.%d",tmpdir,getpid(),++_TMPSEQ,time(NULL));
 
 	errno = 0;
-	fp = fopentmpfileShared(fullPath(path,buff),path==pathb);
+	fp = fopentmpfileShared(fullPath(path,AVStr(buff)),path==pathb);
 
 	if( fp != NULL )
-		syslog_DEBUG("TMPFILE = (%d) %s\n",fileno(fp),path);
-	else	syslog_ERROR("TMPFILE: cannot create (%d) = %s\n",errno,path);
+		syslog_DEBUG("TMPFILE(%s) = (%d) %s\n",what,fileno(fp),path);
+	else	syslog_ERROR("TMPFILE(%s): cannot create (%d) = %s\n",what,errno,path);
 	return fp;
 }
+FILE *TMPFILEX(PVStr(path))
+{
+	return TMPFILEXX("*",AVStr(path));
+}
 
 #ifndef EMFILE
 #define EMFILE -1
 #endif
 
-FILE *TMPFILE(what)
-	char *what;
+FILE *TMPFILE(PCStr(what))
 {	FILE *fp;
 	int besilent;
-	char *tmpdir;
+	const char *tmpdir;
 
-	fp = TMPFILEX(NULL);
+	fp = TMPFILEXX(what,VStrNULL);
 	if( fp == NULL )
 		fp = tmpfile();
 
@@ -596,9 +630,7 @@ FILE *TMPFILE(what)
 
 static int tmpfiles[1] = {-1};
 static int tmpfilepids[1];
-FILE *reusableTMPFILE(what,where)
-	char *what;
-	int (*where)();
+FILE *reusableTMPFILE(PCStr(what),iFUNCP where)
 {	FILE *tmp;
 	int tx,fd,pid;
 
@@ -607,8 +639,8 @@ FILE *reusableTMPFILE(what,where)
 	pid = getpid();
 
 	if( 0 <= fd && tmpfilepids[0] == pid ){
-		lseek(fd,0,0);
 		tmp = fdopen(dup(fd),"w+");
+		fseek(tmp,0,0);
 		Ftruncate(tmp,0,0);
 		syslog_DEBUG(">>>TMPFILE(%s) reused [%d]->[%d]\n",what,
 			fd,fileno(tmp));
@@ -642,9 +674,26 @@ FILE *NULLFP(){
 	return NULLFP;
 }
 
-copyfile2(sfp,dfp,doflush)
-	FILE *sfp,*dfp;
-{	char buff[8*1024];
+int openNull(int rw){
+	int xfd;
+	FILE *xfp;
+
+	xfd = open("/dev/null",rw);
+	if( 0 <= xfd )
+		return xfd;
+
+	xfp = TMPFILE("openNull");
+	if( xfp != NULL ){
+		xfd = dup(fileno(xfp));
+		fclose(xfp);
+		clearCloseOnExec(xfd);
+		return xfd;
+	}
+	return -1;
+}
+
+int copyfile2(FILE *sfp,FILE *dfp,int doflush)
+{	CStr(buff,8*1024);
 	int totalc,rcc;
 
 	totalc = 0;
@@ -661,34 +710,17 @@ copyfile2(sfp,dfp,doflush)
 	return totalc;
 }
 
-copyfile1(sfp,dfp)
-	FILE *sfp,*dfp;
+int copyfile1(FILE *sfp,FILE *dfp)
 {
 	return copyfile2(sfp,dfp,1);
 }
 
 
-extern char *malloc();
-extern char *realloc();
-
-char *Malloc(ptr,size)
-	char *ptr;
-{	char *nptr;
-
-	if( ptr == NULL )
-		nptr = malloc(size);
-	else	nptr = realloc(ptr,size);
-	if( nptr == NULL ){
-		fprintf(stderr,"[%d] DeleGate: Malloc() failed\n",getpid());
-		exit(1);
-	}
-	return nptr;
-}
-
-path_escchar(path)
-	char *path;
-{	char sc,*sp,*dp;
-	char *buff;
+void path_escchar(xPVStr(path))
+{	char sc;
+	const char *sp;
+	refQStr(dp,path); /**/
+	const char *buff;
 
 	if( !ColonForDrive() )
 		return;
@@ -699,60 +731,65 @@ path_escchar(path)
 	if( strpbrk(path,":") == NULL && strstr(path+2,"./") == NULL )
 		return;
 
-	buff = strdup(path);
-	dp = path;
+	buff = stralloc(path);
 	for( sp = buff; sc = *sp; sp++ ){
+		assertVStr(path,dp+3);
 		if( sc == '.' && sp[1] == '/' ){
-			*dp++ = '%';
-			*dp++ = '2';
-			*dp++ = 'E';
+			setVStrPtrInc(dp,'%');
+			setVStrPtrInc(dp,'2');
+			setVStrPtrInc(dp,'E');
 		}else
 		if( sc == ':' ){
 			if( sp[1] == '/' ){
-				*dp++ = '%';
-				*dp++ = '3';
-				*dp++ = 'A';
+				setVStrPtrInc(dp,'%');
+				setVStrPtrInc(dp,'3');
+				setVStrPtrInc(dp,'A');
 			}else{
-				*dp++ = '.';
-				*dp++ = '.';
+				setVStrPtrInc(dp,'.');
+				setVStrPtrInc(dp,'.');
 			}
-		}else	*dp++ = sc;
+		}else	setVStrPtrInc(dp,sc);
 	}
-	*dp = 0;
-	free(buff);
+	setVStrEnd(dp,0);
+	free((char*)buff);
 }
 
-int file_isreg(fd)
+int file_is(int fd)
+{	struct stat stat;
+
+	if( fstat(fd,&stat) == 0 )
+		return 1;
+	else	return 0;
+}
+int file_isreg(int fd)
 {	struct stat status;
 
 	if( fstat(fd,&status) == 0 )
 		return S_ISREG(status.st_mode);
 	return 0;
 }
-int file_isregular(fd)
+int file_isregular(int fd)
 {	struct stat st;
 
 	if( fstat(fd,&st) == 0 )
 		return S_ISREG(st.st_mode) || S_ISDIR(st.st_mode);
 	else	return 0;
 }
-int file_isdir(fd)
+int file_isdir(int fd)
 {	struct stat stat;
 
 	if( fstat(fd,&stat) == 0 )
 		return S_ISDIR(stat.st_mode);
 	return 0;
 }
-int fileIsdir(path)
-	char *path;
+int fileIsdir(PCStr(path))
 {	struct stat status;
 
 	if( stat(path,&status) == 0 )
 		return S_ISDIR(status.st_mode);
 	return 0;
 }
-int fileIsflat(path)
-	char *path;
+int fileIsflat(PCStr(path))
 {	struct stat status;
 
 	if( stat(path,&status) == 0 )
@@ -760,32 +797,29 @@ int fileIsflat(path)
 	return 0;
 }
 
-char *freadfile(fp,sizep)
-	FILE *fp;
-	int *sizep;
+char *freadfile(FILE *fp,int *sizep)
 {	FILE *tmp;
-	char *data;
+	const char *data;
 	int size,rcc;
 
 	if( size = *sizep ){
-		data = malloc(size+1);
-		rcc = fread(data,1,size,fp);
+		data = (char*)malloc(size+1);
+		rcc = fread((char*)data,1,size,fp);
 	}else{
 		tmp = TMPFILE("freadfile");
 		size = copyfile1(fp,tmp);
 		fflush(tmp);
 		fseek(tmp,0,0);
-		data = malloc(size+1);
-		rcc = fread(data,1,size,tmp);
+		data = (char*)malloc(size+1);
+		rcc = fread((char*)data,1,size,tmp);
 		fclose(tmp);
 	}
-	data[rcc] = 0;
+	((char*)data)[rcc] = 0; /**/
 	*sizep = rcc;
-	return data;
+	return (char*)data;
 }
 
-ftouch(fp,time)
-	FILE *fp;
+void ftouch(FILE *fp,int time)
 {	int off;
 
 	off = ftell(fp);
@@ -796,8 +830,7 @@ ftouch(fp,time)
 	Ftruncate(fp,0,1);
 	fseek(fp,off,0);
 }
-File_touch(path,clock)
-	char *path;
+void File_touch(PCStr(path),int clock)
 {	FILE *fp;
 
 	if( fp = fopen(path,"a") ){
@@ -805,38 +838,33 @@ File_touch(path,clock)
 		fclose(fp);
 	}
 }
-char *Fgets(str,size,fp)
-	char *str;
-	FILE *fp;
-{	char *rstr;
-	char *dp;
+char *Fgets(PVStr(str),int size,FILE *fp)
+{	const char *rstr;
+	const char *dp;
 
 	rstr = fgets(str,size,fp);
 	if( rstr != NULL )
 		if( dp = strpbrk(str,"\r\n") )
-			*dp = 0;
-	return rstr;
+			truncVStr(dp);
+	return (char*)rstr;
 }
-Fputs(str,fp)
-	char *str;
-	FILE *fp;
-{	char *dp;
+int Fputs(PCStr(str),FILE *fp)
+{	const char *dp;
 	int rcode;
+	int len,rcc;
 
 	/* signal(SIGPIPE,... */
 
-	if( strchr(str,'\r') == 0 ){
-		if( dp = strchr(str,'\n') )
-			strcpy(dp,"\r\n");
-		else	strcat(str,"\r\n");
-	}
+	if( (dp = strtailstr(str,"\r\n")) || (dp = strtailstr(str,"\n")) )
+		len = dp - str;
+	else	len = strlen(str);
 
-	rcode = fputs(str,fp);
+	rcc = fwrite(str,1,len,fp);
+	rcode = fputs("\r\n",fp);
 	fflush(fp);
 	return rcode;
 }
-Write(fd,buf,len)
-	char *buf;
+int Write(int fd,PCStr(buf),int len)
 {	int nw,wcc,wc1;
 
 	nw = 0;
@@ -848,9 +876,7 @@ Write(fd,buf,len)
 	}
 	return wcc;
 }
-Fwrite(buf,esize,ecount,fp)
-	char *buf;
-	FILE *fp;
+int Fwrite(PCStr(buf),int esize,int ecount,FILE *fp)
 {	int re,we;
 	int fd;
 
@@ -872,10 +898,9 @@ Fwrite(buf,esize,ecount,fp)
 
 static int tmpFileFd = -1;
 static int tmpFilePid = -1;
-static char *tmpFileWhat = "";
+static const char *tmpFileWhat = "";
 
-freeTmpFile(what)
-	char *what;
+void freeTmpFile(PCStr(what))
 {
 	if( 0 <= tmpFileFd ){
 		close(tmpFileFd);
@@ -884,11 +909,10 @@ freeTmpFile(what)
 		tmpFileWhat = "";
 	}
 }
-FILE *getTmpFile(what)
-	char *what;
+FILE *getTmpFile(PCStr(what))
 {	FILE *fp;
 	int pid;
-	char msg[1024];
+	CStr(msg,1024);
 
 	pid = getpid();
 	sprintf(msg,"getTmpFile: fd=%d [%d]%s->[%d]%s",
@@ -914,43 +938,55 @@ FILE *getTmpFile(what)
 	return fp;
 }
 
-extern char **dupv();
-char **vect_PATH(path)
-	char *path;
-{	char pathb[2048],del;
-	char *pathv[16];
-	char **ppv;
+const char **vect_PATH(PCStr(path))
+{	CStr(pathb,2048);
+	char del;
+	const char *pathv[16]; /**/
+	const char **ppv;
 	int lc;
 
 	strcpy(pathb,path);
 	if( strchr(pathb,';') ) /* Windows */
 		del = ';';
 	else	del = ':';
-	lc = list2vect(pathb,del,15,pathv);
+	lc = list2vect(pathb,del,elnumof(pathv)-1,pathv);
 	pathv[lc] = 0;
 	ppv = dupv(pathv,0);
 	return ppv;
 }
-FILE *fopen_PATH(pathv,file,mode,xpath)
-	char *pathv[];
-	char *file,*mode,*xpath;
+/*
+ * "/" in PATH like "/Program Files/DeleGate/lib/"
+ * should be replaced with "\" (_PATHSEP())
+ */
+FILE *fopen_PATH(const char *pathv[],PCStr(file),PCStr(mode),PVStr(xpath))
 {	int li;
-	char *dir1,path1[1024];
+	const char *dir1;
+	CStr(path1,1024);
 	FILE *fp;
 
 	for( li = 0; dir1 = pathv[li]; li++ ){
 		if( *dir1 == 0 || strcmp(dir1,".") == 0 )
 			strcpy(path1,file);
-		else	sprintf(path1,"%s/%s",dir1,file);
+		else
+		if( strtailstr(dir1,_PATHSEP()) )
+			sprintf(path1,"%s%s",dir1,file);
+		else	sprintf(path1,"%s%s%s",dir1,_PATHSEP(),file);
 		if( *mode == 'x' )
 			fp = fopen(path1,"r");
 		else	fp = fopen(path1,mode);
+
+		if( errno == EACCES && fileIsdir(path1) ){
+			syslog_ERROR("WARNING: fopen_PATH(%s):isdir %s\n",
+				mode,path1);
+			fp = TMPFILE("fopen_PATH-DIR");/* dummy for Win32 */
+		}
+
 		syslog_DEBUG("### [%d] %s %x\n",li,path1,fp);
 		if( fp != NULL ){
 			if( xpath ){
 				if( *dir1 == 0 || strcmp(dir1,".") == 0 ){
-					getcwd(xpath,256);
-					strcat(xpath,"/");
+					getcwd((char*)xpath,256);
+					strcat(xpath,_PATHSEP());
 					strcat(xpath,path1);
 				}
 				else
@@ -961,48 +997,48 @@ FILE *fopen_PATH(pathv,file,mode,xpath)
 	}
 	return NULL;
 }
-FILE *fopen_PATHLIST(pathlist,file,mode,xpath)
-	char *pathlist,*file,*mode,*xpath;
-{	char **lpathv;
+FILE *fopen_PATHLIST(PCStr(pathlist),PCStr(file),PCStr(mode),PVStr(xpath))
+{	const char **lpathv;
 	FILE *fp;
 
 	lpathv = vect_PATH(pathlist);
-	fp = fopen_PATH(lpathv,file,mode,xpath);
+	fp = fopen_PATH(lpathv,file,mode,AVStr(xpath));
 	free(lpathv);
 	return fp;
 }
-FILE *fopen_LIBPATH(file,mode,xpath)
-	char *file,*mode,*xpath;
+FILE *fopen_LIBPATH(PCStr(file),PCStr(mode),PVStr(xpath))
 {	FILE *fp;
-	char *libpath;
+	const char *libpath;
 
 	if( isFullpath(file) ){
-		fp = fopen(file,mode);
-		*xpath = 0;
+		if( fp = fopen(file,mode) )
+			strcpy(xpath,file);
+		else	setVStrEnd(xpath,0);
 	}else
 	if( libpath = getenv("LIBPATH") )
-		fp = fopen_PATHLIST(libpath,file,mode,xpath);
+		fp = fopen_PATHLIST(libpath,file,mode,AVStr(xpath));
 	else	fp = 0;
 	return fp;
 }
-LIBFILE_IS(file,rxfile)
-	char *file,*rxfile;
+int LIBFILE_IS(PCStr(file),PVStr(rxfile))
 {	FILE *fp;
-	char afile[1024],*xfile,xfileb[1024];
+	CStr(afile,1024);
+	CStr(xfileb,1024);
+	defQStr(xfile); /*alt*/
 
 	if( rxfile )
-		rxfile[0] = 0;
+		setVStrEnd(rxfile,0);
 	if( file == NULL )
 		return 0;
 
 	if( rxfile )
-		xfile = rxfile;
-	else	xfile = xfileb;
+		setQStr(xfile,rxfile,UTail(rxfile)-rxfile+1);
+	else	setQStr(xfile,xfileb,sizeof(xfileb));
 
-	if( fp = fopen_LIBPATH(file,"r",xfile) ){
+	if( fp = fopen_LIBPATH(file,"r",AVStr(xfile)) ){
 		fclose(fp);
-		if( rxfile && xfile[0] == 0 )
-			strcpy(xfile,file);
+		if( rxfile && rxfile[0] == 0 )
+			strcpy(rxfile,file);
 		return 1;
 	}
 	return 0;
