@@ -1,15 +1,13 @@
 #include <stdio.h>
 #include "fpoll.h"
 
-ready_cc(fp)
-	FILE *fp;
+int ready_cc(FILE *fp)
 {
 	if( fp == NULL )
 		return 0;
 	else	return READYCC(fp);
 }
-ready_CC(fp)
-	FILE *fp;
+int ready_CC(FILE *fp)
 {	int rcc;
 
 	rcc = READYCC(fp);
