@@ -18,14 +18,25 @@ History:
 //////////////////////////////////////////////////////////////////////#*/
 
 #include <stdio.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <time.h>
 #include "ystring.h"
-
-extern char *getenv();
-
-#include "dns.h"
-int DNS_debug;
-
 #include "vsocket.h"
+#include "file.h"
+#include "dns.h"
+void SOCKS_addserv(PCStr(dhost),int dport,PCStr(shost),int sport);
+int RES_next_res(PCStr(where),int ri,PVStr(res),PVStr(arg));
+void set_nameserver(PCStr(domain),PCStr(addr));
+void minit_reshost();
+int regGetResolvConf(PVStr(buf),PVStr(where));
+int connectTO(int sock,SAP addr,int leng,int timeout);
+void *callFuncTimeout(int sec,void *xcode,void *(*func)(void*,...),...);
+
+void VSA_copy(VSAddr *dst,VSAddr *src);
+int VSA_comp(VSAddr *sa1,VSAddr *sa2);
+
+int DNS_debug;
 
 #if defined(sgi) || defined(__RES) && (19931104 <= __RES)
 #define _RSTATE 1
@@ -57,11 +68,39 @@ int MIN_ABSNDOTS = 1;
 #define MAX_SORTLIST	16
 #define ADDRLEN	4
 typedef struct {
-	unsigned char	addr[ADDRLEN];
-	unsigned char	mask[ADDRLEN];
+	char	am_addr[ADDRLEN];
+	char	am_mask[ADDRLEN];
 } AddrMask;
-static AddrMask sort_list[MAX_SORTLIST];
-static int Nsortlist;
+#define RESOLVERS_SIZ 512
+
+typedef struct {
+	MStr(	re_Resolvers,RESOLVERS_SIZ);
+	AddrMask re_sort_list[MAX_SORTLIST]; /**/
+	int	re_Nsortlist;
+	MStr(	re__confid,64);
+	struct	hostent re_myhost;
+	VSAddr	re_selfs[16];
+	int	re_selfN;
+	int	re_lastRRX[256];
+	int	re_F_aton;
+  const	char   *re_confpath;
+	MStr(	re_resolv_errmsg,1024);
+} ResolvyEnv;
+static ResolvyEnv *resolvyEnv;
+#define RE		resolvyEnv[0]
+#define Resolvers	RE.re_Resolvers
+/* MStr(Resolvers,re_Resolvers) */
+/**/
+#define sort_list	RE.re_sort_list
+#define Nsortlist	RE.re_Nsortlist
+#define _confid		RE.re__confid
+#define myhost		RE.re_myhost
+#define selfs		RE.re_selfs
+#define selfN		RE.re_selfN
+#define lastRRX		RE.re_lastRRX
+#define F_aton		RE.re_F_aton
+#define confpath	RE.re_confpath
+struct { defQStr(resolv_errmsg); } resolv_errmsg;
 
 #ifndef RSLV_CONF
 #define RSLV_CONF	"/etc/resolv.conf"
@@ -83,67 +122,73 @@ static int Nsortlist;
 #define RSLV_ORDER	"CFNDS"
 #endif
 
-char *_RSLV_CONF = RSLV_CONF;
-char *_HOSTSFILE = HOSTSFILE;
-char *_NISMAP_NAME = NISMAP_NAME;
-char *_NISMAP_ADDR = NISMAP_ADDR;
+void minit_resconf()
+{
+	if( resolvyEnv == 0 ){
+		resolvyEnv = NewStruct(ResolvyEnv);
+		strcpy(Resolvers,RSLV_ORDER);
+		setQStr(resolv_errmsg.resolv_errmsg,resolvyEnv->re_resolv_errmsg,sizeof(resolvyEnv->re_resolv_errmsg));
+	}
+	minit_reshost();
+}
+char *RES_resolvers()
+{
+	return Resolvers;
+}
 
-#define RESOLVERS_SIZ 512
-char resolvers[RESOLVERS_SIZ] = RSLV_ORDER;
+const char *_RSLV_CONF = RSLV_CONF;
+const char *_HOSTSFILE = HOSTSFILE;
+const char *_NISMAP_NAME = NISMAP_NAME;
+const char *_NISMAP_ADDR = NISMAP_ADDR;
 
-char *RES_VERIFY;
+const char *RES_VERIFY;
 #define VIA_SOCKS	"//"
 
-static char _confid[64];
-
-RES_getconf(buf)
-	char *buf;
-{	char *bp;
-	char *addr,*dom;
+void RES_getconf(PVStr(buf))
+{	refQStr(bp,buf); /**/
+	const char *addr;
+	const char *dom;
 	int ni,si;
 	int options;
 
-	bp = buf;
-	sprintf(bp,"RES_ORDER=%s\n",resolvers); bp += strlen(bp);
+	sprintf(bp,"RES_ORDER=%s\n",Resolvers); bp += strlen(bp);
 	sprintf(bp,"HOSTSFILE=%s\n",_HOSTSFILE); bp += strlen(bp);
-	sprintf(bp,"NSIMAP_NAME=%s\n",_NISMAP_NAME); bp += strlen(bp);
-	sprintf(bp,"NSIMAP_ADDR=%s\n",_NISMAP_ADDR); bp += strlen(bp);
+	sprintf(bp,"NISMAP_NAME=%s\n",_NISMAP_NAME); bp += strlen(bp);
+	sprintf(bp,"NISMAP_ADDR=%s\n",_NISMAP_ADDR); bp += strlen(bp);
 	sprintf(bp,"RSLV_CONF=%s\n",_RSLV_CONF); bp += strlen(bp);
 	sprintf(bp,"DNS_NSCOUNT=%d\n",_RES.nscount); bp += strlen(bp);
 	options = _RES.options & ~(RES_DEBUG);
 	sprintf(bp,"DNS_OPTIONS=%d\n",options); bp += strlen(bp);
 	sprintf(bp,"DNS_DEFDNAME=%s\n",_RES.defdname); bp += strlen(bp);
 	for( ni = 0; ni < _RES.nscount; ni++ ){
-		addr = VSA_ntoa(&_RES.nsaddr_list[ni]);
+		addr = VSA_ntoa((VSAddr*)&_RES.nsaddr_list[ni]);
 		sprintf(bp,"DNS_SERVER=%s\n",addr); bp += strlen(bp);
 	}
 	for( si = 0; dom = _RES.dnsrch[si]; si++ ){
 		sprintf(bp,"DNS_SEARCH=%s\n",dom); bp += strlen(bp);
 	}
 }
-char *RES_confid(id)
-	char *id;
-{	char buf[2048];
+char *RES_confid(PVStr(id))
+{	CStr(buf,2048);
 
 	if( _confid[0] == 0 ){
-		RES_getconf(buf);
+		RES_getconf(AVStr(buf));
 		toMD5(buf,_confid);
 		debug(DBG_CACHE,"#### RES_confid = %s\n%s",_confid,buf);
 	}
 
 	strcpy(id,_confid);
 	debug(DBG_CACHE,"RES_confid = %s\n",id);
-	return id;
+	return (char*)id;
 }
 #define clear_confid()	_confid[0] = 0;
 
-static bindINS(sin,nlisten)
-	VSAddr *sin;
+static int bindINS(VSAddr *sin,int nlisten)
 {	int sock,rcode;
 	int sinlen;
 
 	sock = socket(AF_INET,SOCK_STREAM,0);
-	sinlen = VSA_size(&sin);
+	sinlen = VSA_size(sin);
 	if( bind(sock,(SAP)sin,sinlen) == 0 ){
 		listen(sock,nlisten);
 		return sock;
@@ -152,14 +197,13 @@ static bindINS(sin,nlisten)
 		return -1;
 	}
 }
-static connINS(sin)
-	VSAddr *sin;
+static int connINS(VSAddr *sin)
 {	int sock;
 	int sinlen;
 
 	sock = socket(AF_INET,SOCK_STREAM,0);
-	sinlen = VSA_size(&sin);
-	if( connectTO(sock,sin,sinlen,100) == 0 ){
+	sinlen = VSA_size(sin);
+	if( connectTO(sock,(struct sockaddr*)sin,sinlen,100) == 0 ){
 		return sock;
 	}else{
 		close(sock);
@@ -167,19 +211,15 @@ static connINS(sin)
 	}
 }
 
-extern void *callFuncTimeout();
-static struct hostent *_GethostByname(name)
-	char *name;
+static struct hostent *_GethostByname(PCStr(name))
 {	struct hostent *hp;
 
-	hp = (struct hostent *)callFuncTimeout(3,NULL,EX_GETHOSTBYNAME,name);
+	hp = (struct hostent *)callFuncTimeout(3,NULL,(void*(*)(void*,...))EX_GETHOSTBYNAME,name);
 	return hp;
 }
 
-static struct hostent myhost;
-
 static struct hostent *getmyhost()
-{	char myname[256];
+{	CStr(myname,256);
 	struct hostent *hp;
 
 	if( myhost.h_name != NULL )
@@ -189,7 +229,7 @@ static struct hostent *getmyhost()
 	if( hp = _GethostByname(myname) )
 		return hp;
 
-	myhost.h_name = strdup(myname);
+	myhost.h_name = stralloc(myname);
 	myhost.h_length = 4;
 	myhost.h_addr_list = (char**)malloc(sizeof(char*)*2);
 	myhost.h_addr_list[0] = (char*)calloc(1,4); /* 0.0.0.0 */
@@ -197,20 +237,20 @@ static struct hostent *getmyhost()
 	return &myhost;
 }
 
-static VSAddr selfs[16];
-static int selfN;
-RES_isself(mysock)
+void RES_isself(int mysock)
 {	int len;
 	VSAddr *self1;
 
+	if( elnumof(selfs) <= selfN )
+		return;
+
 	len = sizeof(VSAddr);
 	self1 = &selfs[selfN];
 	getsockname(mysock,(SAP)self1,&len);
 	debug(DBG_ANY,"self [%d] %s\n",selfN,VSA_ntoa(self1));
 	selfN++;
 }
-static isself1(me,to)
-	VSAddr *me,*to;
+static int isself1(VSAddr *me,VSAddr *to)
 {	int testsock;
 	VSAddr sin;
 
@@ -235,8 +275,7 @@ static isself1(me,to)
 		return 1;
 	}else	return 0;
 }
-static isself(to)
-	VSAddr *to;
+static int isself(VSAddr *to)
 {	int si;
 
 	for( si = 0; si < selfN; si++ )
@@ -244,7 +283,7 @@ static isself(to)
 			return 1;
 	return 0;
 }
-RES_nsloopcheck(mysock)
+void RES_nsloopcheck(int mysock)
 {	int nsx,nsi,nsj;
 	int len;
 	VSAddr me,sin;
@@ -254,11 +293,11 @@ RES_nsloopcheck(mysock)
 
 	nsx = _RES.nscount;
 	for( nsi = 0; nsi < nsx; ){
-		VSA_copy(&sin,&_RES.NSLIST(nsi));
+		VSA_copy(&sin,(VSAddr*)&_RES.NSLIST(nsi));
 		if( isself1(&me,&sin) ){
 			for( nsj = nsi; nsj < nsx-1; nsj++ )
-				VSA_copy(&_RES.NSLIST(nsj),&_RES.NSLIST(nsj+1));
-			VSA_zero(&_RES.NSLIST(nsj));
+				VSA_copy((VSAddr*)&_RES.NSLIST(nsj),(VSAddr*)&_RES.NSLIST(nsj+1));
+			VSA_zero((VSAddr*)&_RES.NSLIST(nsj));
 			_RES.nscount -= 1;
 			nsx -= 1;
 			debug(DBG_FORCE,"## removed self as NS[%d] %s:%d\n",
@@ -267,15 +306,18 @@ RES_nsloopcheck(mysock)
 	}
 }
 
-RES_ns1(res,ns,domain)
-	State *res;
-	char *ns,*domain;
-{	VSAddr sin;
+void RES_socks(PCStr(ns),PCStr(socks));
+void RES_ns1(State *res,PCStr(ns),PCStr(domain))
+{
+	VSAddr sin;
 	int nsx;
 	struct hostent *hp;
-	char *saddr;
-	char *socks,*cp,nsb[256],ssb[256];
-	char nsbp[256];
+	const char *saddr;
+	const char *socks;
+	const char *cp;
+	CStr(nsb,256);
+	CStr(ssb,256);
+	CStr(nsbp,256);
 	int port;
 
 	clear_confid();
@@ -284,7 +326,7 @@ RES_ns1(res,ns,domain)
 		/* reserver the position */
 		if( res->nscount < MAXNS ){
 			nsx = res->nscount++;
-			VSA_zero(&res->NSLIST(nsx));
+			VSA_zero((VSAddr*)&res->NSLIST(nsx));
 		}
 	}
 
@@ -293,15 +335,15 @@ RES_ns1(res,ns,domain)
 		strcpy(nsb,ns);
 		ns = nsb;
 		socks = strstr(ns,VIA_SOCKS);
-		*socks = 0;
+		truncVStr(socks);
 		if( cp = strpbrk(ns,"# \t") )
-			*cp = 0;
-		sscanf(socks+strlen(VIA_SOCKS),"%[-_.0-9A-Z:]",ssb);
+			truncVStr(cp);
+		Xsscanf(socks+strlen(VIA_SOCKS),"%[-_.0-9A-Z:]",AVStr(ssb));
 	}
 
 	port = PORT_DNS;
 	if( strchr(ns,':') ){
-		sscanf(ns,"%[^:]:%d",nsbp,&port);
+		Xsscanf(ns,"%[^:]:%d",AVStr(nsbp),&port);
 		ns = nsbp;
 	}
 
@@ -335,8 +377,20 @@ RES_ns1(res,ns,domain)
 	RES_socks(saddr,ssb);
 
 	if( strcmp(domain,RES_NSDOM0) == 0 ){
+	  int nsi;
+	  for( nsi = 0; nsi < res->nscount; nsi++ ){
+		if( VSA_comp(&sin,(VSAddr*)&res->NSLIST(nsi)) == 0 ){
+			debug(DBG_FORCE,"dup. RES_NS[%d]=%s:%d ignored\n",
+				nsi,VSA_ntoa(&sin),VSA_port(&sin));
+			if( 0 <= nsx && nsx+1 == res->nscount ){
+				/* remove the reserved slot */
+				res->nscount--;
+			}
+			return;
+		} 
+	  }
 	  if( 0 <= nsx ){
-		VSA_copy(&res->NSLIST(nsx),&sin);
+		VSA_copy((VSAddr*)&res->NSLIST(nsx),&sin);
 		debug(DBG_ANY,"        RES_NS[%d]=%s/%s\n",nsx,saddr,domain);
 	  }else	debug(DBG_ANY," ignore RES_NS(%d)=%s/%s\n",MAXNS,ns,domain);
 	}else{
@@ -345,45 +399,61 @@ RES_ns1(res,ns,domain)
 	}
 }
 
-DNS_connect(addr,port)
-	char *addr;
+int DNS_connect(PCStr(addr),int port)
 {	VSAddr sin;
 
 	VSA_atosa(&sin,port,addr);
 	return connINS(&sin);
 }
 
-RES_getns1(nsi,sin)
-	VSAddr *sin;
+int RES_getns1(int nsi,VSAddr *sin)
 {	State *res;
 
 	res = &_RES;
 	if( nsi < _RES.nscount ){
-		VSA_copy(sin,&res->NSLIST(nsi));
+		VSA_copy(sin,(VSAddr*)&res->NSLIST(nsi));
 		return 1;
 	}
 	return 0;
 }
 
-load_rslvconf(res,path,loadns)
-	State *res;
-	char *path;
+int stoBV(PCStr(str),PVStr(buf),int mc,int ez){
+	const char *sp = str;
+	int sc;
+	refQStr(b1,buf); /**/
+
+	for(sc = 0; sc < mc; sc++){
+		setQStr(b1,&buf[sc*ez],ez);
+		sp = wordscanX(sp,ZVStr(b1,ez),ez);
+		if( *b1 == 0 )
+			break;
+	}
+	return sc;
+}
+
+void RES_scan_sortlist(PCStr(line));
+void load_rslvconf(State *res,PCStr(path),int loadns)
 {	FILE *fp;
-	char line[1024],com[1024],arg[1024];
-	char *dp;
-	char s[8][128];
+	CStr(line,1024);
+	CStr(com,1024);
+	CStr(arg,1024);
+	const char *dp;
+	ACStr(s,8,128);
 	int sc,si;
 
 	clear_confid();
 	debug(DBG_ANY,"load_rslvconf(%s)\n",path);
 	if( strncmp(path,"sh:",3) == 0 )
 		fp = popen(path+3,"r");
+	else
+	if( strncmp(path,"file:",5) == 0 )
+		fp = fopen(path+5,"r");
 	else	fp = fopen(path,"r");
 
 	if( fp == NULL ){
-		char buff[2048],where[256];
-		FILE *TMPFILE();
-		if( regGetResolvConf(buff,where) == 0 ){
+		CStr(buff,2048);
+		CStr(where,256);
+		if( regGetResolvConf(AVStr(buff),AVStr(where)) == 0 ){
 			fp = TMPFILE("regGetResolvConf");
 			fputs(buff,fp);
 			fflush(fp);
@@ -398,8 +468,8 @@ load_rslvconf(res,path,loadns)
 
 	while( fgets(line,sizeof(line),fp) != NULL ){
 	    if( dp = strpbrk(line,"#;") )
-		*dp = 0;
-	    if( sscanf(line,"%s %s",com,arg) < 2 )
+		truncVStr(dp);
+	    if( Xsscanf(line,"%s %s",AVStr(com),AVStr(arg)) < 2 )
 		continue;
 
 	    if( strcmp(com,"debug") == 0 ){
@@ -411,21 +481,23 @@ load_rslvconf(res,path,loadns)
 		}
 	    }else
 	    if( strcmp(com,"domain") == 0 ){
-		strcpy(res->defdname,arg);
+		Xstrcpy(EVStr(res->defdname),arg);
 		res->options |= RES_DEFNAMES;
 	    }else
 	    if( strcmp(com,"ndots") == 0 ){
 		MIN_ABSNDOTS = atoi(arg);
 	    }else
 	    if( strcmp(com,"search") == 0 ){
-		sc = sscanf(line,"%*s %s %s %s %s %s %s %s %s",
-		    s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7]);
+/*
+		sc = sscanf (line,"%*s %s %s %s %s %s %s %s %s",s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7]);
+*/
+		sc = stoBV(wordscanX(line,EVStr(s[0]),sizeof(s[0])),ZVStr((char*)s,sizeof(s)),8,128);
 
 		if( loadns && 0 < sc ){
-		    strcpy(res->defdname,s[0]);
+		    Xstrcpy(EVStr(res->defdname),s[0]);
 		    for( si = 0; si < MAXDNSRCH && si < sc; si++ ){
 			debug(DBG_ANY,"        RES_SRCH[%d] %s\n",si,s[si]);
-			res->dnsrch[si] = strdup(s[si]);
+			res->dnsrch[si] = stralloc(s[si]);
 		    }
 		    res->dnsrch[sc] = 0;
 		    res->options |=  RES_DNSRCH;
@@ -441,107 +513,110 @@ load_rslvconf(res,path,loadns)
 	else	fclose(fp);
 }
 
-static ns1(name)
-	char *name;
-{	char nameb[128];
-	char *domain;
+static scanListFunc ns1(PCStr(name))
+{	CStr(nameb,128);
+	const char *domain;
 
 	strcpy(nameb,name);
 	if( domain = strchr(nameb,'/') )
 		if( domain[1] == '/' )
 			domain = strchr(domain+2,'/');
-	if( domain )
-		*domain++ = 0;
-	else	domain = RES_NSDOM0;
+	if( domain ){
+		truncVStr(domain); domain++;
+	}else	domain = RES_NSDOM0;
 
 	RES_ns1(&_RES,nameb,domain);
 	return 0;
 }
-RES_ns(nslist)
-	char *nslist;
+int RES_ns(PCStr(nslist))
 {
 	if( nslist )
-		scan_commaList(nslist,0,ns1);
+		scan_commaList(nslist,0,scanListCall ns1);
 	return 0;
 }
-RES_verify(verify)
-	char *verify;
+void RES_verify(PCStr(verify))
 {
-	RES_VERIFY = strdup(verify);
+	RES_VERIFY = stralloc(verify);
 }
-RES_verifyFaiure(host,badhost)
-	char *host,*badhost;
-{	char *sp,sc,*dp;
+void RES_verifyFaiure(PCStr(host),PVStr(badhost))
+{	const char *sp;
+	char sc;
+	refQStr(dp,badhost); /**/
 
-	dp = badhost;
 	if( sp = RES_VERIFY ){
 		for( ; sc = *sp; sp++ ){
+			assertVStr(badhost,dp);
 			if( sc == '*' ){
 				strcpy(dp,host);
 				dp += strlen(dp);
 			}else{
-				*dp++ = sc;
+				setVStrPtrInc(dp,sc);
 			}
 		}
 	}
-	*dp = 0;
+	setVStrEnd(dp,0);
 }
 
-RES_debug(debug)
-	char *debug;
+int RES_debug(PCStr(debug))
 {
 	_RES.options |= RES_DEBUG;
+	if( strncmp(debug,"0x",2) == 0 )
+	sscanf(debug+2,"%x",&DNS_debug);
+	else
 	DNS_debug |= atoi(debug);
 	return 0;
 }
-RES_domain(domain)
-	char *domain;
+int RES_domain(PCStr(domain))
 {
-	strcpy(_RES.defdname,domain);
+	Xstrcpy(EVStr(_RES.defdname),domain);
 	return 0;
 }
-RES_order(order,porder)
-	char *order,*porder;
+int RES_order(PCStr(order),xPVStr(porder))
 {	int oi;
-	char buff[RESOLVERS_SIZ];
+	CStr(buff,RESOLVERS_SIZ);
+
+	minit_resconf();
 
 	clear_confid();
-	if( porder == NULL )
-		porder = buff;
-	strcpy(porder,resolvers);
-	Strncpy(resolvers,order,sizeof(resolvers));
+	if( porder == NULL ){
+		setPStr(porder,buff,sizeof(buff));
+	}
 
-	debug(DBG_ANY,"RES_order(%s,%s)\n",resolvers,porder);
+	strcpy(porder,Resolvers);
+	if( order == NULL )
+		return 0;
+
+	FStrncpy(Resolvers,order);
+
+	debug(DBG_ANY,"RES_order(%s,%s)\n",Resolvers,porder);
 	return 0;
 }
-RES_conf(path)
-	char *path;
-{	static char *confpath;
-
-	if( _RSLV_CONF == confpath )
-		free(confpath);
-	_RSLV_CONF = confpath = strdup(path);
-	return 0;
-}
-RES_hosts(path)
-	char *path;
+int RES_conf(PCStr(path))
 {
-	_HOSTSFILE = strdup(path);
+	if( _RSLV_CONF == confpath )
+		free((char*)confpath);
+	_RSLV_CONF = confpath = stralloc(path);
+	return 0;
+}
+int RES_hosts(PCStr(path))
+{
+	_HOSTSFILE = stralloc(path);
 	return 0;
 }
 
-RES_add_sortlist(addrs,masks,len)
-	char *addrs[],*masks[];
+void RES_add_sortlist1(PCStr(addr),char mask[],int len);
+void RES_add_sortlist(const char *addrs[],char *masks[],int len)
 {	int sx;
 
 	for( sx = 0; addrs[sx]; sx++)
 		RES_add_sortlist1(addrs[sx],masks?masks[sx]:NULL,len);
 }
-RES_add_sortlist1(addr,mask,len)
-	unsigned char *addr,*mask;
+void RES_add_sortlist1(PCStr(addr),char mask[],int len)
 {	AddrMask *sp;
 	int ai,nm;
-	unsigned char dmask[32];
+	CStr(dmask,32);
+	const unsigned char *ua;
+	const unsigned char *um;
 
 	if( MAX_SORTLIST <= Nsortlist+1 ){
 		debug(DBG_ANY,"!! exceed MAX_SORTLIST[%d]\n",MAX_SORTLIST);
@@ -559,67 +634,73 @@ RES_add_sortlist1(addr,mask,len)
 		for( ; ai < len && ai < sizeof(dmask); ai++ )
 			mask[ai] = 0;
 	}
+
+	ua = (unsigned char *)addr;
+	um = (unsigned char *)mask;
 	debug(DBG_ANY,"        sortlist[%d] %d.%d.%d.%d / %d.%d.%d.%d\n",
 		Nsortlist,
-		addr[0],addr[1],addr[2],addr[3],
-		mask[0],mask[1],mask[2],mask[3]);
+		ua[0],ua[1],ua[2],ua[3],
+		um[0],um[1],um[2],um[3]);
 
 	sp = &sort_list[Nsortlist++];
 	for(ai = 0; ai < len; ai++){
-		sp->addr[ai] = addr[ai];
-		sp->mask[ai] = mask[ai];
+		sp->am_addr[ai] = addr[ai];
+		sp->am_mask[ai] = mask[ai];
 	}
 }
-RES_scan_sortlist(line)
-	char *line;
-{	char s[16][128];
-	char addr[32],mask[32];
+void RES_scan_sortlist(PCStr(line))
+{	ACStr(s,16,128);
+	CStr(addr,32);
+	CStr(mask,32);
 	int a[16],m[16];
 	int sc,si,ai;
-	char ba[16],bm[16];
+	CStr(ba,16);
+	CStr(bm,16);
 	int len = 4;
 
-	sc = sscanf(line,"%*s %s %s %s %s %s %s %s %s %s %s",
-	    s[0],s[1],s[2],s[3],s[4],s[5],s[6],s[7],s[8],s[9]);
+	sc = stoBV(wordscanX(line,EVStr(s[0]),sizeof(s[0])),ZVStr((char*)s,sizeof(s)),16,128);
 
 	for( si = 0; si < sc; si++ ){
 		addr[0] = mask[0] = 0;
-		sscanf(s[si],"%[^/]/%s",addr,mask);
+		Xsscanf(s[si],"%[^/]/%s",AVStr(addr),AVStr(mask));
 		for( ai = 0; ai < len; ai++ )
 			a[ai] = m[ai] = 0;
 		sscanf(addr,"%d.%d.%d.%d",&a[0],&a[1],&a[2],&a[3]);
 		sscanf(mask,"%d.%d.%d.%d",&m[0],&m[1],&m[2],&m[3]);
 		for( ai = 0; ai < len; ai++ ){
-			ba[ai] = a[ai];
-			bm[ai] = m[ai];
+			ba[ai] = a[ai]; /**/
+			bm[ai] = m[ai]; /**/
 		}
 		RES_add_sortlist1(ba,bm,len);
 	}
 }
 
-static int lastRRX[256];
 int RES_ROUNDROBIN = 1;
 
-RES_roundrobin(hosts)
-	char *hosts;
+void RES_roundrobin(PCStr(hosts))
 {
 	if( strcmp(hosts,"*") == 0 )
 		RES_ROUNDROBIN = 1;
 	else	RES_ROUNDROBIN = 0;
 }
 
-sort_ipaddrs(addrs)
-	unsigned char *addrs[];
-{	unsigned char *ap,*mp;
-	int si,sx,hit;
+int sort_ipaddrs1(const char *addrs[],PCStr(ap),PCStr(mp));
+void sort_ipaddrs(const char *addrs[])
+{	int si,sx,hit;
 
 	if( RES_ROUNDROBIN ){
 		int sn,si,sx,so,rrx;
-		unsigned char *saddrs[256],*a1;
+		const char *saddrs[256]; /**/
+		const unsigned char *a1;
 
 		rrx = 0;
-		for( sn = 0; a1 = saddrs[sn] = addrs[sn]; sn++ )
+		for( sn = 0; saddrs[sn] = addrs[sn]; sn++ ){
+			if( elnumof(saddrs) <= sn ){
+				break;
+			}
+			a1 = (unsigned char *)saddrs[sn];
 			rrx += a1[0] + a1[1] + a1[2] + a1[3];
+		}
 		rrx %= 256;
 		sx = lastRRX[rrx] % sn;
 		lastRRX[rrx] += 1;
@@ -632,17 +713,21 @@ sort_ipaddrs(addrs)
 
 	hit = 0;
 	for( si = 0; si < Nsortlist; si++ ){
-		ap = sort_list[si].addr;
-		mp = sort_list[si].mask;
+		const char *ap;
+		const char *mp;
+		ap = sort_list[si].am_addr;
+		mp = sort_list[si].am_mask;
 		hit += sort_ipaddrs1(&addrs[hit],ap,mp);
 	}
 }
 
 #define btohl(ap) ((ap[0]<<24) | (ap[1]<<16) | (ap[2]<<8) | ap[3])
-sort_ipaddrs1(addrs,ap,mp)
-	unsigned char *addrs[];
-	unsigned char *ap,*mp;
-{	unsigned char *cap,*saddrs[256],done[1];
+int sort_ipaddrs1(const char *addrs[],PCStr(sap),PCStr(smp))
+{	const unsigned char *ap = (unsigned char *)sap;
+	const unsigned char *mp = (unsigned char *)smp;
+	const char *cap;
+	const char *saddrs[256]; /**/
+	CStr(done,1);
 	int ciaddr,cimasked;
 	int sn,sx,dn,hit,imask;
 
@@ -657,7 +742,9 @@ sort_ipaddrs1(addrs,ap,mp)
 	for( sx = 0; cap = addrs[sx]; sx++ ){
 		if( cap != done )
 		if( (btohl(cap) & imask) == cimasked ){
-			saddrs[dn++] = cap;
+			if( elnumof(saddrs) <= dn )
+				break;
+			saddrs[dn++] = (char*)cap;
 			addrs[sx] = done;
 		}
 	}
@@ -672,10 +759,9 @@ sort_ipaddrs1(addrs,ap,mp)
 		ap[0],ap[1],ap[2],ap[3],mp[0],mp[1],mp[2],mp[3],hit,sn);
 	return hit;
 }
-static res_getoptions(options,soptions)
-	char *soptions;
+static void res_getoptions(int options,PVStr(soptions))
 {
-	*soptions = 0;
+	setVStrEnd(soptions,0);
 	if( options & RES_DEBUG    ) strcat(soptions,"DEBUG ");
 	if( options & RES_RECURSE  ) strcat(soptions,"RECURSE ");
 	if( options & RES_DEFNAMES ) strcat(soptions,"DEFNAMES ");
@@ -683,14 +769,18 @@ static res_getoptions(options,soptions)
 }
 
 int RES_localdns;
-RES_init()
-{	char *env,*conf;
-	char savorder[RESOLVERS_SIZ];
+void RES_init()
+{	const char *env;
+	const char *conf;
+	CStr(savorder,RESOLVERS_SIZ);
 	int loadns;
 	struct hostent *hp;
-	char options[1024];
+	CStr(options,1024);
 	int rx;
-	char res1[RESOLVERS_SIZ],arg[RESOLVERS_SIZ];
+	CStr(res1,RESOLVERS_SIZ);
+	CStr(arg,RESOLVERS_SIZ);
+
+	minit_resconf();
 
 	if( _RES.options & RES_INIT )
 		return;
@@ -704,15 +794,15 @@ RES_init()
 			DNS_debug = DBG_NS;
 	}
 	debug(DBG_ANY,"RES_init()\n");
-	Strncpy(savorder,resolvers,sizeof(savorder));
-	strcpy(resolvers,"FND");
+	FStrncpy(savorder,Resolvers);
+	strcpy(Resolvers,"FND");
 
-	Strncpy(resolvers,savorder,sizeof(resolvers));
+	FStrncpy(Resolvers,savorder);
 	if( env = getenv("RES_ORDER") )
-		Strncpy(resolvers,env,sizeof(resolvers));
-	debug(DBG_ANY,"        RES_ORDER=%s\n",resolvers);
+		FStrncpy(Resolvers,env);
+	debug(DBG_ANY,"        RES_ORDER=%s\n",Resolvers);
 
-	for( rx = 0; rx = RES_next_res(resolvers,rx,res1,arg); ){
+	for( rx = 0; rx = RES_next_res(Resolvers,rx,AVStr(res1),AVStr(arg)); ){
 		if( res1[0] == 'D' && arg[0] != 0 )
 			RES_ns1(&_RES,arg,RES_NSDOM0);
 	}
@@ -735,13 +825,13 @@ RES_init()
 
 	if( Nsortlist == 0 )
 	if( hp = getmyhost() )
-		RES_add_sortlist(hp->h_addr_list,NULL,hp->h_length);
+		RES_add_sortlist((const char**)hp->h_addr_list,NULL,hp->h_length);
 
 	if( _RES.nscount == 0 )
 	/* registory about DNS should be searched on Windows ... */
 	if( hp = getmyhost() ){
-		unsigned char *ap;
-		char saddr[32];
+		const unsigned char *ap;
+		CStr(saddr,32);
 		int dnsock;
 
 		ap = (unsigned char*)hp->h_addr_list[0];
@@ -765,7 +855,7 @@ RES_init()
 
 	}
 	if( (env = getenv("RES_DOMAIN")) || (env = getenv("LOCALDOMAIN")) ){
-		strcpy(_RES.defdname,env);
+		Xstrcpy(EVStr(_RES.defdname),env);
 		_RES.options &= ~RES_DNSRCH;
 		_RES.options |=  RES_DEFNAMES;
 	}
@@ -773,38 +863,35 @@ RES_init()
 	if( _RES.defdname[0] )
 		debug(DBG_ANY,"        RES_DOMAIN=%s\n",_RES.defdname);
 
-	res_getoptions(_RES.options,options);
+	res_getoptions(_RES.options,AVStr(options));
 	debug(DBG_ANY,"        options = %s\n",options);
 }
 
-char resolv_errmsg[512];
-int (*RES_debugprinter)();
-res_debug(flag,fmt,a,b,c,d,e,f,g,h,i,j,k,l,m,n)
-	char *fmt,*a,*b,*c,*d,*e,*f,*g,*h,*i,*j,*k,*l,*m,*n;
+iFUNCP RES_debugprinter;
+int res_debug(int flag,PCStr(fmt),...)
 {	int now;
+	VARGS(14,fmt);
 
 	if( flag == DBG_FORCE || _RES.options & RES_DEBUG && DNS_debug & flag ){
 		now = time(0);
 		if( RES_debugprinter )
-			(*RES_debugprinter)(fmt,a,b,c,d,e,f,g,h,i,j,k,l,m,n);
+			(*RES_debugprinter)(fmt,VA14);
 		else{
 			fprintf(stderr,"%02d:%02d ",(now%3600)/60,now%60);
-			fprintf(stderr,fmt,a,b,c,d,e,f,g,h,i,j,k,l,m,n);
+			fprintf(stderr,fmt,VA14);
 		}
 		return 1;
 	}else	return 0;
 }
-int (*RES_log)();
-res_log(which,byname,name,rv,cname)
-	char *name,*rv[],*cname;
+int (*RES_log)(int,...);
+void res_log(int which,int byname,PCStr(name),char *rv[],PCStr(cname))
 {
 	if( RES_log )
 		(*RES_log)(which,byname,name,rv,cname);
 }
 
-static putNames(hp)
-	struct hostent *hp;
-{	unsigned char *op;
+static void putNames(struct hostent *hp)
+{	const unsigned char *op;
 	int hi;
 
 	printf("%s",hp->h_name);
@@ -815,9 +902,8 @@ static putNames(hp)
 		printf(",%s",op);
 	}
 }
-static putAddrs(hp)
-	struct hostent *hp;
-{	unsigned char *op;
+static void putAddrs(struct hostent *hp)
+{	const unsigned char *op;
 	int hi;
 
 	for( hi = 0; ; hi++ ){
@@ -830,12 +916,11 @@ static putAddrs(hp)
 	}
 }
 
-struct hostent *RES_gethost(addrhost)
-	char *addrhost;
+struct hostent *RES_gethost(PCStr(addrhost))
 {	struct hostent *hp;
 	VSAddr sab;
 	int bleng,btype;
-	char *baddr;
+	const char *baddr;
 
 	if( VSA_strisaddr(addrhost) ){
 		VSA_atosa(&sab,0,addrhost);
@@ -846,9 +931,7 @@ struct hostent *RES_gethost(addrhost)
 	}
 	return hp;
 }
-RES_1(f_aton,fp,arg)
-	FILE *fp;
-	char *arg;
+void RES_1(int f_aton,FILE *fp,PCStr(arg))
 {	struct hostent *hp;
 
 	if( f_aton ){
@@ -863,7 +946,7 @@ RES_1(f_aton,fp,arg)
 	hp = RES_gethost(arg);
 
 	if( hp ){
-		unsigned char *op;
+		const unsigned char *op;
 		int hi;
 		putAddrs(hp);
 		printf("\t");
@@ -876,11 +959,10 @@ RES_1(f_aton,fp,arg)
 		/*exit(1);*/
 	}
 }
-RES_1s(addrhost,addr_host)
-	char *addrhost;
-	char *addr_host;
+int RES_1s(PCStr(addrhost),PVStr(addr_host))
 {	struct hostent *hp;
-	unsigned char *op,addr[32];
+	const unsigned char *op;
+	CStr(addr,32);
 
 	if( hp = RES_gethost(addrhost) ){
 		op = (unsigned char *)hp->h_addr;
@@ -892,22 +974,21 @@ RES_1s(addrhost,addr_host)
 }
 
 extern int RSLV_TIMEOUT;
-RES_timeout(timeout)
+int RES_timeout(int timeout)
 {
 	RSLV_TIMEOUT = timeout;
 	return 1;
 }
 
-RES_socks(ns,socks)
-	char *ns,*socks;
-{	char host[256];
+void RES_socks(PCStr(ns),PCStr(socks))
+{	CStr(host,256);
 	int port;
 
 	if( *socks == 0 )
 		return;
 
 	port = 1080;
-	sscanf(socks,"%[^:]:%d",host,&port);
+	Xsscanf(socks,"%[^:]:%d",AVStr(host),&port);
 	if( !VSA_strisaddr(host) )
 		return;
 
@@ -927,42 +1008,46 @@ char *res_DEFDNAME(){
 }
 
 
+static void resolv1(PCStr(arg));
+void dns_server(int qsock,int rsock);
+void DO_INITIALIZE(int ac,const char *av[]);
 
-static int f_aton;
-resolvy_main(ac,av)
-	char *av[];
+int resolvy_main(int ac,const char *av[])
 {	int ai;
-	char *arg;
+	int itvl;
+	const char *arg;
 
+	minit_resconf();
 	DO_INITIALIZE(ac,av);
 
 	if( ac <= 1 ){
-fprintf(stderr,
+ fprintf(stderr,
 "Usage -- %s [NS=nameserver] { domain-name | ip-address }\n",av[0]);
-fprintf(stderr,
+ fprintf(stderr,
 "<ip-address> can be specified as a range like: aa.bb.cc.dd-ee\n");
 		exit(-1);
 	}
+	itvl = 0;
 	for( ai = 1; ai < ac; ai++ ){
 		arg = av[ai];
 		if( strncmp(arg,"-d",2) == 0 ){
 			RES_debug(arg+2);
 			continue;
 		}else
+		if( strncmp(arg,"-i",2) == 0 ){
+			itvl = atoi(arg+2);
+		}else
 		if( strcmp(arg,"+n") == 0 ){
-			f_aton = 1;
+			F_aton = 1;
 			continue;
 		}else
 		if( strncmp(arg,"NS=",3) == 0 ){
 			RES_ns(arg+3);
 		}else
 		if( strcmp(arg,"-") == 0 ){
-			char line[256],*dp;
-
-			while( fgets(line,sizeof(line),stdin) != NULL ){
-				if( dp = strpbrk(line,"\r\n") )
-					*dp = 0;
-				RES_1(f_aton,stdout,line);
+			CStr(line,256);
+			while( Fgets(AVStr(line),sizeof(line),stdin) != NULL ){
+				RES_1(F_aton,stdout,line);
 				fflush(stdout);
 			}
 		}else
@@ -970,22 +1055,24 @@ fprintf(stderr,
 			dns_server(0,1);
 		}else{
 			resolv1(arg);
+			if( itvl && ai+1 < ac )
+				sleep(itvl);
 		}
 	}
 	exit(0);
+	return 0;
 }
-static resolv1(arg)
-	char *arg;
+static void resolv1(PCStr(arg))
 {	int a1,a2,a3,a41,a42,a4;
-	char addr[256];
+	CStr(addr,256);
 
 	if( sscanf(arg,"%d.%d.%d.%d-%d",&a1,&a2,&a3,&a41,&a42) == 5 ){
 		for( a4 = a41; a4 < a42; a4++ ){
 			sprintf(addr,"%d.%d.%d.%d",a1,a2,a3,a4);
-			RES_1(f_aton,stdout,addr);
+			RES_1(F_aton,stdout,addr);
 		}
 	}else{
-		RES_1(f_aton,stdout,arg);
+		RES_1(F_aton,stdout,arg);
 	}
 	fflush(stdout);
 }
