@@ -18,10 +18,20 @@ History:
 //////////////////////////////////////////////////////////////////////#*/
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <time.h>
 #include "ystring.h"
 #include "vsocket.h"
+#include "fpoll.h"
 #include "dns.h"
-extern char *VSA_ntoa();
+
+int setCloseOnExec(int fd);
+int file_ISSOCK(int fd);
+void msleep(int msec);
+
+int RES_getns1(int nsi,VSAddr *sin);
+int VSA_comp(VSAddr*,VSAddr*);
+static int getns1(int nid,int nsx,int nsc,VSAddr *nservv);
 
 #define MAX_NS		10
 #define MAX_NS_PARA	 1
@@ -34,39 +44,46 @@ int RSLV_INV_TIMEOUT =   6;
 
 int RSLV_TIMEDOUT;
 
-static char *SYM_TYPE[] = {
+static const char *SYM_TYPE[] = {
 	"TYPE=0?",	"A",		"NS",		"MD",
 	"MF",		"CNAME",	"SOA",		"MB",
 	"MG",		"MR",		"NULL",		"WKS",
 	"PTR",		"HINFO",	"MINFO",	"MX",
 	"TXT",		"TYPE=17?",	"TYPE=18?",	"TYPE=19?",
 	"XPTR",		"TYPE=21?",	"TYPE=22?",	"TYPE=23?",
+	"TYPE=24?",	"TYPE=25?",	"TYPE=26?",	"TYPE=27?",
+	"AAAA",		"TYPE=29?",	"TYPE=30?",	"TYPE=31?",
+	"TYPE=32?",	"TYPE=33?",	"TYPE=34?",	"TYPE=35?",
+	"TYPE=36?",	"TYPE=37?",	"A6",		"TYPE=39?",
 };
-#define symTYPE(class)  (class<=20 ? SYM_TYPE[class]:(class==255 ? "QALL":"?"))
+#define symTYPE(class)  ((0<=class&&class<=39) ? SYM_TYPE[class]:\
+	(class==TY_QXFER  ? "QXFER": \
+	(class==TY_QMAILB ? "QMAILB": \
+	(class==TY_QMAILA ? "QMAILA": \
+	(class==TY_QALL   ? "QALL":"?")))))
 
-static char *SYM_RCODE[] = {
+static const char *SYM_RCODE[] = {
 	"No-error",	"Format-error",	"Server-failure", "Name-error",
      "Not-implemented",	"Refused",	"Error-6",	"Error-7",
 	"Error-8",	"Error-9",	"Error-A",	"Error-B",
 	"Error-C",	"Error-D",	"Error-E",	"Error-F",
 };
-static char *SYM_RR[] = {
+static const char *SYM_RR[] = {
 	"RR=0?",
 	"ANS",
 	"SER",
 	"ADD",
 };
-static char *SYM_CLASS[] = {
+static const char *SYM_CLASS[] = {
 	"CLASS=0?",
 	"IN",
 	"CS",
 	"CH",
 	"HS",
 };
-#define symCLASS(class)	(class<=4?SYM_CLASS[class]:"CLASS-X")
+#define symCLASS(class)	((0<=class&&class<=4) ? SYM_CLASS[class]:"CLASS-X")
 
-static octet *makeHeader(head,id,qr,opcode,aa,tc,rd,ra,z,rcode,qdc,anc,nsc,arc)
-	octet *head;
+static char *makeHeader(char head[],int id,int qr,int opcode,int aa,int tc,int rd,int ra,int z,int rcode,int qdc,int anc,int nsc,int arc)
 {
 	head[ 0] = (id>>8) & 0xFF;
 	head[ 1] = id & 0xFF;
@@ -82,10 +99,8 @@ static octet *makeHeader(head,id,qr,opcode,aa,tc,rd,ra,z,rcode,qdc,anc,nsc,arc)
 	head[11] = arc & 0xFF;
 	return &head[12];
 }
-static octet *scanHeader(head,Hp)
-	octet *head;
-	Header *Hp;
-{	octet *hp;
+static const char *scanHeader(PCStr(head),Header *Hp)
+{	const char *hp;
 
 	hp = head;
 	getShort(hp,Hp->id);
@@ -96,10 +111,9 @@ static octet *scanHeader(head,Hp)
 	getShort(hp,Hp->arcount);
 	return &head[12];
 }
-static octet *dumpHeader(head,Hp)
-	octet *head;
-	Header *Hp;
-{	octet *mp,buf[1024];
+static const char *dumpHeader(PCStr(head),Header *Hp)
+{	const char *mp;
+	CStr(buf,1024);
 	int M;
 
 	mp = scanHeader(head,Hp);
@@ -117,7 +131,7 @@ static octet *dumpHeader(head,Hp)
 		H_Z(M),
 		H_RCODE(M)
 	);
-	sprintf(buf+strlen(buf),"QD=%d AN=%d NS=%d AR=%d",
+	Xsprintf(TVStr(buf),"QD=%d AN=%d NS=%d AR=%d",
 		Hp->qdcount,
 		Hp->ancount,
 		Hp->nscount,
@@ -127,10 +141,11 @@ static octet *dumpHeader(head,Hp)
 	return mp;
 }
 
-static octet *putName(qp,name)
-	octet *qp,*name;
-{	octet *np;
-	octet label[255],*lp;
+static char *putName(xPVStr(qp),PCStr(name))
+{	const char *np;
+	CStr(label,64); /* must be in 6bits */
+	refQStr(lp,label); /**/
+	const char *lx;
 
 	for( np = name; *np; np++ ){
 		lp = label;
@@ -138,40 +153,47 @@ static octet *putName(qp,name)
 			debug(DBG_ANY,"FATAL ERROR: empty label [%s]\n",name);
 			return NULL;
 		}
+		lx = label + sizeof(label) - 1;
 		while( *np && *np != '.' )
-			*lp++ = *np++;
-		*lp = 0;
-		*qp++ = strlen(label);
+		{
+			if( lx <= lp ){
+				setVStrEnd(lp,0);
+				debug(DBG_FORCE,"FATAL: label too long [%s]\n",
+					label);
+				break;
+			}
+			setVStrPtrInc(lp,*np++);
+		}
+		setVStrEnd(lp,0);
+		setVStrPtrInc(qp,strlen(label));
 		strcpy(qp,label);
 		qp += strlen(qp);
 		if( *np == 0 )
 			break;
 	}
-	*qp++ = 0;
-	return qp;
+	setVStrPtrInc(qp,0);
+	return (char*)qp;
 }
-static octet *makeQuestion(question,name,type,class)
-	octet *question,*name;
-{	octet *qp;
+static char *makeQuestion(PVStr(question),PCStr(name),int type,int xclass)
+{	refQStr(qp,question);
 
-	qp = question;
-	qp = putName(qp,name);
+	cpyQStr(qp,question);
+	qp = putName(QVStr(qp,question),name);
 	if( qp == NULL )
 		return NULL;
 
-	*qp++ = 0xFF & (type >> 8);
-	*qp++ = 0xFF & type;
-	*qp++ = 0xFF & (class >> 8);
-	*qp++ = 0xFF & class;
-	return qp;
+	setVStrPtrInc(qp,0xFF & (type >> 8));
+	setVStrPtrInc(qp,0xFF & type);
+	setVStrPtrInc(qp,0xFF & (xclass >> 8));
+	setVStrPtrInc(qp,0xFF & xclass);
+	return (char*)qp;
 }
-static octet *scanName(msg,sp,name,lev)
-	octet *msg,*sp;
-	char *name;
-{	char *np;
+static const char *scanName(PCStr(msg),PCStr(ssp),PVStr(name),int siz,int lev)
+{	refQStr(np,name);
+	const octet *sp = (octet*)ssp;
 	int len,low,off,here;
 
-	np = name;
+	cpyQStr(np,name);
 	while( len = *sp++ ){
 		if( len & 0xC0 ){
 			/* This is a pointer (2 bytes).  First 2 bits should
@@ -179,6 +201,7 @@ static octet *scanName(msg,sp,name,lev)
 			 * are reserved for future use.
 			 */
 			if( (len & 0xC0) != 0xC0 ){
+here = (char*)sp - msg;
 debug(DBG_ANY,"### bad pointer(%d): %x: %x\n",lev,here,len);
 				sprintf(np,"BAD-POINTER-%x",len);
 				np += strlen(np);
@@ -187,59 +210,55 @@ debug(DBG_ANY,"### bad pointer(%d): %x: %x\n",lev,here,len);
 			low = *sp++;
 			off = ((len & 0x3F) << 8) | low;
 
-			here = sp - msg;
+			here = (char*)sp - msg;
 			if( here <= off ){
 debug(DBG_ANY,"### bad pointer(%d): %d <= %d: %x %x\n",lev,here,off,len,low);
 				sprintf(np,"BAD-POINTER-%x-%x",len,low);
 				np += strlen(np);
 				break;
 			}
-			scanName(msg,&msg[off],np,lev+1);
-			return sp;
+			scanName(msg,&msg[off],QVStr(np,name),siz-(np-name),lev+1);
+			return (char*)sp;
 		}else{
-/*
-			strncpy(np,sp,len);
-*/
-			bcopy(sp,np,len);
+			if( siz-(np-name) < len+2 ){
+debug(DBG_FORCE,"### name buffer overflow: lev=%d,len=%d,siz=%d\n",lev,len,siz);
+				break;
+			}
+			Bcopy(sp,np,len);
 			np += len;
 			sp += len;
 			if( *sp != 0 )
-				*np++ = '.';
+				setVStrPtrInc(np,'.');
 		}
 	}
-	*np = 0;
-	return sp;
+	setVStrEnd(np,0);
+	return (const char*)sp;
 }
-static octet *scanQuery(msg,question,name,typep,classp)
-	octet *msg,*question,*name;
-	int *typep,*classp;
-{	octet *qp;
+static const char *scanQuery(PCStr(msg),PCStr(question),PVStr(name),int nsiz,int *typep,int *classp)
+{	const char *qp;
 
 	qp = question;
-	qp = scanName(msg,qp,name,0);
+	qp = scanName(msg,qp,AVStr(name),nsiz,0);
 	getShort(qp,*typep);
 	getShort(qp,*classp);
 	return qp;
 }
-static octet *dumpQuestion(qdi,msg,question)
-	octet *msg,*question;
-{	octet *qp;
-	octet name[512];
-	int type,class;
+static const char *dumpQuestion(int qdi,PCStr(msg),PCStr(question))
+{	const char *qp;
+	CStr(name,512);
+	int type,xclass;
 	int nid;
 
-	qp = scanQuery(msg,question,name,&type,&class);
+	qp = scanQuery(msg,question,AVStr(name),sizeof(name),&type,&xclass);
 
 	nid = DNS_putbyname(name);
-	debug(DBG_NS,"QUE[%2d]%8s: <%d>%s %s %s\n",
-		qdi,"",nid,name,symCLASS(class),symTYPE(type));
+	debug(DBG_NS|DBG_QANDR,"QUE[%2d]%8s: <%d>%s %s %s\n",
+		qdi,"",nid,name,symCLASS(xclass),symTYPE(type));
 	DNS_putbyname(name);
 	return qp;
 }
 
-static hexd(fmt,col,str,len)
-	char *fmt;
-	octet *str;
+static void hexd(PCStr(fmt),int col,octet *str,int len)
 {	int i;
 	for( i = 0; i < len; i++ ){
 		if( (i % col) == 0 ){
@@ -251,15 +270,19 @@ static hexd(fmt,col,str,len)
 	}
 	debug(DBG_ALL,"\n");
 }
-static octet *dumpResourceRecord(whatx,rri,msg,size,rr)
-	octet *msg,*rr;
-{	octet name[512],rname[512];
-	octet *mp;
-	int type,class,ttl,rdlength;
-	char *what,*stype,*sclass;
+static
+char *dumpResourceRecord(int whatx,int rri,PCStr(msg),int size,PCStr(rr))
+{	CStr(name,512);
+	CStr(rname,512);
+	const char *mp;
+	int type,xclass,ttl,rdlength;
+	const char *what;
+	const char *stype;
+	const char *sclass;
 	int nid,rnid;
-	char addr[32];
-	char *rp,rdata[512];
+	CStr(addr,32);
+	CStr(rdata,512);
+	refQStr(rp,rdata);
 	int pref;
 
 	what = SYM_RR[whatx];
@@ -269,22 +292,22 @@ static octet *dumpResourceRecord(whatx,rri,msg,size,rr)
 		return 0;
 	}
 
-	mp = scanName(msg,rr,name,0);
+	mp = scanName(msg,rr,AVStr(name),sizeof(name),0);
 	nid = DNS_putbyname(name);
 
 	getShort(mp,type);
-	getShort(mp,class);
+	getShort(mp,xclass);
 	getLong(mp,ttl);
 	getShort(mp,rdlength);
 
-	if( type <= 0 || 16 < type || class <= 0 ||  4 < class ){
+	if( type <= 0 || 16 < type || xclass <= 0 ||  4 < xclass ){
 debug(DBG_ANY,"### %s[%2d]%8d: <%d>%d class=%x type=%x\n",
-	what,rri,ttl,nid,name,class,type);
+	what,rri,ttl,nid,name,xclass,type);
 		return 0;
 	}
 
 	stype = symTYPE(type);
-	sclass = symCLASS(class);
+	sclass = symCLASS(xclass);
 
 	debug(DBG_RR,"%s[%2d]%8d: <%d>%s %s %s\n",
 		what,rri,ttl,nid,name,sclass,stype);
@@ -292,46 +315,69 @@ debug(DBG_ANY,"### %s[%2d]%8d: <%d>%d class=%x type=%x\n",
 	rnid = 0;
 
 	rp = rdata;
-	*rp++ = type;
-	*rp++ = class;
+	setVStrPtrInc(rp,type);
+	setVStrPtrInc(rp,xclass);
 
 	switch( type ){
 	    case TY_A:
 		sprintf(addr,"%d.%d.%d.%d",mp[0],mp[1],mp[2],mp[3]);
 		rnid = DNS_putbyaddr(addr);
-		*rp++ = mp[0];
-		*rp++ = mp[1];
-		*rp++ = mp[2];
-		*rp++ = mp[3];
+		setVStrPtrInc(rp,mp[0]);
+		setVStrPtrInc(rp,mp[1]);
+		setVStrPtrInc(rp,mp[2]);
+		setVStrPtrInc(rp,mp[3]);
 		DNS_putattr(nid,typemask(type),ttl,rdata,6);
 		debug(DBG_RR,"<%d>%d.%d.%d.%d\n",
 			rnid,mp[0],mp[1],mp[2],mp[3]);
+
+/*
+may be useful for inverse query, but could be bad for consistency...
+ {
+rp = rdata;
+*rp++ = TY_PTR;
+rp++;
+*rp++ = 0xFF & (nid >> 24);
+*rp++ = 0xFF & (nid >> 16);
+*rp++ = 0xFF & (nid >>  8);
+*rp++ = 0xFF & (nid >>  0);
+DNS_putattr(rnid,typemask(TY_PTR),ttl,rdata,6);
+debug(DBG_FORCE,"%s <%d>%d.%d.%d.%d\n",name,rnid,mp[0],mp[1],mp[2],mp[3]);
+ }
+*/
 		break;
 
 	    case TY_MX:
 	    case TY_NS: case TY_CNAME: case TY_PTR:
 		if( type != TY_MX )
-			scanName(msg,mp,rname,0);
-		else	scanName(msg,mp+2,rname,0);
+			scanName(msg,mp,AVStr(rname),sizeof(rname),0);
+		else	scanName(msg,mp+2,AVStr(rname),sizeof(rname),0);
+		if( rname[0] == 0 ){
+			debug(DBG_FORCE,"### ignored empty name for %s %s\n",
+				stype,name);
+			break;
+		}
 
 		rnid = DNS_putbyname(rname);
-		*rp++ = 0xFF & (rnid >> 24);
-		*rp++ = 0xFF & (rnid >> 16);
-		*rp++ = 0xFF & (rnid >>  8);
-		*rp++ = 0xFF & (rnid >>  0);
+		setVStrPtrInc(rp,0xFF & (rnid >> 24));
+		setVStrPtrInc(rp,0xFF & (rnid >> 16));
+		setVStrPtrInc(rp,0xFF & (rnid >>  8));
+		setVStrPtrInc(rp,0xFF & (rnid >>  0));
 		if( type == TY_MX ){
-			*rp++ = mp[0];
-			*rp++ = mp[1];
+			setVStrPtrInc(rp,mp[0]);
+			setVStrPtrInc(rp,mp[1]);
+		DNS_putattr(nid,typemask(type),ttl,rdata,rp-rdata);
 		}
+		else
 		DNS_putattr(nid,typemask(type),ttl,rdata,6);
 		debug(DBG_RR,"%s <%d>%s\n",name,rnid,rname);
 		break;
 	    case TY_SOA:
 {
-char mname[512],rname[512];
+CStr(mname,512);
+CStr(rname,512);
 int serial,refresh,retry,expire,minimum;
-		scanName(msg,mp,mname,0);
-		scanName(msg,mp,rname,0);
+		scanName(msg,mp,AVStr(mname),sizeof(mname),0);
+		scanName(msg,mp,AVStr(rname),sizeof(rname),0);
 		getLong(mp,serial);
 		getLong(mp,refresh);
 		getLong(mp,retry);
@@ -347,39 +393,45 @@ int serial,refresh,retry,expire,minimum;
 		break;
 	}
 	mp += rdlength;
-	return mp;
+	return (char*)mp;
 }
 
 /*
  *	Name Servers
  */
 
-static VSAddr Me;
+static VSAddr *Me;
 
-static makeMysock()
+static int makeMysock()
 {	int len;
 	int rcode;
 	int mysock;
 
+	if( Me == 0 )
+		Me = NewStruct(VSAddr);
+
 	mysock = socket(AF_INET,SOCK_DGRAM,0);
 	if( mysock < 0 ){
 		debug(DBG_FORCE,"RESOLVY cannot get socket() [errno=%d]\n",
 			errno);
 		return mysock;
 	}
-	len = VSA_atosa(&Me,0,"0.0.0.0");
-	rcode = bind(mysock,(SAP)&Me,len);
-	len = sizeof(Me);
-	getsockname(mysock,(SAP)&Me,&len);
-	debug(DBG_ANY,"DNS port = %d/udp\n",VSA_port(&Me));
+	len = VSA_atosa(Me,0,"0.0.0.0");
+	rcode = bind(mysock,(SAP)Me,len);
+	if( rcode < 0 ){
+		debug(DBG_FORCE,"cannot bind() [errno=%d]\n",errno);
+		close(mysock);
+		return -1;
+	}
+	len = sizeof(VSAddr);
+	getsockname(mysock,(SAP)Me,&len);
+	debug(DBG_ANY,"DNS port = %d/udp [%d]\n",VSA_port(Me),mysock);
 	return mysock;
 }
 
-static RR_isin(v1,rrv,rri)
-	octet *v1;
-	octet *rrv[];
+static int RR_isin(PCStr(v1),char *rrv[],int rri)
 {	int ri,vi,leng;
-	octet *r1;
+	const char *r1;
 
 	for( ri = 0; ri < rri; ri++ ){
 		r1 = rrv[ri];
@@ -399,26 +451,31 @@ static RR_isin(v1,rrv,rri)
 }
 
 static int Phase;
-static lookupCache(nid,qtype,rri,rrc,rrv)
-	octet *rrv[];
-{	char dname[512],cdname[512];
-	octet *av[128];
-	octet *a1,*ap;
+static int lookupCache(int nid,int qtype,int rri,int rrc,char *rrv[])
+{	CStr(dname,512);
+	CStr(cdname,512);
+	char *av[128]; /**/
+	const octet *a1;
+	const octet *ap;
 	int ac,ai;
 	int ival;
-	int type,class;
+	int type,xclass;
 	int cnid;
 
 	++Phase;
 	DNS_nodephase(nid,Phase);
-	DNS_nodename(nid,dname);
+	DNS_nodename(nid,AVStr(dname));
 	ac = DNS_getattr(nid,typemask(qtype)|typemask(TY_CNAME),0,128,av);
 	if( 0 < ac ){
 		debug(DBG_CACHE,"lookup <%d>%s [%d]\n",nid,dname,ac);
 		for( ai = 0; ai < ac; ai++ ){
-			a1 = av[ai];
+			if( rrc <= rri ){
+				debug(DBG_FORCE,"too many RR: %s\n",dname);
+				break;
+			}
+			a1 = (octet*)av[ai];
 			type = *a1++;
-			class = *a1++;
+			xclass = *a1++;
 			ap = a1;
 			getLong(ap,ival);
 
@@ -434,12 +491,12 @@ static lookupCache(nid,qtype,rri,rrc,rrv)
 				rrv[rri++] = av[ai];
 			}else
 			if( type == TY_PTR ){
-				DNS_nodename(ival,dname);
+				DNS_nodename(ival,AVStr(dname));
 				debug(DBG_CACHE,"[%d] PTR %s\n",ai,dname);
 				rrv[rri++] = av[ai];
 			}else
 			if( type == TY_CNAME ){
-				DNS_nodename(ival,cdname);
+				DNS_nodename(ival,AVStr(cdname));
 				if( DNS_nodephase(ival,Phase) == Phase ){
 				debug(DBG_ANY,"LOOP[%d] in CNAME: %s -> %s\n",
 						Phase,dname,cdname);
@@ -451,7 +508,7 @@ static lookupCache(nid,qtype,rri,rrc,rrv)
 				rri = lookupCache(cnid,qtype,rri,rrc,rrv);
 			}else
 			if( type == TY_MX ){
-				DNS_nodename(ival,dname);
+				DNS_nodename(ival,AVStr(dname));
 				debug(DBG_CACHE,"[%d] MX %s\n",ai,dname);
 				rrv[rri++] = av[ai];
 			}
@@ -460,10 +517,9 @@ static lookupCache(nid,qtype,rri,rrc,rrv)
 	return rri;
 }
 
-set_nameserver(domain,addr)
-	char *domain,*addr;
+void set_nameserver(PCStr(domain),PCStr(addr))
 {	unsigned int nsnid,nsanid;
-	octet rdata[6];
+	CStr(rdata,6);
 	unsigned int iaddr;
 
 	nsnid = DNS_putbyname(domain);
@@ -486,24 +542,26 @@ set_nameserver(domain,addr)
 	DNS_putattr(nsanid,typemask(TY_A),3600,rdata,6);
 }
 
-static getns1(nid,nsx,nsc,nservv)
-	VSAddr *nservv;
+static int getns1(int nid,int nsx,int nsc,VSAddr *nservv)
 {	VSAddr *ns;
-	octet *nsv[32],*nsav[32],*ap;
+	char *nsv[32]; /**/
+	char *nsav[32]; /**/
+	const char *ap; /* should be unsigend */
 	int nns,nsi,na;
 	int nsnid;
 	int addr;
 
-	char dname[512],sname[512];
+	CStr(dname,512);
+	CStr(sname,512);
 
-	DNS_nodename(nid,dname);
+	DNS_nodename(nid,AVStr(dname));
 	nns = DNS_getattr(nid,typemask(TY_NS),0,32,nsv);
 
 	for( nsi = 0; nsi < nns; nsi++ ){
 		ap = nsv[nns-1-nsi];
 		ap += 2;
 		getLong(ap,nsnid);
-		DNS_nodename(nsnid,sname);
+		DNS_nodename(nsnid,AVStr(sname));
 
 		if( na = DNS_getattr(nsnid,typemask(TY_A),0,32,nsav) ){
 			ap = nsav[0];
@@ -514,7 +572,7 @@ debug(DBG_NS,"        %s[%d] server=<%s %s[%d.%d.%d.%d]>\n",
 dname,nsi,dname,sname,ap[0],ap[1],ap[2],ap[3]);
 
 			ns = &nservv[nsx++];
-			VSA_btosa(ns,AF_INET,ap,53);
+			VSA_btosa(ns,AF_INET,(unsigned char*)ap,53);
 			getLong(ap,addr);
 			if( nsc <= nsx )
 				break;
@@ -525,8 +583,7 @@ dname,nsi,dname,sname,ap[0],ap[1],ap[2],ap[3]);
 	}
 	return nsx;
 }
-static getns(nid,nsc,nservv)
-	VSAddr *nservv;
+static int getns(int nid,int nsc,VSAddr *nservv)
 {	int nsx,nsi;
 	int dnid;
 
@@ -558,13 +615,10 @@ static getns(nid,nsc,nservv)
 	return nsx;
 }
 
-extern char resolv_errmsg[512];
-static dumpResponse(qidbase,resp,rc,hp,addr)
-	octet *resp;
-	Header *hp;
-	char *addr;
-{	octet *mp;
-	char qname[1024];
+extern struct { defQStr(resolv_errmsg); } resolv_errmsg;
+static int dumpResponse(int qidbase,PVStr(resp),int rc,Header *hp,PCStr(addr))
+{	const char *mp;
+	CStr(qname,1024);
 	int nqd,qdi,rri;
 
 	mp = dumpHeader(resp,hp);
@@ -574,7 +628,7 @@ static dumpResponse(qidbase,resp,rc,hp,addr)
 	qname[0] = 0;
 	if( nqd = hp->qdcount ){
 		for( qdi = 0; qdi < nqd; qdi++ ){
-			scanName(resp,mp,qname,0);
+			scanName(resp,mp,AVStr(qname),sizeof(qname),0);
 			mp = dumpQuestion(qdi,resp,mp);
 		}
 	}
@@ -585,38 +639,39 @@ static dumpResponse(qidbase,resp,rc,hp,addr)
 	for( rri = 0; mp && rri < hp->arcount; rri++ )
 		mp = dumpResourceRecord(RR_ADDITIONAL,rri,resp,rc,mp);
 
-resolv_errmsg[0] = 0;
+setVStrEnd(resolv_errmsg.resolv_errmsg,0);
 if( addr[0] )
-	sprintf(resolv_errmsg,"recv[%s](%d) ",addr,rc);
-sprintf(resolv_errmsg+strlen(resolv_errmsg),
+	sprintf(resolv_errmsg.resolv_errmsg,"recv[%s](%d) ",addr,rc);
+Xsprintf(TVStr(resolv_errmsg.resolv_errmsg),
 "Q[%s] ID=%d/%d AA=%d RD=%d RA=%d RCODE=%d ans,ns,add=%d,%d,%d",
 qname, hp->id,qidbase,
 H_AA(hp->M),H_RD(hp->M),H_RA(hp->M),H_RCODE(hp->M),
 hp->ancount,hp->nscount,hp->arcount);
 
-	debug(DBG_NS,"%s\n",resolv_errmsg);
+	debug(DBG_NS|DBG_QANDR,"ANS: %s\n",resolv_errmsg.resolv_errmsg);
 	return hp->ancount + hp->nscount + hp->arcount;
 }
-static recvResponse(mysock,qidbase,from,hp)
-	VSAddr *from;
-	Header *hp;
+static int recvResponse(int mysock,int qidbase,VSAddr *from,Header *hp)
 {	int fromlen;
 	int rc;
-	octet *mp;
-	octet resp[2048];
-	char addr[64];
+	const char *mp;
+	CStr(resp,2048);
+	CStr(addr,64);
 
 	fromlen = sizeof(*from);
 rc = SOCKS_recvfrom(mysock,(char*)resp,sizeof(resp),0,(SAP)from,&fromlen);
 	if( rc <= 0 )
+	{
+		debug(DBG_FORCE,"recvfrom(%d)=%d [errno=%d]\n",mysock,rc,errno);
 		return -1;
+	}
 	if( fromlen <= 0 ){
 		debug(DBG_FORCE,"recvResponse(%d): rc=%d fromlen=%d\n",
 			mysock,rc,fromlen);
 		return -1;
 	}
 	strcpy(addr,VSA_ntoa(from));
-	return dumpResponse(qidbase,resp,rc,hp,addr);
+	return dumpResponse(qidbase,AVStr(resp),rc,hp,addr);
 }
 
 static int QID;
@@ -626,14 +681,21 @@ static int mysocks[MAX_NS_PARA];	/* connected to each foreground server
 					 * to detect UNREACHABLE error on recv
 					 */
 
-initDNSconn(){
+int initDNSconn(){
 	int mypid;
 	int nsi,mysock1;
 
 	mypid = getpid();
 	if( ownerPID == mypid )
+	{
+		if( file_ISSOCK(mysock0) )
 		return mysock0;
 
+		debug(DBG_FORCE,"DNS socket [%d] broken ??? %d/%d\n",
+			mysock0,ownerPID,mypid);
+		ownerPID = 0;
+	}
+
 	if( ownerPID != 0 ){
 		debug(DBG_NS,"initDNSconn(): previous pid=%d close",
 			ownerPID);
@@ -665,9 +727,7 @@ initDNSconn(){
 
 int RES_BACKGROUND = 1;
 
-getRRbynameaddr(timeout,name,qtype,rrc,rrv)
-	octet *name;
-	char *rrv[];
+int getRRbynameaddr(int timeout,PCStr(name),int qtype,int rrc,char *rrv[])
 {	int wc;
 	int nid;
 	int nhit;
@@ -679,8 +739,8 @@ getRRbynameaddr(timeout,name,qtype,rrc,rrv)
 	int nns,nsi,sns,nque,nrns,rnsi;
 	int nsent,nrecv;
 	int qidbase;
-	char qbuf[1024];
-	octet *qp;
+	CStr(qbuf,1024);
+	refQStr(qp,qbuf);
 	int qlen;
 	int start;
 	int mysock1;
@@ -692,6 +752,7 @@ getRRbynameaddr(timeout,name,qtype,rrc,rrv)
 	Header Head;
 	int M,gotAA;
 	int gotRecovErr; /* got recoverable errors */
+	int nturn;
 
 	nid = DNS_putbyname(name);
 	if( nhit = lookupCache(nid,qtype,0,rrc,rrv) )
@@ -700,9 +761,13 @@ getRRbynameaddr(timeout,name,qtype,rrc,rrv)
 	if( initDNSconn() < 0 )
 		goto EXIT;
 
+	/*
 	qidbase = ++QID;
+	*/
+	QID = (QID + 1) & 0xFFFF;
+	qidbase = QID;
 	qp = makeHeader(qbuf,QID, 0,O_QUERY,0,0,1,0,0,0, 1,0,0,0);
-	qp = makeQuestion(qp,name,qtype,CL_IN);
+	qp = makeQuestion(QVStr(qp,qbuf),name,qtype,CL_IN);
 	if( qp == NULL )
 		goto EXIT;
 
@@ -720,7 +785,7 @@ dumpQuestion(0,qbuf,qbuf+12);
 	nsent = nrecv = 0;
 
 	start = time(0);
-	for(;;){
+	for( nturn = 0; ; nturn++ ){
 		debug(DBG_NS,"queries=%d responses=%d\n",nque,nrns);
 
 		if( nhit = lookupCache(nid,qtype,0,rrc,rrv) )
@@ -752,6 +817,12 @@ dumpQuestion(0,qbuf,qbuf+12);
 			break;
 		}
 
+		if( 0 < nturn ){ /* to escape possible tight loop ... */
+			debug(DBG_FORCE,"*%d q=%d,a=%d, s=%d,r=%d (%ds)\n",
+				nturn,nque,nrns,nsent,nrecv,time(0)-start);
+			msleep(50);
+		}
+
 		fds[0] = mysock0;
 		sns = 0;
 		for( nsi = 0; nsi<nns && sns<MAX_NS_PARA; nsi++ ){
@@ -770,7 +841,7 @@ if( ntry == MAX_RETRY ){
 ns = &qservers[pnsi];
 salen = VSA_size(ns);
 SOCKS_sendto(mysock0,qbuf,qlen,0,(SAP)ns,salen);
-debug(DBG_NS,"sent BACKGROUND, ID=%d, sent=%d, recv=%d, time=%d, %s\n",
+debug(DBG_NS|DBG_QANDR,"sent BACKGROUND, ID=%d, sent=%d, recv=%d, time=%d, %s\n",
 QID,nsent,nrecv,time(0)-start,VSA_ntoa(ns));
 }
 					goto NEXT;
@@ -796,7 +867,7 @@ wc = SOCKS_sendto(mysock1,qbuf,qlen,0,(SAP)ns,salen);
 			if( wc < 0 )
 				qserverx[pnsi] = MAX_RETRY;
 
-			debug(DBG_NS,"sent[%s] (%d) put=%d ID=%d\n",
+			debug(DBG_NS|DBG_QANDR,"sent[%s] (%d) put=%d ID=%d\n",
 				VSA_ntoa(ns),qserverx[pnsi],wc,
 				QID);
 		NEXT:;
@@ -834,6 +905,11 @@ wc = SOCKS_sendto(mysock1,qbuf,qlen,0,(SAP)ns,salen);
 				}
 			}
 			nready = PollIns(timeout,sns+1,fds,rdv);
+			if( nready < 0 ){
+				debug(DBG_FORCE,"poll()=%d [errno=%d] %d:%d,%d\n",
+					nready,errno,sns+1,fds[0],fds[1]);
+				goto EXIT;
+			}
 			if( nready <= 0 ){
 				RSLV_TIMEDOUT = 1;
 				break;
@@ -891,24 +967,36 @@ EXIT:
 	return nhit;
 }
 
-gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
-	octet *name,*rrv[],*rrb;
-	char *cname;
-{	octet *rp;
+static int mxsort(char **rr1,char **rr2)
+{	const char *rp1;
+	const char *rp2;
+	int pri1,pri2;
+
+	rp1 = *rr1;
+	rp2 = *rr2;
+	if( *rp1 != TY_MX || *rp2 != TY_MX )
+		return 0;
+
+	rp1 = *rr1+6; getShort(rp1,pri1);
+	rp2 = *rr2+6; getShort(rp2,pri2);
+	return  pri1 - pri2;
+}
+int gethostbynameaddr_dns(PCStr(name),int qtype,int rrc,char *rrv[],PVStr(rrb),PVStr(cname))
+{	const char *rp;
 	int rri,rro;
 	int ival;
-	char dname[512];
-	octet *rrp;
+	CStr(dname,512);
+	refQStr(rrp,rrb);
 	int start;
 	int timeout;
 	int type;
 	int mx,nmx,mxi,rmx;
-	octet *mxv[64];
+	char *mxv[64]; /**/
 
 	mx = qtype == TY_A && strncmp(name,"-MX.",4) == 0;
 	if( mx ){
 		name += 4;
-		nmx = gethostbynameaddr_dns(name,TY_MX,rrc,rrv,rrb,cname);
+		nmx = gethostbynameaddr_dns(name,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
 		/* multiple MXs should be sorted by their preference...
 		 * - returned in -PREFERENCE-VALUE.mx-host.domain
 		 * - sort 
@@ -916,21 +1004,26 @@ gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
 		 */
 		if( nmx == 0 && cname[0] != 0 ){
 			/* not MX but CNAME is cached */
-			char cn[512];
+			CStr(cn,512);
 			strcpy(cn,cname);
-			cname[0] = 0;
-			nmx = gethostbynameaddr_dns(cn,TY_MX,rrc,rrv,rrb,cname);
+			setVStrEnd(cname,0);
+			nmx = gethostbynameaddr_dns(cn,TY_MX,rrc,rrv,AVStr(rrb),AVStr(cname));
 		}
 		if( nmx <= 0 )
-			return gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname);
+			return gethostbynameaddr_dns(name,qtype,rrc,rrv,AVStr(rrb),AVStr(cname));
 
-		for( mxi = 0; mxi < nmx; mxi++ )
-			mxv[mxi] = rrv[mxi];
-		rmx = 0;
-		rrp = rrb;
+		if( elnumof(mxv) <= nmx )
+			nmx = elnumof(mxv);
 		for( mxi = 0; mxi < nmx; mxi++ ){
+			mxv[mxi] = rrv[mxi];
+		}
+		rmx = 0;
+		rrp = (char*)rrb;
+		for( mxi = 0; mxi < nmx; mxi++ ){
+			if( strncasecmp(mxv[mxi],"-MX",3) == 0 )
+				continue;
 			rmx += gethostbynameaddr_dns(mxv[mxi],TY_A,
-				rrc-rmx,rrv+rmx,rrp,NULL);
+				rrc-rmx,rrv+rmx,AVStr(rrp),VStrNULL);
 			rrp = rrv[rmx+1];
 		}
 		return rmx;
@@ -942,7 +1035,11 @@ gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
 	else	timeout = RSLV_INV_TIMEOUT;
 
 	rrc = getRRbynameaddr(timeout,name,qtype,rrc,rrv);
-	rrp = rrb;
+	rrp = (char*)rrb;
+
+	if( qtype == TY_MX && 0 < rrc ){
+		qsort(rrv,rrc,sizeof(char*),(sortFunc)mxsort);
+	}
 
 	rro = 0;
 	for( rri = 0; rri < rrc; rri++ ){
@@ -954,8 +1051,8 @@ gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
 		    case TY_A:
 			/*printf("[%d] %d.%d.%d.%d\n",
 				rri,rp[0],rp[1],rp[2],rp[3]);*/
-			rrv[rro] = rrp;
-			bcopy(rp,rrp,4);
+			rrv[rro] = (char*)rrp;
+			Bcopy(rp,rrp,4);
 			rrp += 4;
 			rro++;
 			break;
@@ -967,14 +1064,14 @@ gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
 
 		    case TY_PTR:
 			getLong(rp,ival);
-			DNS_nodename(ival,dname);
+			DNS_nodename(ival,AVStr(dname));
 			if( type == TY_CNAME ){
 				if( cname != 0 )
 					strcpy(cname,dname);
 				if( qtype != TY_CNAME && qtype != TY_PTR )
 					continue;
 			}
-			rrv[rro] = rrp;
+			rrv[rro] = (char*)rrp;
 			strcpy(rrp,dname);
 			rrp += strlen(rrp)+1;
 			rro++;
@@ -982,35 +1079,36 @@ gethostbynameaddr_dns(name,qtype,rrc,rrv,rrb,cname)
 		}
 	}
 	rrv[rro] = 0;
-	rrv[rro+1] = rrp; /* return value to MX search */
+	rrv[rro+1] = (char*)rrp; /* return value to MX search */
 
 	debug(DBG_NS,"%d seconds, %d(%d) records\n",time(0)-start,rro,rrc);
 	return rro;
 }
 
 
-char *DNS_DOMAIN;
-char *DNS_ORIGIN;
-char *DNS_ADMIN;
+const char *DNS_DOMAIN;
+const char *DNS_ORIGIN;
+const char *DNS_ADMIN;
+const char *DNS_MX;
 int   DNS_SERIAL;
 int   DNS_REFRESH;
 int   DNS_RETRY;
 int   DNS_EXPIRE;
 int   DNS_MINTTL;
 
-static struct hostent *gethostbyN(name)
-	char *name;
+static struct hostent *gethostbyN(PCStr(name))
 {	struct hostent *ht;
-	octet xname[512],*dp;
+	CStr(xname,512);
+	const char *dp;
 
 	ht = NULL;
 	if( dp = strcasestr(name,DNS_DOMAIN) )
 	if( dp[strlen(DNS_DOMAIN)] == 0 ){
 		strcpy(xname,name);
 		dp = strcasestr(xname,DNS_DOMAIN);
-		*dp = 0;
+		truncVStr(dp);
 		if( xname < dp && dp[-1] == '.' )
-			dp[-1] = 0;
+			((char*)dp)[-1] = 0;
 		ht = _GETHOSTBYNAME(xname);
 	}
 	if( ht == NULL )
@@ -1021,13 +1119,25 @@ static struct hostent *gethostbyN(name)
 	}
 	return ht;
 }
-static putRR_A(rrpp,name)
-	octet **rrpp;
-	char *name;
-{	octet *rrp;
+static
+char *putRR(PVStr(rrpa),PCStr(name),int type,int ttl,int length,PCStr(data))
+{	refQStr(rrp,rrpa);
+
+	rrp = putName(AVStr(rrp),name);
+	putShort((char*)rrp,type);
+	putShort((char*)rrp,CL_IN);
+	putLong((char*)rrp,ttl);
+	putShort((char*)rrp,length);
+	Bcopy(data,rrp,length);
+	rrp += length;
+	return (char*)rrp;
+}
+static int putRR_A(PVStr(rrb),char **rrpp,PCStr(name))
+{	refQStr(rrp,rrb);
 	struct hostent *ht;
 	int anc,rttl,rrdlength;
 	int ai;
+	const char *baddr;
 
 	rrp = *rrpp;
 
@@ -1039,24 +1149,67 @@ static putRR_A(rrpp,name)
 	rttl = DNS_MINTTL;
 	rrdlength = ht->h_length;
 	for( ai = 0; ht->h_addr_list[ai] != 0; ai++ ){
-		rrp = putName(rrp,name);
-		putShort(rrp,TY_A);
-		putShort(rrp,CL_IN);
-		putLong(rrp,rttl);
-		putShort(rrp,rrdlength);
-		bcopy(ht->h_addr_list[ai],rrp,rrdlength);
-		rrp += rrdlength;
+		baddr = ht->h_addr_list[ai];
+		rrp = putRR(AVStr(rrp),name,TY_A,rttl,rrdlength,baddr);
 		anc++;
 	}
 
-	*rrpp = rrp;
+	*rrpp = (char*)rrp;
 	return anc;
 }
-static putRR_SOA(rrpp,name)
-	octet **rrpp;
-	char *name;
-{	octet *rrp;
-	octet rbuff[512],*np;
+static
+int putRR_name(char **rrpp,xPVStr(rrp),struct hostent *ht,int type,PCStr(name))
+{	int anc;
+	int rttl,rrdlength;
+	CStr(rbuff,512);
+	refQStr(np,rbuff);
+
+	anc = 0;
+	rttl = DNS_MINTTL;
+
+	if( type == TY_MX ){
+		struct hostent *ht1;
+		int ai,ax;
+		const char *baddr;
+		char baddrs[16][16]; /**/
+		int bleng,btype;
+		int pri;
+
+		bleng = ht->h_length;
+		btype = ht->h_addrtype;
+		for( ax = 0; ax < 16 && (baddr = ht->h_addr_list[ax]); ax++ )
+			bcopy(baddr,baddrs[ax],bleng);
+
+		for( ai = 0; ai < ax; ai++ ){
+			baddr = baddrs[ai];
+			ht1 = _GETHOSTBYADDR(baddr,bleng,btype);
+			if( ht1 == NULL )
+				continue;
+			pri = 0;
+			np = rbuff;
+			putShort((char*)np,pri);
+			np = putName(QVStr(np,rbuff),ht1->h_name);
+			rrdlength = np - rbuff;
+			rrp = putRR(AVStr(rrp),name,type,rttl,rrdlength,rbuff);
+			anc++;
+		}
+	}else{
+		{
+			np = rbuff;
+			np = putName(QVStr(np,rbuff),ht->h_name);
+			rrdlength = np - rbuff;
+			rrp = putRR(AVStr(rrp),name,type,rttl,rrdlength,rbuff);
+			anc++;
+		}
+	}
+
+	*rrpp = (char*)rrp;
+	return anc;
+}
+static int putRR_SOA(PVStr(rrb),char **rrpp,PCStr(name))
+{	refQStr(rrp,rrb);
+	CStr(rbuff,512);
+	refQStr(np,rbuff);
 	int anc,rttl,rrdlength;
 	struct hostent *ht;
 
@@ -1067,37 +1220,103 @@ static putRR_SOA(rrpp,name)
 		return 0;
 
 	np = rbuff;
-	np = putName(np,DNS_ORIGIN);
-	np = putName(np,DNS_ADMIN);
-	putLong(np,DNS_SERIAL);
-	putLong(np,DNS_REFRESH);
-	putLong(np,DNS_RETRY);
-	putLong(np,DNS_EXPIRE);
-	putLong(np,DNS_MINTTL);
+	np = putName(QVStr(np,rbuff),DNS_ORIGIN);
+	np = putName(QVStr(np,rbuff),DNS_ADMIN);
+	PutLong(np,DNS_SERIAL);
+	PutLong(np,DNS_REFRESH);
+	PutLong(np,DNS_RETRY);
+	PutLong(np,DNS_EXPIRE);
+	PutLong(np,DNS_MINTTL);
 	rrdlength = np - rbuff;
 
 	anc = 1;
 	rttl = 0;
-	rrp = putName(rrp,DNS_DOMAIN);
-	putShort(rrp,TY_SOA);
-	putShort(rrp,CL_IN);
-	putLong(rrp,rttl);
-	putShort(rrp,rrdlength);
-	bcopy(rbuff,rrp,rrdlength);
+	rrp = putName(AVStr(rrp),DNS_DOMAIN);
+	PutShort(rrp,TY_SOA);
+	PutShort(rrp,CL_IN);
+	PutLong(rrp,rttl);
+	PutShort(rrp,rrdlength);
+	Bcopy(rbuff,rrp,rrdlength);
 	rrp += rrdlength;
 
-	*rrpp = rrp;
+	*rrpp = (char*)rrp;
 	return anc;
 }
-static putRR_PTR(rrpp,name)
-	octet **rrpp;
-	char *name;
-{	octet *rrp;
+static int relay_cachedRR(PVStr(rrb),char **rrrp,PCStr(name),int qtype)
+{	int anc,nid,ac,ai,type,xclass,ival,pri,rttl,rrdlength;
+	refQStr(rrp,rrb);
+	CStr(rbuff,512);
+	refQStr(np,rbuff);
+	char *av[8]; /**/
+	const char *a1;
+	const char *ap;
+	CStr(dname,512);
+
+	anc = 0;
+	rrp = *rrrp;
+
+	if( nid = DNS_getbyname(name) )
+	if( ac = DNS_getattr(nid,typemask(qtype),0,8,av) )
+	for( ai = 0; ai < ac; ai++ ){
+		a1 = av[ai];
+		type = *a1++;
+		if( type != qtype )
+			continue;
+		xclass = *a1++;
+		ap = a1;
+		getLong(ap,ival);
+		np = rbuff;
+
+		getShort(ap,pri); /* MX preference */
+		putShort((char*)np,pri);
+
+		DNS_nodename(ival,AVStr(dname));
+		np = putName(QVStr(np,rbuff),dname);
+		rttl = DNS_MINTTL;
+		rrdlength = np - rbuff;
+		rrp = putRR(QVStr(rrp,rrb),name,type,rttl,rrdlength,rbuff);
+		anc++;
+	}
+	*rrrp = (char*)rrp;
+	return anc;
+}
+
+/*
+ * to be used in hosts or NIS as
+ *    a1.a2.a3.a4 hostA -MX.hostB
+ *    b1.b2.b3.b4 hostB
+ */
+static int putRR_MX(PVStr(rrb),char **rrpp,PCStr(name))
+{	refQStr(rrp,rrb);
+	struct hostent *ht;
+	CStr(mxname,512);
+	int anc;
+
+	rrp = *rrpp;
+
+	sprintf(mxname,"-MX.%s",name);
+	ht = gethostbyN(mxname);
+
+	if( ht != NULL )
+	if( anc = relay_cachedRR(AVStr(rrb),rrpp,name,TY_MX) )
+		return anc;
+
+	if( ht == NULL ){
+		if( DNS_MX )
+			ht = gethostbyN(DNS_MX);
+		if( ht == NULL )
+			ht = gethostbyN(name);
+		if( ht == NULL )
+			return 0;
+	}
+
+	return putRR_name(rrpp,AVStr(rrp),ht,TY_MX,name);
+}
+static int putRR_PTR(PVStr(rrb),char **rrpp,PCStr(name))
+{	refQStr(rrp,rrb);
 	struct hostent *ht;
-	octet rbuff[512],*np;
-	int anc,rttl,rrdlength;
 	VSAddr sab;
-	char *baddr;
+	const char *baddr;
 	int bleng,btype;
 
 	rrp = *rrpp;
@@ -1111,105 +1330,128 @@ static putRR_PTR(rrpp,name)
 	if( ht == NULL )
 		return 0;
 
-	anc = 0;
-	rttl = DNS_MINTTL;
-	np = putName(rbuff,ht->h_name);
-	rrdlength = np - rbuff;
-	{
-		rrp = putName(rrp,name);
-		putShort(rrp,TY_PTR);
-		putShort(rrp,CL_IN);
-		putLong(rrp,rttl);
-		putShort(rrp,rrdlength);
-		bcopy(rbuff,rrp,rrdlength);
-		rrp += rrdlength;
-		anc++;
-	}
-	/*
-	for( ai = 0; ht->h_aliases[ai]; ai++ )
-		anc++;
-	*/
-
-	*rrpp = rrp;
-	return anc;
+	return putRR_name(rrpp,AVStr(rrp),ht,TY_PTR,name);
 }
 
-dns_search(reply,query,qcc)
-	octet *reply,*query;
-{	octet *qp,*rp;
+int dns_search(PVStr(reply),PCStr(query),int qcc,PCStr(froma),int fromp)
+{	const char *qp;
+	refQStr(rp,reply);
 	Header Head,RHead;
 	int QM,RM,qdc,anc,nsc,arc;
-	octet name[512];
-	octet RRbuf[2048],*rrp;
+	CStr(name,512);
+	CStr(RRbuf,2048);
+	refQStr(rrp,RRbuf);
 	int qtype,qclass;
 	int rtype,rclass,rttl,rrdlength;
-	octet rbuff[512],*np;
 	int rleng;
+	int unknown;
+	int isauth;
+	refQStr(xrrp,RRbuf);
 
 	qp = query;
 	qp = scanHeader(qp,&Head);
-	qp = scanQuery(query,qp,name,&qtype,&qclass);
+	qp = scanQuery(query,qp,AVStr(name),sizeof(name),&qtype,&qclass);
 
-	debug(DBG_FORCE,"QUERY %s %s %s\n",
-		name,symCLASS(qclass),symTYPE(qtype));
+	debug(DBG_FORCE,"QUERY %s %s %s ID=%d\n",
+		name,symCLASS(qclass),symTYPE(qtype),Head.id);
+	setVStrEnd(resolv_errmsg.resolv_errmsg,0);
+
+	if( H_QR(Head.M) != 0 ){
+		debug(DBG_FORCE,"Got response as query, ignored.\n");
+		return -1;
+	}
 
 	rrp = RRbuf;
 
 	anc = 0;
 	switch( qtype ){
-	    case TY_SOA:anc += putRR_SOA(&rrp,name); break;
-	    case TY_A:  anc += putRR_A(&rrp,name);   break;
-	    case TY_PTR:anc += putRR_PTR(&rrp,name); break;
+	    case TY_SOA:anc += putRR_SOA(AVStr(RRbuf),(char**)&rrp,name); break;
+	    case TY_A:  anc += putRR_A(AVStr(RRbuf),(char**)&rrp,name);   break;
+	    case TY_PTR:anc += putRR_PTR(AVStr(RRbuf),(char**)&rrp,name); break;
 
 	    case TY_NS:
 		break;
 	    case TY_MX:
+	    case TY_QMAILA:
+		anc += putRR_MX(AVStr(RRbuf),(char**)&rrp,name); break;
 		break;
 
 	    case TY_QALL:
-			anc += putRR_A(&rrp,name);
-			anc += putRR_SOA(&rrp,name);
-			anc += putRR_PTR(&rrp,name);
+			anc += putRR_A(AVStr(RRbuf),(char**)&rrp,name);
+			anc += putRR_MX(AVStr(RRbuf),(char**)&rrp,name);
+			anc += putRR_SOA(AVStr(RRbuf),(char**)&rrp,name);
+			anc += putRR_PTR(AVStr(RRbuf),(char**)&rrp,name);
 			break;
 	}
 
+	xrrp = rrp;
+	unknown = anc==0 && putRR_A(AVStr(RRbuf),(char**)&xrrp,name)==0 && putRR_PTR(AVStr(RRbuf),(char**)&xrrp,name)==0;
+	isauth = strtailstr(name,DNS_DOMAIN) != NULL;
+
 	QM = Head.M;
 	RM = 0;
 	SET_QR(RM,1);
 	SET_OPCODE(RM,H_OPCODE(QM));
+	/*
 	SET_AA(RM,1);
+	*/
+	if( !unknown || isauth )
+		SET_AA(RM,1);
+	else	SET_AA(RM,0);
+
 	SET_RD(RM,H_RD(QM));
 	SET_RA(RM,0);
 	if( anc == 0 )
+	{
+		/*
+		if( gethostbyN(name) == NULL ){
 		SET_RCODE(RM,3);
+		}
+		*/
+		if( unknown ){
+		    if( isauth )
+			SET_RCODE(RM,3); /* UNKNOWN */
+		    else{
+			debug(DBG_FORCE,"%s<-REFUSED %s (DNSCONF=domain:%s)\n",
+				froma,name,DNS_DOMAIN);
+			SET_RCODE(RM,5); /* REFUSED */
+		    }
+		}
+	}
 	qdc = 1;
 
-	rp = reply;
-	putShort(rp,Head.id);
-	putShort(rp,RM);
-	putShort(rp,qdc);
-	putShort(rp,anc);
-	putShort(rp,0);
-	putShort(rp,0);
-	rp = makeQuestion(rp,name,qtype,qclass);
+	cpyQStr(rp,reply);
+	putShort((char*)rp,Head.id);
+	putShort((char*)rp,RM);
+	putShort((char*)rp,qdc);
+	putShort((char*)rp,anc);
+	putShort((char*)rp,0);
+	putShort((char*)rp,0);
+	rp = makeQuestion(QVStr(rp,reply),name,qtype,qclass);
 
 	if( anc ){
-		bcopy(RRbuf,rp,rrp-RRbuf);
+		Bcopy(RRbuf,rp,rrp-RRbuf);
 		rp += (rrp-RRbuf);
 	}
 
 	rleng = rp - reply;
-	dumpResponse(Head.id,reply,rleng,&RHead,"");
+	dumpResponse(Head.id,AVStr(reply),rleng,&RHead,"");
+
+	if( resolv_errmsg.resolv_errmsg[0] ){
+		debug(DBG_FORCE,"ANSWER %s [ID=%d]\n",
+			resolv_errmsg.resolv_errmsg,QID);
+	}
 	return rleng;
 }
 
-dns_server(qsock,rsock)
+void dns_server(int qsock,int rsock)
 {	int icc,occ;
-	char ib[2048],ob[2048];
+	CStr(ib,2048);
+	CStr(ob,2048);
 
 	icc = read(qsock,ib,sizeof(ib));
-	occ = dns_search(ob,ib,icc);
+	occ = dns_search(AVStr(ob),ib,icc,"",0);
 	write(rsock,ob,occ);
 }
-int (*RES_DNSSERVER)() = dns_server;
-int (*RES_DNSSEARCH)() = dns_search;
+void (*RES_DNSSERVER)(int,int) = dns_server;
+int (*RES_DNSSEARCH)(PVStr(r),const char*,int,const char*,int) = dns_search;
