@@ -45,18 +45,19 @@ Description:
 History:
 	980609	created
 //////////////////////////////////////////////////////////////////////#*/
+#include <errno.h>
+#include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
+#include "ystring.h"
+#include "vsocket.h"
+#include "proc.h"
+#include "file.h"
+#include "fpoll.h"
+#include "dglib.h"
 #include <ctype.h>
-extern double Time();
-extern char *getenv();
-extern char *gethostaddr();
-extern char *cachedir();
-extern FILE *dirfopen();
-extern char *frex_create(),*frex_match();
-extern char *_inet_ntoaV4I();
+
 extern int CHILD_SERNO;
-extern CHILD_SERNO_MULTI;
+extern int CHILD_SERNO_MULTI;
 
 #define MY_ICP_VER	2
 #define MAX_PACKSIZE	(16*1024)
@@ -89,7 +90,7 @@ int ICP_DEBUGLOG = 0;
 #define ICP_FLAG_VALIDS	\
 	(ICP_FLAG_HIT_OBJ|ICP_FLAG_SRC_RTT|ICP_FLAG_X_NAVI)
 
-static char *opsym(op){
+static const char *opsym(int op){
 	switch( op ){
 		case ICP_OP_INVALID:	return "INVALID";
 		case ICP_OP_QUERY:	return "QUERY";
@@ -122,13 +123,13 @@ typedef struct {
 #define HEADSIZE	sizeof(ICPhead)
 
 typedef struct {
-	ipv4a	ib_requester;
-	char	ib_url[MAX_PACKSIZE-HEADSIZE-IPV4SIZE];
+	ipv4a	iq_requester;
+	MStr(	iq_url,MAX_PACKSIZE-HEADSIZE-IPV4SIZE);
 } ICPbody_QUERY;
 
 typedef union {
 	ICPbody_QUERY	ib_query;
-	char	ib_url[MAX_PACKSIZE-HEADSIZE];
+	MStr(	ib_url,MAX_PACKSIZE-HEADSIZE);
 } ICPbody;
 
 typedef struct {
@@ -145,8 +146,10 @@ typedef struct {
 #define IH_SENDER	i_head.ih_sender
 
 #define IB_URL		i_body.ib_url
-#define IQ_REQUESTER	i_body.ib_query.ib_requester
-#define IQ_URL		i_body.ib_query.ib_url
+/**/
+#define IQ_REQUESTER	i_body.ib_query.iq_requester
+#define IQ_URL		i_body.ib_query.iq_url
+/**/
 
 #define ICP_ITYPE_SIBLING	0x1000
 #define ICP_ITYPE_PARENT	0x0001
@@ -160,9 +163,8 @@ typedef struct {
 #define ICP_PTYPES	       (0x00F0|ICP_PTYPE_HTTP)
 #define ICP_TEMP_FLAGS		0xF000
 
-static bdump(msg,pack,leng)
-	unsigned char *msg,*pack;
-{	unsigned char *pp;
+static void bdump(xPVStr(msg),PCStr(pack),int leng)
+{	const unsigned char *pp;
 	int px;
 
 	pp = (unsigned char*)pack;
@@ -177,14 +179,13 @@ static bdump(msg,pack,leng)
 		msg += strlen(msg);
 	}
 }
-static dumppack(msg,leng,pack)
-	char *msg;
-	ICPpack *pack;
+static void dumppack(xPVStr(msg),int leng,ICPpack *pack)
 {	int op;
-	char *sym,*psym;
-	char *url;
+	const char *sym;
+	const char *psym;
+	const char *url;
 	int opt,optdata;
-	char opts[32];
+	CStr(opts,32);
 
 	op = pack->IH_OPCODE;
 	psym = sym = opsym(op);
@@ -243,14 +244,12 @@ static dumppack(msg,leng,pack)
 }
 
 /* if the RTT to the destination server is long ? */
-static char *noicp_domain;
-static MISScode(qpack,host,port,froma,fromp)
-	ICPpack *qpack;
-	char *host,*froma;
-{	char *tail;
+static void *noicp_domain;
+static int MISScode(ICPpack *qpack,PCStr(host),int port,PCStr(froma),int fromp)
+{	const char *tail;
 
 	if( noicp_domain )
-	if( tail = frex_match(noicp_domain,host) )
+	if( tail = frex_match((struct fa_stat*)noicp_domain,host) )
 	if( *tail == 0 )
 		return ICP_OP_MISS_NOFETCH;
 
@@ -260,8 +259,7 @@ static MISScode(qpack,host,port,froma,fromp)
 	return ICP_OP_MISS;
 }
 
-static valid_object(url,obj,size)
-	char *url,*obj;
+static int valid_object(PCStr(url),PCStr(obj),int size)
 {
 	if( strncmp(obj,"HTTP/",5) == 0 )
 		return 1;
@@ -275,17 +273,28 @@ static int nMiss;
 static int nErr;
 static int nDenied;
 static int nUnknown;
+extern int errorECONNRESET;
+#ifndef ECONNRESET
+#define ECONNRESET -1
+#endif
 
-icp_server(sock)
+static void icp_reply(int sock,ICPpack *qpack,ICPpack *rpack,PCStr(froma),int fromp,double start,PCStr(url));
+int icp_select(PCStr(url),int icpopts[],const char *svaddrs[],int svports[],double timeout,FILE *log,PVStr(sxaddr),int *sxport,int first,int *hitobj,PVStr(objbuf));
+int icp_selectconf(PVStr(conf),PCStr(dproto),PCStr(dhost),int dport,PCStr(shost),int sport,PCStr(suser));
+int icp_getconf(PCStr(conf),int icptypes[],const char *icpaddrs[],int icpports[],const char *pxaddrs[],int pxports[],double *timeout,PVStr(buff));
+void scan_ICPCONF(DGC*Conn,PCStr(conf));
+void scan_ICP(DGC*Conn,PCStr(conf));
+
+void icp_server(int sock)
 {	ICPpack qpack,rpack;
-	char froma[32];
+	CStr(froma,32);
 	int fromp;
 	int rcc;
-	char *url;
-	char msg[128+MAX_PACKSIZE];
+	const char *url;
+	CStr(msg,128+MAX_PACKSIZE);
 	double start,done;
 	int parent,mypid;
-	char *env;
+	const char *env;
 	int npara,ni;
 
 	setsockbuf(sock,64*1024,0);
@@ -317,17 +326,30 @@ icp_server(sock)
 	done = Time();
 
 	for( nRecv = 1;; nRecv++ ){
-		rcc = RecvFrom(sock,&qpack,sizeof(qpack),froma,&fromp);
+		errno = 0;
+		errorECONNRESET = 0;
+		rcc = RecvFrom(sock,(char*)&qpack,sizeof(qpack),AVStr(froma),&fromp);
+		if( rcc <= 0 || strcmp(froma,"0.0.0.0") == 0 ){
+			if( errorECONNRESET || errno == ECONNRESET ){
+			daemonlog("F","## RecvFrom(%d) = %d ECONNRESET\n",
+				sock,rcc,errno);
+				continue;
+			}
+			daemonlog("F","FATAL RecvFrom(%d) = %d errno=%d\n",
+				sock,rcc,errno);
+			sleep(10);
+			continue;
+		}
 		start = Time();
 		sprintf(msg,"%-5.2f %s ",start-done,froma);
 		lRecv += rcc;
 		CHILD_SERNO_MULTI++;
 
 		if( qpack.IH_OPCODE == ICP_OP_QUERY )
-			dumppack(msg+strlen(msg),rcc,&qpack);
+			dumppack(TVStr(msg),rcc,&qpack);
 		daemonlog("E","%s\n",msg);
 
-		url == NULL;
+		url = NULL;
 		if( qpack.IH_OPCODE == ICP_OP_QUERY )
 			url = qpack.IQ_URL;
 		else	nUnknown++;
@@ -339,18 +361,17 @@ icp_server(sock)
 		done = Time();
 	}
 }
-static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
-	ICPpack *qpack,*rpack;
-	char *froma;
-	double start;
-	char *url;
-{	char proto[256],hostport[1024],host[1024],upath[MAX_PACKSIZE];
+static void icp_reply(int sock,ICPpack *qpack,ICPpack *rpack,PCStr(froma),int fromp,double start,PCStr(url))
+{	CStr(proto,256);
+	CStr(hostport,1024);
+	CStr(host,1024);
+	CStr(upath,MAX_PACKSIZE);
 	int port;
-	char cachepath[MAX_PACKSIZE];
+	CStr(cachepath,MAX_PACKSIZE);
 	int ftime,fsize,rsize,psize,fage;
 	int msglen;
-	char fbuff[MAX_PACKSIZE];
-	char msg[256];
+	CStr(fbuff,MAX_PACKSIZE);
+	CStr(msg,256);
 	FILE *fp;
 	int qopts;
 	int opcode,options,optdata,sender;
@@ -359,8 +380,8 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 	if( url == NULL )
 		return;
 
-	decomp_absurl(url,proto,hostport,upath,sizeof(upath));
-	port = scan_hostport(proto,hostport,host);
+	decomp_absurl(url,AVStr(proto),AVStr(hostport),AVStr(upath),sizeof(upath));
+	port = scan_hostport(proto,hostport,AVStr(host));
 
 	qopts = ntohL(qpack->IH_OPTIONS);
 
@@ -383,7 +404,7 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 	}
 
 	if( qopts & ICP_FLAG_HIT_OBJ )
-	if( !method_permitted0(froma,fromp,"icp","hitobj",host,port) ){
+	if( !method_permitted0(/*froma,fromp,"icp","hitobj",host,port*/) ){
 		qopts &= ~ICP_FLAG_HIT_OBJ;
 	}
 
@@ -394,8 +415,8 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 
 	if( cachedir() != NULL ){
 		opcode = MISScode(qpack,host,port,froma,fromp);
-		cache_path(proto,host,port,upath,cachepath);
-		if( fp = dirfopen("ICP",cachepath,"r") ){
+		cache_path(proto,host,port,upath,AVStr(cachepath));
+		if( fp = dirfopen("ICP",AVStr(cachepath),"r") ){
 			ftime = file_mtime(fileno(fp));
 			fsize = file_size(fileno(fp));
 			fage = time(0) - ftime;
@@ -423,14 +444,18 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 		goto REPLY;
 
 	if( qopts & ICP_FLAG_X_NAVI || ICP_NUMSERV ){
-		char conf[1024],buff[1024];
+		CStr(conf,1024);
+		CStr(buff,1024);
 		int nserv;
 		int icptypes[32];
-		char *icpaddrs[32],*pxaddrs[32];
-		int icpports[32],pxports[32],pxtypes[32];
+		const char *icpaddrs[32]; /**/
+		const char *pxaddrs[32]; /**/
+		int icpports[32];
+		int pxports[32];
+		int pxtypes[32];
 		double timeout;
 		int sx;
-		char sxaddr[32];
+		CStr(sxaddr,32);
 		int sxport;
 		int *hitobjp;
 
@@ -439,10 +464,10 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 
 		/* MOUNT URL here ? */
 
-		if( icp_selectconf(conf,proto,host,port,froma,fromp,"") < 0 )
+		if( icp_selectconf(AVStr(conf),proto,host,port,froma,fromp,"") < 0 )
 			goto REPLY;
 		nserv = icp_getconf(conf,icptypes,icpaddrs,icpports,
-				pxaddrs,pxports,&timeout,buff);
+				pxaddrs,pxports,&timeout,AVStr(buff));
 		if( nserv < 0 )
 			goto REPLY;
 
@@ -450,7 +475,7 @@ static icp_reply(sock,qpack,rpack,froma,fromp,start,url)
 			hitobjp = &rsize;
 		else	hitobjp = NULL;
 		sx = icp_select(url,icptypes,icpaddrs,icpports,timeout,
-			stderr, sxaddr,&sxport,1,hitobjp,fbuff);
+			stderr, AVStr(sxaddr),&sxport,1,hitobjp,AVStr(fbuff));
 		/* multiple responses should be relayed ?? */
 
 		if( 0 <= sx ){
@@ -487,7 +512,7 @@ REPLY:
 		rpack->IH_OPCODE = ICP_OP_HIT;
 		nHit++;
 	}else{
-		rpack->IH_OPCODE = MISScode(qpack,host,port);
+		rpack->IH_OPCODE = MISScode(qpack,host,port,NULL,0);
 		nMiss++;
 	}
 
@@ -507,13 +532,13 @@ REPLY:
 
 	strcpy(rpack->IB_URL,url);
 	if( opcode == ICP_OP_HIT_OBJ ){
-		char *op;
+		MrefQStr(op,rpack->IB_URL); /**/
 		op = rpack->IB_URL+strlen(url)+1;
-		*op++ = rsize >> 24;
-		*op++ = rsize >> 16;
-		*op++ = rsize >> 8;
-		*op++ = rsize;
-		bcopy(fbuff,op,rsize);
+		setVStrPtrInc(op,rsize >> 24);
+		setVStrPtrInc(op,rsize >> 16);
+		setVStrPtrInc(op,rsize >> 8);
+		setVStrPtrInc(op,rsize);
+		Bcopy(fbuff,op,rsize);
 
 		msglen += 4 + rsize;
 		rpack->IH_MSGLEN = htonS(msglen);
@@ -522,7 +547,7 @@ REPLY:
 		/* >> HTTP PROTOLOG */
 	}
 
-	SendTo(sock,rpack,msglen,froma,fromp);
+	SendTo(sock,(char*)rpack,msglen,froma,fromp);
 	sprintf(msg,"%5.3f %s %s %d %db/%3.1fh (%dH+%dM+%dE+%dD+%dU/%d) %d",
 		Time()-start,
 		froma,
@@ -535,28 +560,34 @@ REPLY:
 }
 
 static int ReqId = 100;
-icp_client(ac,av)
-	char *av[];
+int icp_client(int ac,const char *av[])
 {	int ai;
-	char *arg;
-	char *server;
-	char *url;
+	const char *arg;
+	const char *server;
+	const char *url;
 	int serverx;
-	char svhost[128],*svaddr;
+	CStr(svhost,128);
+	const char *svaddr;
 	int svport;
-	char svaddrb[32][128],*svaddrs[32];
+	ACStr(svaddrb,32,128);
+	const char *svaddrs[32]; /**/
 	int svports[32];
 	float fv;
 	FILE *log;
 	int sx;
-	char sxhost[32];
+	CStr(sxhost,32);
 	int sxport;
 	int hitobj;
-	char objbuf[MAX_PACKSIZE];
+	CStr(objbuf,MAX_PACKSIZE);
 	int icpflag,icpopts[32];
 	int first;
-	char proto[128],hostport[128],host[128],upath[1024];
-	char conf[256],*pxaddrs[32],buff[256];
+	CStr(proto,128);
+	CStr(hostport,128);
+	CStr(host,128);
+	CStr(upath,1024);
+	CStr(conf,256);
+	const char *pxaddrs[32]; /**/
+	CStr(buff,256);
 	int port,nserv,pxports[32];
 
 	log = stderr;
@@ -572,15 +603,18 @@ icp_client(ac,av)
 		if( *arg == '-' ){
 			switch( arg[1] ){
 			case 'h':
+				if( elnumof(svaddrs) <= serverx ){
+					break;
+				}
 				server = av[++ai];
 				svport = 3130;
-				sscanf(server,"%[^:]:%d",svhost,&svport);
+				Xsscanf(server,"%[^:]:%d",AVStr(svhost),&svport);
 				svaddr = gethostaddr(svhost);
 				if( svaddr == NULL ){
 					fprintf(stderr,"%s ?\n",svhost);
-					return;
+					return -1;
 				}
-				strcpy(svaddrb[serverx],svaddr);
+				Xstrcpy(EVStr(svaddrb[serverx]),svaddr);
 				svaddrs[serverx] = svaddrb[serverx];
 				svports[serverx] = svport;
 				icpopts[serverx] = icpflag;
@@ -619,18 +653,18 @@ icp_client(ac,av)
 		return -1;
 	}
 
-	decomp_absurl(url,proto,hostport,upath,sizeof(upath));
-	port = scan_hostport(proto,hostport,host);
-	if( icp_selectconf(conf,proto,host,port,"",0,"") == 0 ){
+	decomp_absurl(url,AVStr(proto),AVStr(hostport),AVStr(upath),sizeof(upath));
+	port = scan_hostport(proto,hostport,AVStr(host));
+	if( icp_selectconf(AVStr(conf),proto,host,port,"",0,"") == 0 ){
 		nserv = icp_getconf(conf,
 			&icpopts[serverx],&svaddrs[serverx],&svports[serverx],
-			pxaddrs,pxports,&DFLT_TIMEOUT,buff);
+			pxaddrs,pxports,&DFLT_TIMEOUT,AVStr(buff));
 		if( 0 < nserv )
 			serverx += nserv;
 	}
 
 	if( serverx == 0 ){
-		strcpy(svaddrb[0],"127.0.0.1");
+		Xstrcpy(EVStr(svaddrb[0]),"127.0.0.1");
 		svaddrs[0] = svaddrb[0];
 		svports[0] = 3130;
 		icpopts[0] = icpflag;
@@ -643,33 +677,23 @@ icp_client(ac,av)
 
 	sxport = 0;
 	sx = icp_select(url,icpopts,svaddrs,svports,DFLT_TIMEOUT,log,
-		sxhost,&sxport,first,&hitobj,objbuf);
+		AVStr(sxhost),&sxport,first,&hitobj,AVStr(objbuf));
 	if( 0 <= sx )
 		fprintf(log,"%s:%d\n",sxhost,sxport);
+	return 0;
 }
 
 static int mysock;
 static int mysockPID;
-static myICPsock()
+static int myICPsock()
 {	int pid = getpid();
 	if( mysockPID == pid )
 		return mysock;
 	mysockPID = pid;
-	mysock = Socket1("ICP",-1,"inet","dgram","udp", NULL,0,NULL,0,0,NULL,0);
+	mysock = Socket1("ICP",-1,"inet","dgram","udp", VStrNULL,0,NULL,0,0,NULL,0);
 	return mysock;
 }
-icp_select(url,icpopts,svaddrs,svports,timeout,log,
-sxaddr,sxport,first,hitobj,objbuf)
-	char *url;
-	char *svaddrs[];
-	int svports[];
-	int icpopts[];
-	double timeout;
-	FILE *log;
-	char *sxaddr;
-	int *sxport;
-	int *hitobj;
-	char *objbuf;
+int icp_select(PCStr(url),int icpopts[],const char *svaddrs[],int svports[],double timeout,FILE *log,PVStr(sxaddr),int *sxport,int first,int *hitobj,PVStr(objbuf))
 {	ICPpack qpack,rpack;
 	int nserv,nwatcher;
 	int sock;
@@ -677,10 +701,10 @@ sxaddr,sxport,first,hitobj,objbuf)
 	int sx,nresp;
 	int wcc,rcc;
 	double start,sent,done;
-	char msg[128+MAX_PACKSIZE];
+	CStr(msg,128+MAX_PACKSIZE);
 	int qoptions;
 	int opcode,options,optdata,sender,hsize,rsize;
-	char froma[32];
+	CStr(froma,32);
 	int fromp;
 	int hit = -1;
 	int miss = -1;
@@ -725,9 +749,9 @@ sxaddr,sxport,first,hitobj,objbuf)
 		}
 		qpack.IH_OPTIONS = htonL(qoptions);
 		qpack.IH_REQNUM = htonL((ReqId<<8) | sx);
-		wcc = SendTo(sock,&qpack,msglen,svaddrs[sx],svports[sx]);
+		wcc = SendTo(sock,(char*)&qpack,msglen,svaddrs[sx],svports[sx]);
 		sent = Time();
-		dumppack(msg,msglen,&qpack);
+		dumppack(AVStr(msg),msglen,&qpack);
 		Lprintf(log,"(ICP)[%d][%d] %5.3f [%s:%d]%s<< %s\n",
 			sx+1,sx,sent-start,svaddrs[sx],svports[sx],
 			(icpopts[sx] & ICP_ITYPE_PARENT)?"p":"s", msg);
@@ -740,17 +764,17 @@ sxaddr,sxport,first,hitobj,objbuf)
 	nresp = 0;
 	for(;;){
 		if( nresp < nserv-nwatcher )
-			timeout1 = 1000 * timeout;
+			timeout1 = (int)(1000 * timeout);
 		else
 		if( mcast )
-			timeout1 = 1000 * 0.1;
+			timeout1 = (int)(1000 * 0.1);
 		else	break;
 
 		if( PollIn(sock,timeout1) <= 0 ){
 			Lprintf(log,"(ICP)Timeout %dms\n",timeout1);
 			break;
 		}
-		rcc = RecvFrom(sock,&rpack,sizeof(rpack),froma,&fromp);
+		rcc = RecvFrom(sock,(char*)&rpack,sizeof(rpack),AVStr(froma),&fromp);
 		rreqnum = ntohL(rpack.IH_REQNUM);
 		rreqid = 0xFFFFFF & (rreqnum >> 8);
 		rreqsx = 0xFF & rreqnum;
@@ -803,13 +827,13 @@ sxaddr,sxport,first,hitobj,objbuf)
 		}
 
 		done = Time();
-		dumppack(msg,rcc,&rpack);
+		dumppack(AVStr(msg),rcc,&rpack);
 		Lprintf(log,"(ICP)[%d][%d] %5.3f [%s:%d]%s>> %s\n",
 			nresp,rreqsx,done-start,froma,fromp,
 			(icpopts[rreqsx] & ICP_ITYPE_PARENT)?"p":"s", msg);
 
 		if( opcode == ICP_OP_HIT_OBJ && (options & ICP_FLAG_HIT_OBJ) ){
-			unsigned char *hop;
+			const unsigned char *hop;
 
 			url = rpack.IB_URL;
 			hop = (unsigned char *)(url + strlen(url) + 1);
@@ -820,13 +844,13 @@ sxaddr,sxport,first,hitobj,objbuf)
 			Lprintf(log,"(ICP)HIT_OBJ %d + %d = %d / %d\n",
 				hsize,rsize,hsize+rsize,rcc);
 
-			if( valid_object(url,hop,rsize) )
+			if( valid_object(url,(char*)hop,rsize) )
 			if( hsize + rsize == rcc ){
 				if( hitobj != NULL )
 					*hitobj = rsize;
 				if( objbuf != NULL ){
-					bcopy(hop,objbuf,rsize);
-					objbuf[rsize] = 0;
+					Bcopy(hop,objbuf,rsize);
+					setVStrEnd(objbuf,rsize);
 					break;
 				}
 			}
@@ -841,28 +865,21 @@ sxaddr,sxport,first,hitobj,objbuf)
 	return miss;
 }
 
-
-typedef struct {
-	char	i_addr[32];
-	int	i_port;
-	char	i_pxtype;
-	char	i_pxaddr[32];
-	int	i_pxport;
-} ICPconf;
-
 #define ICPMAP	"ICPmap"
-scan_ICPCONF(Conn,conf)
-	void *Conn;
-	char *conf;
+void scan_ICPCONF(DGC*Conn,PCStr(conf))
 {	int cc,ci;
-	char confb[1024],*confv[32],*conf1,name[128],val[128];
+	CStr(confb,1024);
+	const char *confv[32]; /**/
+	const char *conf1;
+	CStr(name,128);
+	CStr(val,128);
 
 	strcpy(confb,conf);
 	cc = list2vect(confb,',',32,confv);
 	for( ci = 0; ci < cc; ci++ ){
 		conf1 = confv[ci];
 		name[0] = val[0] = 0;
-		sscanf(conf1,"%[^:]:%s",name,val);
+		Xsscanf(conf1,"%[^:]:%s",AVStr(name),AVStr(val));
 
 		if( streq(name,"para") )
 			PARASERV = atoi(val);
@@ -895,13 +912,16 @@ scan_ICPCONF(Conn,conf)
  *    icpServer=icpHost/icpType/proxyPort/icpPort
  *    icpType={s|p|o|n|H|D|P|O}*
  */
-scan_ICP(Conn,conf)
-	void *Conn;
-	char *conf;
-{	char confb[2048],*confv[8];
+void scan_ICP(DGC*Conn,PCStr(conf))
+{	CStr(confb,2048);
+	const char *confv[8]; /**/
 	int cc;
-	char *icphost,*icport,*pxport,*icpopt,*icpxmap;
-	char map[2048];
+	const char *icphost;
+	const char *icport;
+	const char *pxport;
+	const char *icpopt;
+	const char *icpxmap;
+	CStr(map,2048);
 
 	ICP_NUMSERV++;
 	strcpy(confb,conf);
@@ -916,11 +936,12 @@ scan_ICP(Conn,conf)
 	scan_CMAP2(Conn,ICPMAP,map);
 }
 
-static scanoptsG(optlist,timeout)
-	char *optlist;
-	double *timeout;
+static int scanoptsG(PCStr(optlist),double *timeout)
 {	int gf;
-	char optbuf[1024],*optv[32],*opt1,*op;
+	CStr(optbuf,1024);
+	const char *optv[32]; /**/
+	const char *opt1;
+	const char *op;
 	int cc,ci;
 
 	gf = 0;
@@ -944,10 +965,9 @@ static scanoptsG(optlist,timeout)
 	}
 	return gf;
 }
-static scanoptsP(optlist,gf)
-	char *optlist;
+static int scanoptsP(PCStr(optlist),int gf)
 {	int pf;
-	char *op;
+	const char *op;
 
 	pf = 0;
 	for( op = optlist; *op; op++ ){
@@ -973,57 +993,59 @@ static scanoptsP(optlist,gf)
 	pf &= ~ICP_TEMP_FLAGS;
 	return pf;
 }
-static optsym(flag,opts)
-	char *opts;
-{	char *op;
+static void optsym(int flag,PVStr(opts))
+{	refQStr(op,opts); /**/
 
-	op = opts;
-	if( flag & ICP_FLAG_HIT_OBJ	) *op++ = 'o';
-	if( flag & ICP_FLAG_X_NAVI	) *op++ = 'n';
-	if( flag & ICP_ITYPE_PARENT	) *op++ = 'p';
-	if( flag & ICP_ITYPE_LISTENER	) *op++ = 'l'; else *op++ = 's';
+	op = (char*)opts;
+	if( flag & ICP_FLAG_HIT_OBJ	) setVStrPtrInc(op,'o');
+	if( flag & ICP_FLAG_X_NAVI	) setVStrPtrInc(op,'n');
+	if( flag & ICP_ITYPE_PARENT	) setVStrPtrInc(op,'p');
+	if( flag & ICP_ITYPE_LISTENER	) setVStrPtrInc(op,'l'); else
+					  setVStrPtrInc(op,'s');
 
-	if( flag & ICP_PTYPE_DELEGATE	) *op++ = 'D'; else
-	if( flag & ICP_PTYPE_ORIG_PROXY	) *op++ = 'P'; else
-	if( flag & ICP_PTYPE_ORIG_SERVER) *op++ = 'O'; else *op++ = 'H';
-	*op = 0;
+	if( flag & ICP_PTYPE_DELEGATE	) setVStrPtrInc(op,'D'); else
+	if( flag & ICP_PTYPE_ORIG_PROXY	) setVStrPtrInc(op,'P'); else
+	if( flag & ICP_PTYPE_ORIG_SERVER) setVStrPtrInc(op,'O'); else
+					  setVStrPtrInc(op,'H');
+	setVStrEnd(op,0);
 }
 
-icp_selectconf(conf,dproto,dhost,dport,shost,sport,suser)
-	char *conf,*dproto,*dhost,*shost,*suser;
+int icp_selectconf(PVStr(conf),PCStr(dproto),PCStr(dhost),int dport,PCStr(shost),int sport,PCStr(suser))
 {
 	if( ICP_NUMSERV == 0 )
 		return -1;
-	if( find_CMAPX(ICPMAP,conf,dproto,dhost,dport,shost,sport,suser) < 0 )
+	if( find_CMAPX(ICPMAP,AVStr(conf),dproto,dhost,dport,shost,sport,suser) < 0 )
 		return -1;
 	return 0;
 }
 /*
  * Conf is {icpHost/icpType/proxyPort/icpPort}*:icpOption:proxyPort:icpPort
  */
-icp_getconf(conf,icptypes,icpaddrs,icpports,pxaddrs,pxports,timeout,buff)
-	char *conf;
-	int icptypes[];
-	char *icpaddrs[];
-	int icpports[];
-	char *pxaddrs[];
-	int pxports[];
-	double *timeout;
-	char *buff;
-{	char *addr;
-	char dcnfb[1024],*dcnfv[8];  /* default hosts:opts:pport:iport */
-	char servb[1024],*servv[32]; /* each server */
-	char srv1b[1024],*confv[8];  /* host/type/pport/iport */
+int icp_getconf(PCStr(conf),int icptypes[],const char *icpaddrs[],int icpports[],const char *pxaddrs[],int pxports[],double *timeout,PVStr(buff))
+{	const char *addr;
+	CStr(dcnfb,1024);
+	const char *dcnfv[8];  /**/ /* default hosts:opts:pport:iport */
+	CStr(servb,1024);
+	const char *servv[32]; /**/ /* each server */
+	CStr(srv1b,1024);
+	const char *confv[8];  /**/ /* host/type/pport/iport */
 	int dcc,scc,cc;
-	char *dicport,*dpxport,*dpxopts;
-	char *icphosts,*icport,*pxport,pxtype[32];
+	const char *dicport;
+	const char *dpxport;
+	const char *dpxopts;
+	const char *icphosts;
+	const char *icport;
+	const char *pxport;
+	CStr(pxtype,32);
 	int icptype,icptype1;
-	char *icpaddr,*pxaddr,pxhost[128];
+	const char *icpaddr;
+	const char *pxaddr;
+	CStr(pxhost,128);
 	int pxportn;
-	char *icphost1;
+	const char *icphost1;
 	int ii;
-	char *bp;
-	char *tp;
+	refQStr(bp,buff); /**/
+	const char *tp;
 	FILE *log = stderr;
 
 	strcpy(dcnfb,conf);
@@ -1037,10 +1059,9 @@ icp_getconf(conf,icptypes,icpaddrs,icpports,pxaddrs,pxports,timeout,buff)
 
 	scc = list2vect(servb,',',4,servv);
 
-	bp = buff;
 	for( ii = 0; ii < scc; ii++ ){
 		strcpy(srv1b,servv[ii]);
-		cc = list2vect(srv1b,'/',32,confv);
+		cc = list2vect(srv1b,'/',4,confv);
 		if( 0<cc && *confv[0] )
 			icphost1 = confv[0]; else icphost1 = "localhost";
 		if( 1<cc && *confv[1] )
@@ -1055,7 +1076,7 @@ icp_getconf(conf,icptypes,icpaddrs,icpports,pxaddrs,pxports,timeout,buff)
 
 		if( strchr(pxport,':') ){
 			pxportn = 8080;
-			sscanf(pxport,"%[^:]:%d",pxhost,&pxportn);
+			Xsscanf(pxport,"%[^:]:%d",AVStr(pxhost),&pxportn);
 		}else{
 			pxportn = atoi(pxport);
 			strcpy(pxhost,icphost1);
@@ -1073,7 +1094,7 @@ icp_getconf(conf,icptypes,icpaddrs,icpports,pxaddrs,pxports,timeout,buff)
 		pxports[ii] = pxportn;
 
 		icptypes[ii] = icptype1;
-		optsym(icptype1,pxtype);
+		optsym(icptype1,AVStr(pxtype));
 
 		Lprintf(log,"(ICP)%s/%s/%s:%d/%s\n",
 			icpaddr,pxtype,pxaddr,pxportn,icport);
