@@ -16,8 +16,10 @@ Description:
 History:
 	950205	created
 //////////////////////////////////////////////////////////////////////#*/
-#include <string.h>
+#include "ystring.h"
 #include <stdio.h>
+#include "delegate.h"
+#include "param.h"
 
 char P_ABORTLOG[] = "ABORTLOG";
 char P_ACTDIR[]   = "ACTDIR";
@@ -30,6 +32,7 @@ char P_BASEURL[]  = "BASEURL";
 char P_CACHE[]    = "CACHE";
 char P_CACHEDIR[] = "CACHEDIR";
 char P_CACHEFILE[]= "CACHEFILE";
+char P_CFI[]      = "CFI";
 char P_CGIENV[]   = "CGIENV";
 char P_CHARCODE[] = "CHARCODE";
 char P_CHARSET[]  = "CHARSET";
@@ -37,14 +40,18 @@ char P_CHROOT[]   = "CHROOT";
 char P_CMAP[]     = "CMAP";
 char P_CRON[]     = "CRON";
 char P_CRONS[]    = "CRONS";
+char P_CRYPT[]    = "CRYPT";
 char P_CONNECT[]  = "CONNECT";
 char P_COUNTER[]  = "COUNTER";
+char P_DATAPATH[] = "DATAPATH";
 char P_DBFILE[]   = "DBFILE";
 char P_DGOPTS[]   = "DGOPTS";
 char P_DGPATH[]   = "DGPATH";
 char P_DGROOT[]   = "DGROOT";
+char P_DGSIGN[]   = "DGSIGN";
 char P_DELAY[]    = "DELAY";
 char P_DELEGATE[] = "DELEGATE";
+char P_DEST[]     = "DEST";
 char P_DNSCONF[]  = "DNSCONF";
 char P_ERRORLOG[] = "ERRORLOG";
 char P_ETCDIR[]   = "ETCDIR";
@@ -57,20 +64,21 @@ char P_FFROMCL[]  = "FFROMCL";
 char P_FFROMSV[]  = "FFROMSV";
 char P_FFROMMD[]  = "FFROMMD";
 char P_FILETYPE[] = "FILETYPE";
+char P_FORWARD[]  = "FORWARD";
 char P_FTOCL[]    = "FTOCL";
 char P_FTOMD[]    = "FTOMD";
 char P_FTOSV[]    = "FTOSV";
 char P_FTPCONF[]  = "FTPCONF";
+char P_FTPTUNNEL[] = "FTPTUNNEL";
 char P_FUNC[]     = "FUNC";
 char P_HOSTS[]    = "HOSTS";
+char P_HOSTLIST[] = "HOSTLIST";
 char P_HTMLCONV[] = "HTMLCONV";
-char P_HTTPMAIL[] = "HTTPMAIL";
 char P_HTTPCONF[] = "HTTPCONF";
 char P_ICP[]      = "ICP";
 char P_ICPCONF[]  = "ICPCONF";
 char P_INETD[]    = "INETD";
 char P_INPARAM[]  = "INPARAM";
-char P_FORCEON[]  = "FORCEON";
 char P_LIBDIR[]   = "LIBDIR";
 char P_LIBPATH[]  = "LIBPATH";
 char P_LOG[]      = "LOG";
@@ -84,21 +92,23 @@ char P_MASTERP[]  = "MASTERP";
 char P_MAXIMA[]   = "MAXIMA";
 char P_MIMECONV[] = "MIMECONV";
 char P_MOUNT[]    = "MOUNT";
+char P_MYAUTH[]   = "MYAUTH";
 char P_NNTPCONF[] = "NNTPCONF";
 char P_OWNER[]    = "OWNER";
 char P_OVERRIDE[] = "OVERRIDE";
 char P_PERMIT[]   = "PERMIT";
 char P_REMITTABLE[] = "REMITTABLE";
+char P_PAMCONF[]  = "PAMCONF";
 char P_PGP[]      = "PGP";
 char P_PIDFILE[]  = "PIDFILE";
 char P_PORT[]     = "PORT";
 char P_PROTOLOG[] = "PROTOLOG";
 char P_PROXY[]    = "PROXY";
-char P_PUBLIC[]   = "PUBLIC";
 char P_QPORT[]    = "QPORT";
 char P_RIDENT[]   = "RIDENT";
 char P_RPORT[]    = "RPORT";
 char P_REACHABLE[]= "REACHABLE";
+char P_REJECT[]   = "REJECT";
 char P_RELIABLE[] = "RELIABLE";
 char P_RES_CONF[] = "RES_CONF";
 char P_RES_DEBUG[]= "RES_DEBUG";
@@ -109,15 +119,22 @@ char P_RES_VRFY[] = "RES_VRFY";
 char P_RESOLV[]   = "RESOLV";
 char P_RELAY[]    = "RELAY";
 char P_ROUTE[]    = "ROUTE";
+char P_SERVCONF[] = "SERVCONF";
 char P_SERVER[]   = "SERVER";
+char P_SERVICE[]  = "SERVICE";
 char P_SHARE[]    = "SHARE";
 char P_SMTPCONF[] = "SMTPCONF";
 char P_SMTPGATE[] = "SMTPGATE";
 char P_SMTPSERVER[] = "SMTPSERVER";
+char P_SOCKOPT[]  = "SOCKOPT";
 char P_SOCKS[]    = "SOCKS";
+char P_SOXCONF[]  = "SOXCONF";
+char P_SRCIF[]    = "SRCIF";
 char P_SSLTUNNEL[] = "SSLTUNNEL";
+char P_STLS[]     = "STLS";
 char P_SYSLOG[]   = "SYSLOG";
 char P_INVITE[]   = "INVITE";
+char P_TELNETCONF[] = "TELNETCONF";
 char P_THRUWAY_ENTR[] = "THRUWAY_ENTR";
 char P_THRUWAY_EXIT[] = "THRUWAY_EXIT";
 char P_TMPDIR[]   = "TMPDIR";
@@ -148,12 +165,14 @@ char P_INPUT[]       = "_input";
 #define INT	5	/* internal use */
 
 typedef struct {
-	char	*p_name;
+  const	char	*p_name;
 	int	 p_stat;
-	char	*p_mssg;
+  const	char	*p_mssg;
 	char	 p_mpass;	/* pass to private-MASTER */
 	char	 p_fpass;	/* pass to external filter program */
 	char	 p_spass;	/* pass to spawned self */
+     scanPFUNCP	 p_scanner;
+	int	 p_breaksticky;
 } ParamSpec;
 
 #define M0	NULL
@@ -168,10 +187,11 @@ static ParamSpec params[] = {
 { P_ADMINPASS,	STD,M0,1,1},
 { P_AUTH,	STD,M0,1,1},
 { P_AUTHORIZER,	STD,M0,0,1},
-{ P_BASEURL,	STD,M0,0,1},
+{ P_BASEURL,	STD,M0,0,1, 0,scan_BASEURL},
 { P_CACHE,	EXP,M0,1,1},
 { P_CACHEDIR,	STD,M0,1,1},
 { P_CACHEFILE,	STD,M0,1,1},
+{ P_CFI,	STD,M0,1,1},
 { P_CGIENV,	STD,M0,1,1},
 { P_CHARCODE,	STD,M0,0,1},
 { P_CHARSET,	STD,M0,0,1},
@@ -180,33 +200,38 @@ static ParamSpec params[] = {
 { P_COUNTER,	NOT,M0,1,1},
 { P_CRON,	EXP,M0,0,0},
 { P_CRONS,	EXP,M0,0,0},
+{ P_CRYPT,	EXP,M0,1,1},
+{ P_DATAPATH,	EXP,M0,1,1},
 { P_DBFILE,	EXP,M0,1,1},
 { P_DGOPTS,	STD,M0,1,1},
 { P_DGPATH,	STD,M0,1,1},
 { P_DGROOT,	STD,M0,1,1},
-{ P_DELAY,	STD,M0,0,1},
-{ P_DELEGATE,	STD,M0,0,1},
+{ P_DGSIGN,	STD,M0,1,1},
+{ P_DELAY,	STD,M0,0,1, 0,scan_DELAY},
+{ P_DELEGATE,	STD,M0,0,1, 0,scan_DELEGATE},
+{ P_DEST,	EXP,M0,1,1},
 { P_DNSCONF,	EXP,M0,1,1},
 { P_ERRORLOG,	STD,M0,1,1},
 { P_ETCDIR,	STD,M0,1,1},
 { P_EXPIRE,	STD,M0,1,1},
 { P_EXPIRELOG,	STD,M0,1,1},
-{ P_FCL,	EXP,M0,0,0},
+{ P_FCL,	EXP,M0,0,0, 0,scan_FCL},
 { P_FMD,	EXP,M0,0,0},
-{ P_FSV,	EXP,M0,0,0},
-{ P_FFROMCL,	EXP,M0,0,0},
+{ P_FSV,	EXP,M0,0,0, 0,scan_FSV},
+{ P_FFROMCL,	EXP,M0,0,0, 0,scan_FFROMCL},
 { P_FFROMMD,	EXP,M0,0,0},
-{ P_FFROMSV,	EXP,M0,0,0},
+{ P_FFROMSV,	EXP,M0,0,0, 0,scan_FFROMSV},
 { P_FILETYPE,	EXP,M0,0,0},
-{ P_FTOCL,	EXP,M0,0,0},
+{ P_FORWARD,	EXP,M0,0,0},
+{ P_FTOCL,	EXP,M0,0,0, 0,scan_FTOCL},
 { P_FTOMD,	EXP,M0,0,0},
-{ P_FTOSV,	EXP,M0,0,0},
+{ P_FTOSV,	EXP,M0,0,0, 0,scan_FTOSV},
 { P_FTPCONF,	EXP,M0,1,1},
+{ P_FTPTUNNEL,	EXP,M0,1,1},
 { P_FUNC,	EXP,M0,0,0},
-{ P_FORCEON,	OBS,M0,0,0},
 { P_HOSTS,	STD,M0,1,1},
+{ P_HOSTLIST,	STD,M0,1,1},
 { P_HTMLCONV,	STD,M0,0,1},
-{ P_HTTPMAIL,	STD,M0,0,0},
 { P_HTTPCONF,	STD,M0,0,0},
 { P_ICP,	STD,M0,0,0},
 { P_ICPCONF,	STD,M0,0,0},
@@ -216,30 +241,32 @@ static ParamSpec params[] = {
 { P_LIBPATH,	EXP,M0,1,1},
 { P_LOG,	EXP,M0,1,1},
 { P_LOGDIR,	STD,M0,1,1},
-{ P_LOGFILE,	STD,M0,1,1},
+{ P_LOGFILE,	STD,M0,1,1, 0,scan_LOGFILE},
 { P_LOGCENTER,	STD,M0,1,1},
 { P_MAILSPOOL,	STD,M0,1,1},
 { P_MANAGER,	OBS,M0,1,1},
 { P_CMAP,	STD,M0,0,0},
 { P_MASTER,	STD,M0,0,0},
 { P_MASTERP,	STD,M0,0,0},
-{ P_MAXIMA,	EXP,M0,1,1},
+{ P_MAXIMA,	EXP,M0,1,1, 0,scan_MAXIMA},
 { P_MIMECONV,	STD,M0,0,1},
 { P_MOUNT,	STD,M0,0,0},
+{ P_MYAUTH,	STD,M0,0,0},
 { P_NNTPCONF,	EXP,M0,1,1},
 { P_OWNER,	STD,M0,1,1},
 { P_OVERRIDE,	EXP,M0,0,0},
 { P_PERMIT,	STD,M0,0,0},
+{ P_PAMCONF,	STD,M0,0,0},
 { P_PGP,	STD,M0,0,0},
 { P_PIDFILE,	STD,M0,1,1},
 { P_PORT,	EXP,M0,0,0},
 { P_PROTOLOG,	STD,M0,1,1},
 { P_PROXY,	STD,M0,0,0},
-{ P_PUBLIC,	EXP,M0,0,0},
 { P_QPORT,	EXP,M0,1,1},
-{ P_RIDENT,	EXP,M0,1,1},
+{ P_RIDENT,	EXP,M0,1,1, 0,scan_RIDENT},
 { P_RPORT,	EXP,M0,1,1},
 { P_REACHABLE,	DIS,M1,0,0},
+{ P_REJECT,	DIS,M1,0,0},
 { P_RELIABLE,	DIS,M1,0,0},
 { P_REMITTABLE,	STD,M0,0,0},
 { P_RES_CONF,	STD,M0,1,1},
@@ -251,20 +278,27 @@ static ParamSpec params[] = {
 { P_RESOLV,	STD,M0,1,1},
 { P_RELAY,	DIS,M1,0,0},
 { P_ROUTE,	STD,M0,1,1},
+{ P_SERVCONF,	STD,M0,0,0},
 { P_SERVER,	STD,M0,0,0},
+{ P_SERVICE,	EXP,M0,0,0},
 { P_SHARE,	EXP,M0,1,1},
 { P_SMTPCONF,	STD,M0,1,1},
 { P_SMTPGATE,	STD,M0,1,1},
 { P_SMTPSERVER,	STD,M0,0,0},
+{ P_SOCKOPT,	STD,M0,1,1},
 { P_SOCKS,	STD,M0,1,1},
+{ P_SOXCONF,	STD,M0,1,1},
+{ P_SRCIF,	STD,M0,1,1},
 { P_SSLTUNNEL,	EXP,M0,1,1},
+{ P_STLS,	EXP,M0,1,1},
 { P_SYSLOG,	EXP,M0,1,1},
 { P_INVITE,	EXP,M0,1,1},
+{ P_TELNETCONF,	EXP,M0,0,0},
 { P_THRUWAY_ENTR,EXP,M0,0,0},
 { P_THRUWAY_EXIT,EXP,M0,0,0},
 { P_TMPDIR,	STD,M0,1,1},
 { P_TUNNEL,	STD,M0,1,1},
-{ P_TIMEOUT,	STD,M0,1,1},
+{ P_TIMEOUT,	STD,M0,1,1, 0,scan_TIMEOUT},
 { P_TRACELOG,	EXP,M0,1,1},
 { P_UMASK,	STD,M0,1,1},
 { P_URICONV,	STD,M0,0,1},
@@ -283,23 +317,37 @@ static ParamSpec params[] = {
 { 0 }
 };
 
-script_asis(param)
-	char *param;
+int script_asis(PCStr(param))
 {
 	if( streq(param,P_CRON)  ) return 1;
 	if( streq(param,P_INETD) ) return 1;
 	return 0;
 }
 
-check_param(param,warn)
-	char *param;
-{	char *pn,*pm,*dp;
+scanPFUNCP param_scanner(PCStr(param)/*,int breaksticky*/)
+{	int pi,nlen;
+	const char *name;
+
+	for( pi = 0; name = params[pi].p_name; pi++ ){
+		nlen = strlen(name);
+		if( strncmp(param,name,nlen) == 0 )
+		if( param[nlen] == '=' ){
+			return params[pi].p_scanner;
+		}
+	}
+	return 0;
+}
+
+int check_param(PCStr(param),int warn)
+{	const char *pn;
+	const char *pm;
+	const char *dp;
 	int pi;
-	char *name,nameb[128];
+	const char *name;
+	CStr(nameb,128);
 
 	if( dp = strchr(param,'=') ){
-		strncpy(nameb,param,dp-param);
-		nameb[dp-param] = 0;
+		strncpy(nameb,param,dp-param); setVStrEnd(nameb,dp-param);
 		name = nameb;
 	}else	name = param;
 
@@ -318,16 +366,15 @@ check_param(param,warn)
 	return -1;
 }
 
-copy_param(param,dav,sav)
-	char *param;
-	char *dav[],*sav[];
+const char *skip_argcond(PCStr(name));
+int copy_param(PCStr(param),int mac,const char **dav,const char*const*sav)
 {	int dai,sai,pi;
 	int valid_all;
 	int option_also;
 	int script_also;
 	int for_filters;
 	int len;
-	char *name;
+	const char *name;
 
 	valid_all = 0;
 	option_also = 0;
@@ -349,12 +396,23 @@ copy_param(param,dav,sav)
 	if( param != NULL ){
 	    len = strlen(param);
 	    for( sai = 0; name = sav[sai]; sai++ ){
+		name = skip_argcond(name);
+		if( name == NULL )
+			continue;
 		if( name[len] == '=' )
 		if( strncmp(name,param,len) == 0 )
 		    dav[dai++] = sav[sai];
 	    }
 	}else{
 	    for( sai = 0; name = sav[sai]; sai++ ){
+		if( mac-1 <= dai ){
+			break;
+		}
+		name = skip_argcond(name);
+		if( name == NULL )
+			continue;
+		if( !option_also && strncmp(name,"DGOPTS=",7) == 0 )
+			continue;
 		if( option_also && name[0] == '-' )
 			dav[dai++] = sav[sai];
 		else
